Package: subselect
Version: 0.16.1
Date: 2026-05-04
Title: Selecting Variable Subsets
Authors@R: c(person("Jorge" ,"Orestes Cerdeira", role="aut", email="jo.cerdeira@fct.unl.pt"), 
             person("Pedro", "Duarte Silva", role=c("aut","cre"), email="psilva@ucp.pt"), 
             person("Jorge", "Cadima", role="aut",  email="jcadima@isa.ulisboa.pt"), 
             person("Manuel",  "Minhoto", role="aut", email="minhoto@uevora.pt"))
Author: Jorge Orestes Cerdeira [aut],
  Pedro Duarte Silva [aut, cre],
  Jorge Cadima [aut],
  Manuel Minhoto [aut]
Maintainer: Pedro Duarte Silva <psilva@ucp.pt>
Description: A collection of functions which (i) assess the quality of variable subsets as surrogates for a full data set, in either an exploratory data analysis or in the context of a multivariate linear model, and (ii) search for subsets which are optimal under various criteria. Theoretical support for the heuristic search methods and exploratory data analysis criteria is in Cadima, Cerdeira, Minhoto (2003, <doi:10.1016/j.csda.2003.11.001>). Theoretical support for the leap and bounds algorithm and the criteria for the general multivariate linear model is in Duarte Silva (2001, <doi:10.1006/jmva.2000.1920>). There is a package vignette "subselect", which includes additional references.
Depends: R (>= 3.5.0)
Imports: MASS, ISwR, corpcor
License: GPL (>= 2)
Repository: CRAN
LazyData: TRUE
NeedsCompilation: yes
Packaged: 2026-05-04 10:15:17 UTC; antonio
Date/Publication: 2026-05-04 10:50:03 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-05-04 12:25:52 UTC; unix
Archs: subselect.so.dSYM

useDynLib(subselect)
exportPattern(".*.coef")
exportPattern(".*Hmat")
importFrom("stats", "coef", "cor", "cov", "model.extract", "model.matrix", "var", "vcov")
import(ISwR,MASS,corpcor)
export(anneal,genetic,improve,eleaps,leaps,trim.matrix) 
S3method(lmHmat,data.frame)
S3method(lmHmat,default)
S3method(lmHmat,formula)
S3method(lmHmat,lm)
S3method(glhHmat,data.frame)
S3method(glhHmat,default)
S3method(glhHmat,formula)
S3method(glmHmat,glm)
S3method(ldaHmat,data.frame )
S3method(ldaHmat,default)
S3method(ldaHmat,formula)

validgenetic<-function(kmin, kmax, popsize, mutprob, exclude,
nexclude, include, ninclude, initialpop){



################################################################
# validation of input that is specific to the genetic function #
################################################################
# WARNING: Some initializations look similar to those of the   #
# anneal and improvement functions, but here input MUST be a   #
# 3-dimensional array.                                         #
################################################################
        
######################################
# validation of mutation probability #
######################################

        if ((mutprob < 0) | (mutprob > 1)) stop("\n The mutation probability parameter (mutprob) must be between 0 and 1 \n")


################################################################
# initializations when initial population has been specified;  #
# checking the nature of initialpop (and how to interpret it)  #
# and for conflicts with the exclude and include requirements  #
# (which initial the population must respect)                  #
################################################################

        if (is.null(initialpop)) {pilog <- FALSE}
        else {pilog <- TRUE  # initial population has been specified by user          
################################################
# initial solution is a vector: not acceptable #
################################################

            if (is.vector(initialpop)) {
                     {stop("\n The specified initial population must have different k-subsets \n")}
                    }
################################################
# checking for the presence of variables that  #
# are to be forcefully excluded                #
################################################

            if ((nexclude != 0) & (sum(exclude == rep(initialpop,rep(length(exclude),length(as.vector(initialpop))))) !=0)) stop("\n the specified initial population contains variables that are to be excluded \n")

#########################################################
# how to deal with various formats of input (of initial #
#  population) for a single cardinality                 #
#########################################################

            dimpop<-dim(initialpop)
            if (length(dimpop) > 3) stop("\n Can't handle arrays of more than 3 dimensions \n")
            if (kmin == kmax) {

##############################
# initial solution is array? #
##############################

              if (is.array(initialpop)) {

###########################
# initialpop is 3-d array #
###########################

              if (length(dimpop) == 3) {
                 if (dimpop[[3]] > 1) stop("\n The input array of initial population must have dimensions as \n popsize x k x 1 when a single cardinality is requested \n")
                 else initialpop<-matrix(nrow=dimpop[[1]],ncol=dimpop[[2]],initialpop)
              }
              else # initialpop is matrix: must be of form popsize x k
 
                 {if (dimpop[[2]] != kmax) stop("\n Input matrix of initial solutions must have as many columns as variables in the requested subset \n")
                  else 
                      {if  (dimpop[[1]] != popsize) {
                        stop("\n The number of initial solutions can only be 1 or popsize (number of final solutions requested) \n")}}}
              }}

##########################################################
# how to deal with various formats of input (initialpop) #
# if more than one cardinality is requested              #
##########################################################

             else  #(if kmax > kmin), i.e., more than one cardinality requested

#########################################
# initial population must be 3-d array  #
#########################################


              {if (length(dimpop) < 3) stop("\n There must be initial populations for all cardinalities requested \n")
               else 
               {if ((dimpop[[3]] != length(kmin:kmax)) | (dimpop[[2]] != kmax) | (dimpop[[1]] != popsize)) stop("\n The input array of initial solutions must have dimensions as \n popsize x kmax x no. of different cardinalities requested \n")
                  }
                 if ((ninclude != 0) & (sum(include == rep(initialpop,rep(length(include),length(as.vector(initialpop))))) != popsize*length(kmin:kmax)*length(include))) stop("\n Not all the specified initial solutions contain the variables that are to be included \n") 
}}

##################################
#  assigning any changed values  #
##################################

         assign("pilog",pilog,pos=parent.frame())          
         assign("initialpop",initialpop,pos=parent.frame())

}

validnovcrit<-function(mat,criterion,H,r,p,tolval,tolsym){



################################################################
# validation of input that is specific to the criteria "TAU_2",#
#    "XI_2","ZETA_2", "CCR1_2" and "WALD"                      #    
################################################################

	if (!is.numeric(r)) stop("\n Argument r must be numeric.\n Perhaps unnamed arguments in the wrong order?\n")	
             
       	if (r<=0) stop("\n Argument r must be a positive integer (expected rank of effects matrix H)\n")
       	if (r!=floor(r)) {
                   warning("\n The specified expected rank 'r' of the effects matrix must be an integer. \n The specified value of the parameter 'r' has been truncated.\n")
                   r <- floor(r)
        }
     
	if (!is.matrix(H)) stop("\nEffect description data (H) is missing or is not given in matrix form \n")
	if (nrow(H) != p || ncol(H) != p) stop("\nEffect description matrix (H) does not have correct dimensions \n")
                
# checking symmetry of the H effects matrix
                
	maxabssym <- max(abs(H-t(H)))
	if (maxabssym > tolsym)
              {stop("\n The effect description matrix (H) supplied is not symmetric.\n Symmetric entries differ by up to ",maxabssym,".")}
      	else if (maxabssym > .Machine$double.eps) {
              H <-(H+t(H))/2
              warning("\n The effect description matrix (H) supplied was slightly asymmetric: \n symmetric entries differed by up to ",maxabssym,".\n (less than the 'tolsym' parameter).\n The H matrix has been replaced by its symmetric part.\n")
	}

# checking whether H matrix has the specified rank
                
	if (qr(H)$rank != r) {
                  if (criterion == "CCR1_2") stop("\nEffect description matrix (H) does not have the specified rank\n")
                  else warning("\n The expected rank of the effect matrix supplied by the user is ",r," but \n the matrix rank (as computed by R's qr function) is ",qr(H)$rank,".\n")
	}

# checking whether the E matrix is positive definite (not applicable to the Wald criterion)

                if (criterion != "WALD") {
			Eeigval = eigen(mat-H,only.values=T)$value

			if (Eeigval[p]/Eeigval[1] < -tolval) stop("\nError matrix (E=T-H) is not positive definite. Check whether the efects (H) matrix has been correctly specified.\n") 
		}

}

trim.matrix <- function(mat,tolval=10*.Machine$double.eps)
{
  p <- dim(mat)[2]
  matindices <- 1:p                  
  mat.eig <- eigen(mat,symmetric=TRUE)
  discard <- rep(FALSE,p)
  newmat <- mat
  newmatindices <- matindices        
  while(mat.eig$values[p]/mat.eig$values[1] < tolval)
  {
    int <- as.numeric(newmatindices[order(abs(mat.eig$vectors[,p]),decreasing=TRUE)[1]])   
    discard[int] <- TRUE
    newmat <- mat[!discard,!discard]
    newmatindices <- matindices[!discard]   
    p <- p-1
    mat.eig <- eigen(newmat,symmetric=TRUE)   
  }
  size <- dim(newmat)[2]
  output <- list(newmat,as.numeric(matindices[discard]),colnames(mat)[discard],size)   
  names(output) <- c("trimmedmat","numbers.discarded","names.discarded","size")
  output
}

gcd.coef<-function(mat, indices, pcindices = NULL)
{
#   
#       calcula o GCD entre um subconjunto ("indices") de variaveis
#       e um subconjunto ("pcindices") das CPs de todas as variaveis, 
#       cuja matriz de covariancias e "mat".

#  error checking

         if  (sum(!(as.integer(indices) == indices)) > 0) stop("\n The variable indices must be integers")
         if  (!is.null(pcindices) & (sum(!(as.integer(pcindices) == pcindices)) > 0)) stop("\n The PC indices must be integers")

         if (!is.matrix(mat)) {
         stop("Data is missing or is not given in matrix form")}
     if (dim(mat)[1] != dim(mat)[2]) {
         mat<-cor(mat)
         warning("Data must be given as a covariance or correlation matrix. \n It has been assumed that you wanted the correlation matrix of the \n data matrix which was supplied.")
     }        

# body of function

# initializations

      if (!is.null(pcindices)) {
       if (!is.vector(pcindices)) stop("If Principal Components are user-specified, only one set of PCs is allowed for each function call")
      }
  
     dvsmat <- svd(mat)
      tr<-function(mat){sum(diag(mat))}
      gcd.1d<-function(mat,indices, pcindices){
             if (is.null(pcindices)) {pcindices <- 1:sum(!indices==0)}
             indices<-indices[!indices == 0]
             svdapprox <- function(mat, indices) {
             t(dvsmat$v[, indices] %*% (t(dvsmat$u[, indices]) * dvsmat$d[indices]))
                           }
             sum(diag(solve(mat[indices, indices]) %*% svdapprox(mat, 
             pcindices)[indices, indices]))/sqrt(length(indices) * 
             length(pcindices))
        }
      dimension<-length(dim(indices))

# output for each dimension of input array

      if (dimension > 1){
         gcd.2d<-function(mat,subsets,pcindices){
             apply(subsets,1,function(indices){gcd.1d(mat,indices,pcindices)})
            }  
           if (dimension > 2) {               
            gcd.3d<-function(mat,array3d,pcindices){
             apply(array3d,3,function(subsets){gcd.2d(mat,subsets,pcindices)})
            }
            output<-gcd.3d(mat,indices,pcindices)
           }
           if (dimension == 2) {output<-gcd.2d(mat,indices,pcindices)}
      }

      if (dimension < 2) {output<-gcd.1d(mat,indices,pcindices)}
      output
}

tau2.coef<-function(mat,H,r,indices,tolval=10*.Machine$double.eps,tolsym=1000*.Machine$double.eps)
{
#   Function tau_2.coef
#   Computes the tau^2 index of "effect magnitude".  
#   This criterion to the minimization of Wilks lambda statistic.
#   Expected input: a variance-covariance (or correlation) matrix, 
#   Effect descrption matrix (H) and its rank (r)

#  error checking

#
#  mat and indices
#

  if  (sum(!(as.integer(indices) == indices)) > 0) stop("\n The variable indices must be integers")

  p <- dim(mat)[2]
  validmat(mat,p,tolval,tolsym,allowsingular=TRUE,algorithm="none")

#
# checks on r and H
#

  validnovcrit(mat,criterion="TAU_2",H,r,p,tolval,tolsym)

#
# Computing the criterion value
#

	 tau2.1d<-function(mat,H,r,indices){
		l <- min(r,length(indices))
	    if (length(indices)==1) { detE <- mat[indices,indices]-H[indices,indices]
						detmat <- mat[indices,indices]  }
          else
              { detE <- det(mat[indices,indices]-H[indices,indices])
						detmat <- det(mat[indices,indices])  }

	1 - (detE/detmat)^(1/l)	
       }
      dimension<-length(dim(indices))
      if (dimension > 1){
         tau2.2d<-function(mat,H,r,subsets){
             apply(subsets,1,function(indices){
			
			tau2.1d(mat,H,r,indices)})
            }  
             if (dimension > 2) {
               tau2.3d<-function(mat,H,r,array3d){
                   apply(array3d,3,function(subsets){tau2.2d(mat,H,r,subsets)})
                 }
               output<-tau2.3d(mat,H=H,r,indices)
              }
              if (dimension == 2) {output<-tau2.2d(mat=mat,H,r,indices)}
      }

      if (dimension < 2) {output<-tau2.1d(mat,H,r,indices)}
      output
}

*#######################################################################
       SUBROUTINE improve(criterio,p,svector,kmin,
     +   kmax,valores,vars,bestval,bestvar,nfora,fora,ndentro,
     +   dentro,nsol,nqsi,qsi,esp,silog,solinit, valp,
     +   vecvecp,hvector,rh,checksg,numtol,numprb)
*#######################################################################

* Restricted improvement algorithm to search for a k-variable subset
* of a set of p original variables, which maximizes one of several criteria.
* (The criteria currently considered are RM, RV, GCD, TAU_2,XI_2,ZETA_2 and 
*  CCR1_2).
* The user may force the inclusion and/or exclusion of certain 
* variables from the k-subset.
* 
* Designed to be called by an R/S function "improve".
*
* WARNING: Uses R's default Random Number Generator.
*
* INPUT : 
*
*criterio  - integer variable indicating which criterion of subset quality
*            was requested.
*   p      - integer variable indicating the number of original variables.
* svector  - double precision vector, giving the full covariance (or 
*            correlation) matrix of the p variables (given as a vector
*            for convenience in passing from R to Fortran). 
*kmin,kmax - integer variables, giving the smallest and largest cardinalities
*           of the subsets of variables that are wanted.
*  nfora   - integer variable indicating the number of original variables 
*            that are to be forcefully excluded from the k-subset.
*   fora   - integer vector, indicating the variable numbers associated
*            with the nfora variables that are to be excluded by force.
* ndentro  - integer variable indicating the number of original variables 
*            that are to be forcefully included in the k-subset.
*  dentro  - integer vector, indicating the variable numbers associated
*            with the ndentro variables that are to be included by force.
*  nsol    - integer indicating how many different solutions are to be 
*            produced by the simulated annealing search.
*  nqsi    - integer variable, indicating the dimension of the Principal 
*           Subspace with which the subspace spanned by the k-subset is
*            to be compared. (For GCD criterion only).
*  qsi     - integer vector, giving the ranks of the Principal Components 
*           which span the Principal Subspace described above (see nqsi).
*           (For GCD criterion only).
*  esp     - logical vector. If .true., indicates that the user has specified
*            a set of Principal Components different from the first k, to 
*            compare with the k-variable subset. (For GCD criterion only).
* silog    - logical variable indicating whether or not the user has specified
*             initial solutions.
* solinit  - integer variable, giving the user-specified initial solutions
*             (if any).
*   valp   - a double precision vector with the eigenvalues of the covariance 
*            (correlation) matrix, computed in R.
* vecvecp  - double precision vector, giving the eigenvectors of the 
*            covariance (or correlation) matrix of the p variables (given 
*            as a vector for convenience in passing from R to Fortran). 
* hvector  - double precision vector, giving the full effect descrption 
*            matrix (H) of the p variables (given as a vector for
*            convenience in passing from R to Fortran). 
*   rh     - integer variable giving the expected rank of H matrix.
* checksg - logical, flag indicating if a test for ill-conditioned 
*	    (E being numerically singular) problems should be implemented
*  numtol - double precision tolerance level for the ratio of the diagonal elements of E Choleski decomposition

* OUTPUT: 

* valores  - double precision variable giving the final values produced by the 
*            restricted local search Algorithm for each of the nsol solutions, 
*            in each cardinality, for the criterion required. Will pass to the
*            R/S function "improve" for conversion to an nsol x (kmax-kmin+1) 
*            matrix within R/S, with each column associated with a cardinality
*            and each row with a different solution. 
*  vars    - integer vector giving the list of variable numbers belonging to 
*            each of the nsol final subsets, for each cardinality (padded 
*            with zeros if k<kmax). Will pass to the R/S function "improve" 
*            for conversion to an nsol x kmax x (kmax-kmin+1) 3-d array.
* bestval - double precision vector with the best value of the criterion
*           obtained by running the routine, for each cardinality (R/S output).
* bestvar - integer vector with the variable numbers of the best subset 
*           obtained, for each cardinality (ouput for R/S).
* numprb -  logical flag set to true if numerical problems precluded some
*           (ill-conditioned) subsets to be evaluated, and to false otherwise.   
*
*

* general declarations
       INTEGER p,criterio,poriginal,rh
       INTEGER fora(0:nfora),fica(0:p),dentro(0:ndentro),auxw(kmax)
       INTEGER vars(nsol*(kmax-kmin+1)*kmax)
       INTEGER bestvar((kmax-kmin+1)*kmax)
       LOGICAL setk(p),setkmax(p)
       DOUBLE PRECISION s(p,p),sq(p,p),svector(p*p),hvector(p*p),h(p,p)
       DOUBLE PRECISION vmax,vactual,numtol
       DOUBLE PRECISION valores((kmax-kmin+1)*nsol)
       DOUBLE PRECISION critvalue,bestval(kmax-kmin+1)
       DOUBLE PRECISION vecvecp(p*p)
       LOGICAL silog,checksg,numprb
       INTEGER solinit(kmax*nsol*(kmax-kmin+1))
* FIM NOVO TESTE
* declarations only for the RM criterion
       DOUBLE PRECISION tracos,dobjrm
* declarations only for the RV criterion
       DOUBLE PRECISION tracosq,dobjrv
* declarations only for the GCD criterion
       DOUBLE PRECISION valp(p),vecp(p,p),dobjgcd
       INTEGER nqsi,qsi(p)
       LOGICAL esp
* declarations for tau2,xi2,zeta2 e ccr12
       DOUBLE PRECISION dobjtau2,dobjxi2,dobjzeta2,dobjccr12
* declarations of local arrays and matrices
       INTEGER randsk1pp(p),setint(p)
       DOUBLE PRECISION skinput(kmax,kmax),hkinput(kmax,kmax)
       DOUBLE PRECISION ekinput(kmax,kmax),workmat(kmax,kmax)
       DOUBLE PRECISION work(6*kmax),egval(kmax)

       external dmelhoramentogen,newinicializar,randsk1,dcorrigesk
       external dobjrm,dobjrv,dobjgcd
       external dobjtau2,dobjxi2,dobjzeta2,dobjccr12

* initializations 
       critvalue = -1.0D0
       numprb=.false.

       call newinicializar(criterio,p,s,svector,sq,nfora,fora,
     +  ndentro, dentro,fica,tracos,tracosq,vecp,poriginal,vecvecp,
     +  h,hvector,rh)

**********************************
* The loop which is to be repeated for each cardinality of subsets, 
* between kmin e kmax starts here and ends together with the subroutine.

      do k=kmin,kmax

* Generates nsol solutions at random, and runs the Restricted Local
* Improvement algorithm on them.

         if (criterio.eq.3) then
             if (.not.esp) then
               nqsi=k
               do m=1,nqsi
                 qsi(m)=m
               end do
             end if
         end if
         vmax=-1
         do ksol=1,nsol

* setting up the inital solutions

           if (silog) then
             do m=1,poriginal
               setk(m)=.false.
             end do           
             do m=1,k
               setk(solinit(nsol*kmax*(k-kmin)+nsol*(m-1)+ksol))=.true.
             end do
             if (nfora.ge.1) then
               do m=1,p
                 setk(m) = setk(fica(m))
               end do
             end if  
           else
               call randsk1(p-ndentro,k-ndentro,setk,randsk1pp)           
               if(ndentro .GT. 0) then 
                  call dcorrigesk(ndentro,dentro,p,setk,poriginal)
               endif
           endif

           if (criterio.eq.1) then
		vactual=dobjrm(k,setk,p,poriginal,kmax,s,sq,workmat,
     +   setint,skinput)
           end if
           if (criterio.eq.2) then
               vactual=dobjrv(k,setk,p,poriginal,kmax,s,sq,workmat,
     +   setint,skinput)
           end if
           if (criterio.eq.3) then
		vactual=dobjgcd(k,setk,p,poriginal,kmax,s,workmat,
     +   nqsi,qsi,valp,setint,vecp,fica,skinput)
           end if
           if (criterio.eq.4) then
               vactual=dobjtau2(k,setk,p,poriginal,kmax,s,h,rh,checksg,
     +   numtol,setint,work,egval,skinput,hkinput,ekinput,workmat)
           end if
	   if (criterio.eq.5) then
               vactual=dobjxi2(k,setk,p,poriginal,kmax,s,h,rh,checksg,
     +   numtol,setint,work,egval,skinput,hkinput,workmat)
           end if
	   if (criterio.eq.6) then
               vactual=dobjzeta2(k,setk,p,poriginal,kmax,s,h,rh,checksg,
     +   numtol,setint,work,egval,skinput,hkinput,ekinput,workmat)
           end if		 
	  if (criterio.eq.7) then
* tirei rh
               vactual=dobjccr12(k,setk,p,poriginal,kmax,s,h,checksg,
     +   numtol,setint,work,egval,skinput,hkinput,workmat)
           end if

           if (checksg.and.criterio.GT.3.and.vactual.eq.-0.9999D0) then
	       numprb=.true.        
           end if
	   if (vactual.GE.0) then
		call dmelhoramentogen(criterio,p,kmax,setk,vactual,
     +         ndentro,dentro,k,s,sq,nqsi,qsi,valp,vecp,fica,
     +         poriginal,h,rh,skinput,hkinput,ekinput,egval,
     +         work,workmat,randsk1pp,setint,checksg,numtol,numprb)
	   end if

           if (criterio.eq.1) then
              critvalue = dsqrt(vactual/tracos)
           end if
           if (criterio.eq.2) then
              critvalue = dsqrt(vactual/tracosq)
           end if
           if (criterio.eq.3) then
              critvalue = vactual/dsqrt(DBLE(nqsi*k))
           end if
           if (criterio.eq.4) then
              critvalue = vactual
           end if

	     if (criterio.eq.5) then
               critvalue = vactual
           end if
		  if (criterio.eq.6) then
              critvalue = vactual
           end if

	     if (criterio.eq.7) then
               critvalue = vactual
           end if

* preparing the output for R/S

            valores((k-kmin)*nsol+ksol)=critvalue
            jjaux=0
           do i=1,p
             if (setk(i)) then
               jjaux=jjaux+1
		if (critvalue.LT.0.0D0) then 
                   vars(nsol*kmax*(k-kmin)+nsol*(jjaux-1)+ksol) = 0
		else
                   vars(nsol*kmax*(k-kmin)+nsol*(jjaux-1)+ksol) = 
     +           fica(i)
                endif
             end if
           end do  

           if(vactual .GT. vmax) then
              vmax=vactual
              do j=1,p
                 setkmax(j)=setk(j)
              end do
           end if
         end do

         naux=0
         do j=1,p
           if(setkmax(j)) then
              naux=naux+1
              auxw(naux)= fica(j)
           end if
         end do
         if (criterio.eq.1) then
            critvalue=dsqrt(vmax/tracos)
         end if
         if (criterio.eq.2) then
            critvalue=dsqrt(vmax/tracosq)
         end if
         if (criterio.eq.3) then
              critvalue = vmax/dsqrt(DBLE(nqsi*k))
         end if
		if (criterio.eq.4) then
            critvalue=vmax
         end if

         if (criterio.eq.5) then
             critvalue=vmax  
           end if
		 
          if (criterio.eq.6) then
            critvalue=vmax
         end if

         if (criterio.eq.7) then
             critvalue=vmax  
           end if

        bestval(k-kmin+1)=critvalue
        do j=1,k
            bestvar(kmax*(k-kmin)+j) = auxw(j)
        end do
        end do  

* End of the k-loop

      return
      end

initialization <- function(mat,criterion,r){
 
#####################################
#  set parameters to default values #
#####################################

  
	if (!is.null(attr(mat,"FisherI")) && attr(mat,"FisherI")==TRUE)  {
		if (r!=1) stop("In generalized linear model problems the argument r should always be set to 1.\n")
		if (criterion=="default")  criterion <- "WALD"
		else if (criterion!="WALD" && criterion!="Wald" && criterion!="wald") 
			stop("In generalized linear model problems only the Wald criterion is currently available.\n")
	}
	else {
		if (r==0 && criterion=="default")  criterion <- "RM"
        	if (r>0  && criterion=="default")  criterion <- "TAU_2"
		if (criterion=="WALD" || criterion=="Wald" || criterion=="wald") 
			warning("Since the Wald criterion was chosen, it was assumed that the mat matrix represented\n the Fisher information resulting from the estimation of a generalized linear model.\n")
	}

        assign("criterion",criterion,pos=parent.frame())

}

                
*#######################################################################
       SUBROUTINE genetic(criterio,p,svector,kmin,kmax,valores,vars,
     +   bestval,bestvar,nfora,fora,ndentro,dentro,npopinic,nger,nclone,
     +   melhorar,mutprob,improvement,nqsi,qsi,esp,kabort,pilog,popinit,
     +   valp, vecvecp,hvector,rh,checksg,numtol,numprb)
*#######################################################################

* Routine to perform a Genetic Algorithm search for a k-variable subset
* of a set of p original variables, which maximizes one of several criteria.
* (The criteria currently considered are RM, RV, GCD, TAU_2,XI_2,ZETA_2 and 
*  CCR1_2). The user may force  the inclusion and/or exclusion of certain 
*  variables from the k-subset.
* 
* Designed to be called by an R/S function "genetic" (included in the package 
* "subselect", available from CRAN).
*
* WARNING: Requires the LAPACK routine DPOSV.
* WARNING: Uses R's default Random Number Generator.
*
* INPUT : 
*
*criterio  - integer variable indicating which criterion of subset quality
*            was requested.
*   p      - integer variable indicating the number of original variables.
* svector  - double precision vector, giving the full covariance (or 
*            correlation) matrix of the p variables (given as a vector
*            for convenience in passing from R to Fortran). 
*kmin,kmax - integer variables, giving the smallest and largest cardinalities
*           of the subsets of variables that are wanted.
*  nfora   - integer variable indicating the number of original variables 
*            that are to be forcefully excluded from the k-subset.
*   fora   - integer vector, indicating the variable numbers associated
*            with the nfora variables that are to be excluded by force.
* ndentro  - integer variable indicating the number of original variables 
*            that are to be forcefully included in the k-subset.
*  dentro  - integer vector, indicating the variable numbers associated
*            with the ndentro variables that are to be included by force.
* npopinic - integer indicating the size of the initial population of 
*            k-subsets.
*  nger   - integer variable, number of generations for the genetic algorithm. 
* nclone  - integer variable indicating the maximum number of clones (identical
*           individuals) that will be tolerated. Clones in excess of nclone 
*           are excluded.
* melhorar - logical variable indicating whether each new child is to be 
*            mutated with probability mutprob, via a restricted local 
*            improvement algorithm, before undergoing survival process.
* mutprob  - user-specified probability for each child to undergo mutation
*            (in the form of submission to local improvement algorithm).
*improvement- logical variable. If true, the best solution produced by 
*            the Genetic Algorithm will be subjected to a restricted local
*            improvement algorithm. (By default .true.).
*  nqsi    - integer variable, indicating the dimension of the Principal 
*            Subspace with which the subspace spanned by the k-subset is
*            to be compared. (For GCD criterion only).
*  qsi     - integer vector, giving the ranks of the Principal Components 
*            which span the Principal Subspace described above (see nqsi).
*            (For GCD criterion only).
*  esp     - logical vector. If .true., indicates that the user has specified
*            a set of Principal Components different from the first k, to 
*            compare with the k-variable subset. (For GCD criterion only).
* pilog    - logical variable indicating whether or not the user has specified
*            an initial population.
* popinit  - integer variable, giving the user-specified initial population
*            (if any).
*   valp   - a double precision vector with the eigenvalues of the covariance 
*            (correlation) matrix, computed in R.
* vecvecp  - double precision vector, giving the eigenvectors of the 
*            covariance (or correlation) matrix of the p variables (given 
*            as a vector for convenience in passing from R to Fortran). 
* hvector  - double precision vector, giving the full effect descrption 
*            matrix (H) of the p variables (given as a vector for
*            convenience in passing from R to Fortran). 
*   rh     - integer variable giving the expected rank of H matrix.
* checksg - logical, flag indicating if a test for ill-conditioned 
*	    (E being numerically singular) problems should be implemented
*  numtol - double precision tolerance level for the ratio of the diagonal elements of E Choleski decomposition
*
* OUTPUT: 
*
* valores  - double precision variable giving the final values produced by 
*            the Genetic Algorithm for each of the npopinic members of the 
*            population, in each cardinality, for the criterion required. 
*            Will pass to the R/S function "genetic" for conversion to an 
*            npopinic x (kmax-kmin+1) matrix within R/S, with each column 
*            associated with a  cardinality and each row with a different 
*            solution. 
*  vars    - integer vector giving the list of variable numbers  belonging 
*            to  each of the npopinic final subsets, for each cardinality 
*            (padded with zeros if k<kmax). Will pass to the R/S function 
*            "genetic"  for conversion to an npopinic x kmax x (kmax-kmin+1) 
*            3-dimensional array.
* bestval -  double precision vector with the best value of the criterion
*            obtained by running the routine, for each cardinality.  If 
*            improvement=.true. (the default), then the value given is the 
*            value resulting from running the restricted local search
*            algorithm on the best solution from the Genetic Algorithm 
*            (output for R/S).
* bestvar - integer vector with the variable numbers of the best subset 
*           obtained, for each cardinality.  If improvement=.true. (the 
*           default), then the subset given results from running the 
*           restricted local search algorithm on the best solution from the 
*           Genetic Algorithm (output for R/S).
* kabort  - integer variable indicating the cardinality in which an execution 
*           of the algorithm may have been aborted for lack of genetic 
*           variability (ouput for R/S).
* numprb -  logical flag set to true if numerical problems precluded some
*           (ill-conditioned) subsets to be evaluated, and to false otherwise.   
*
*

* general declarations
       INTEGER p,kmin,kmax,criterio,npopinic,nger,nclone,poriginal
       INTEGER fora(0:nfora),fica(0:p),dentro(0:ndentro),auxw(kmax)
       INTEGER talvez,painaomae(kmax),maenaopai(kmax),rh
       INTEGER ordemger(0:npopinic)
       INTEGER garanhao(npopinic),femea(npopinic),randint
       INTEGER vars(npopinic*(kmax-kmin+1)*kmax)
       INTEGER bestvar((kmax-kmin+1)*kmax),kabort
       INTEGER popinit(kmax*npopinic*(kmax-kmin+1))
       LOGICAL setk(p),improvement 
       LOGICAL pop(npopinic,p),novager(npopinic,p),rep(p+1)
       LOGICAL filho(npopinic,p),igual,melhorar
       LOGICAL pilog,checksg,numprb
       DOUBLE PRECISION s(p,p),sq(p,p),svector(p*p), vecvecp(p*p),h(p,p)
       DOUBLE PRECISION vactual,vcorrente,numtol
       DOUBLE PRECISION valores((kmax-kmin+1)*npopinic),hvector(p*p)
       DOUBLE PRECISION valuepop(npopinic+1),valueger(0:npopinic)
       DOUBLE PRECISION critvalue,amax,aux,somavalue,acumulado,vantes
       DOUBLE PRECISION vmaximo,valuenovager(npopinic)
       DOUBLE PRECISION bestval(kmax-kmin+1)
       DOUBLE PRECISION testran, mutprob,r,ranaux,unifrnd
* declarations only for the RM criterion
       DOUBLE PRECISION tracos,dobjrm
* declarations only for the RV criterion
       DOUBLE PRECISION tracosq,dobjrv
* declarations only for the GCD criterion
       DOUBLE PRECISION valp(p), vecp(p,p),dobjgcd
       INTEGER nqsi,qsi(p)
       LOGICAL esp
* declarations for tau2,xi2,zeta2 e ccr12
       Double precision dobjtau2,dobjxi2,dobjzeta2,dobjccr12
* declarations of local arrays and matrices
       INTEGER randsk1pp(p),setint(p)
       DOUBLE PRECISION skinput(kmax,kmax),hkinput(kmax,kmax)
       DOUBLE PRECISION ekinput(kmax,kmax),workmat(kmax,kmax)
       DOUBLE PRECISION work(6*kmax),egval(kmax)

       external dobjrm,dobjrv,dobjgcd
       external dobjtau2,dobjxi2,dobjzeta2,dobjccr12
       external dcorrigesk,dmelhoramentogen,newinicializar
       external intpr,dblepr
       external randsk1,rndstart,rndend,unifrnd

* initializations
       critvalue = -1.0D0
       numprb=.false.

       call rndstart()       
       call newinicializar(criterio,p,s,svector,sq,nfora,fora,ndentro,
     +  dentro,fica,tracos,tracosq,vecp,poriginal,vecvecp,
     +  h,hvector,rh)

      kabort = kmax+1


**********************************
* The loop which is to be repeated for each cardinality of subsets, 
* between kmin e kmax starts here and ends together with the subroutine.
         
      valueger(0)=DBLE(2)
      ordemger(0)=0
      npop=npopinic
      do k=kmin,kmax
         if (criterio.eq.3) then
             if (.not.esp) then
               nqsi=k
               do m=1,nqsi
                 qsi(m)=m
               end do
             end if
         end if


* Generating the initial population
* Remark: vector rep tests whether all elements of {1,2,...,p} are   
* represented in the initial population, in which case rep(i)=.true. 
* for i=1,...,p

         do i=1,p
            rep(i)=.false.
         end do

         do kpop=1,npopinic

* setting up the initial population

           if (pilog) then
             do m=1,poriginal
               setk(m)=.false.
             end do           
             do m=1,k
               setk(popinit(npopinic*kmax*(k-kmin)+npopinic*(m-1)+kpop))
     +  =.true.
             end do
             if (nfora.ge.1) then
               do m=1,p
                 setk(m) = setk(fica(m))
               end do
             end if  
           else
               call randsk1(p-ndentro,k-ndentro,setk,randsk1pp)           
               if(ndentro .GT. 0) then
                call dcorrigesk(ndentro,dentro,p,setk,poriginal)
               endif
           endif

           do i=1,p
               novager(kpop,i)=setk(i)
	       rep(i)=rep(i).or.setk(i)
           end do
         end do

* tests if all elements are represented

         i=1
         rep(p+1)=.false.
         do while(rep(i))
            i=i+1
         end do
         if(i.LE.p) then 
           call intpr('WARNING: Not all variables are present in the ini
     +tial population for cardinality k=',-1,k,1) 
           call intpr('A larger initial population may be adviseable.'
     +  ,-1,k,0)
         end if

* Determines the value of the criterion for each member of the initial
* population and ranks them by decreasing order. Vector ordemger will 
* register that ranking. Thus, ordemger(1) will contain the number of the 
* subset in the population with the largest criterion value; ordemger(2)
* will contain the number of the subset with the second largest criterion
* values, and so on.


         somavalue=0
         vmaximo=0
         do kpop=1,npop
           do j=1,p
              setk(j)=novager(kpop,j)
           end do

           if (criterio.eq.1) then
		amax = dobjrm(k,setk,p,poriginal,kmax,s,sq,workmat,
     +   setint,skinput)
               critvalue = dsqrt(amax/tracos)
           end if
           if (criterio.eq.2) then
               amax = dobjrv(k,setk,p,poriginal,kmax,s,sq,workmat,
     +   setint,skinput)
               critvalue = dsqrt(amax/tracosq)
           end if
           if (criterio.eq.3) then
	     amax = dobjgcd(k,setk,p,poriginal,kmax,s,workmat,
     +   nqsi,qsi,valp,setint,vecp,fica,skinput)
            critvalue = amax/dsqrt(DBLE(nqsi*k))
           end if
           if (criterio.eq.4) then
               amax=dobjtau2(k,setk,p,poriginal,kmax,s,h,rh,checksg,
     +   numtol,setint,work,egval,skinput,hkinput,ekinput,workmat)
               critvalue = amax
           end if		 
           if (criterio.eq.5) then
               amax=dobjxi2(k,setk,p,poriginal,kmax,s,h,rh,checksg,
     +   numtol,setint,work,egval,skinput,hkinput,workmat)
               critvalue = amax
           end if		 
	   if (criterio.eq.6) then
               amax=dobjzeta2(k,setk,p,poriginal,kmax,s,h,rh,checksg,
     +   numtol,setint,work,egval,skinput,hkinput,ekinput,workmat)
	       critvalue = amax
           end if		 
	   if (criterio.eq.7) then
* tirei rh
               amax=dobjccr12(k,setk,p,poriginal,kmax,s,h,checksg,
     +   numtol,setint,work,egval,skinput,hkinput,workmat)
	       critvalue = amax
           end if

           if (checksg.and.criterio.GT.3.and.vtroca.eq.-0.9999D0) then
	       numprb=.true.        
           end if
	 
	   valueger(kpop) = critvalue
           somavalue=somavalue+valueger(kpop)
           aux=valueger(kpop)
           j=kpop-1
           do while(aux  .GT.  valueger(ordemger(j)))
             ordemger(j+1) = ordemger(j)
             j=j-1
           end do
           ordemger(j+1) = kpop
         end do

* Orders the initial population by decreasing values of the criterion.

         do kpop=1,npop
           do i=1,p
              pop(kpop,i)=novager(ordemger(kpop),i)
           end do
           valuepop(kpop)=valueger(ordemger(kpop))
         end do

***************************************************************************
* Here begins a lengthy loop that will repeat the following procedures
* for each generation: arrange marriages, mate and produce offspring, 
* and select, from among the parents 
* and the offspring, the survivors that will make up a new generation.

         do kger=1,nger

*  Select the individuals to make up the couples. The rules are:
*  1. Select npop/2 males ("garanhao"), each male being randomly selected 
*   with probability distribution proportional to its criterion value, i.e., 
*   prob(kpop selected)=valuepop(kpop)/sum of valuespop of the population 
*   elements (k-subsets).

           ngaranhao=npop/2
           do kgaranhao=1,ngaranhao
*            call rndstart()
            ranaux = unifrnd()
*            call rndend()
            r=ranaux*somavalue
             kpop=1
             acumulado=0
             do while(valuepop(kpop)+acumulado.LT.r)
               acumulado=acumulado+valuepop(kpop)
               kpop=kpop+1
             end do
             garanhao(kgaranhao)=kpop
           end do
  
* 2.  For each male, a female ("femea") will be randomly (uniformly) chosen 
*   from among the population elements (k-subsets) which differ from the male 
*   in at least two genes (variables). This restriction seeks to avoid 
*   consanguinity. Note that if the intersection of the parents is of
*   cardinality k-1, offspring (who must inherent the common part of their 
*   parents) would necessarily be identical to either the mother or the father.

           nfemea=npop/2
           kfemea=1
           ntalvez=0
           do while(kfemea .LE. nfemea)
             talvez=randint(1,npop)
             i=1
             ngenesdistintos=0
             do while (i.LE.p .and. ngenesdistintos.LE.1)
	       if (pop(garanhao(kfemea),i).and. .not.pop(talvez,i)) then
                 ngenesdistintos=ngenesdistintos+1
               end if
               i=i+1
             end do
             if (ngenesdistintos .GE. 2) then 
	       femea(kfemea)=talvez
               kfemea=kfemea+1
	       ntalvez=0
	      else
	       ntalvez=ntalvez+1
	       if(ntalvez .GT. 10000) then
                call intpr(' ABORTING. For cardinality k=',-1,k,1)
                call intpr(' there is not enough genetic diversity in ge
     +neration number ',-1,kger,1) 
                call intpr(' for acceptable levels of consanguinity (cou
     +ples differing by at least 2 genes).',-1,k,0)
                call intpr(' Try reducing the maximum acceptable number
     + of clones (maxclone) or increasing the population size (popsize)'
     + , -1, k,0)
                if (valuepop(1).GE.0.0D0) then
		 call dblepr(' Best criterion value found so far:',
     + -1,valuepop(1),1)
		end if 
                kabort = k

*  preparing the output which will be used in R.
                do kpop=1,npop
                  jjaux=0
                  do i=1,p
                    if (pop(kpop,i)) then
                      jjaux=jjaux+1
		      if (valuepop(kpop).LT.0.0D0) then 
                        vars(npop*kmax*(k-kmin)+npop*(jjaux-1)+kpop)=0
		      else
                        vars(npop*kmax*(k-kmin)+npop*(jjaux-1)+kpop) = 
     +                 fica(i)
		      end if
                    end if
                  end do  
                  valores((k-kmin)*npop+kpop) = valuepop(kpop) 
                end do
                bestval(k-kmin+1)=valuepop(1) 
         	do j=1,k
                   bestvar(kmax*(k-kmin)+j) =
     +           vars(npop*kmax*(k-kmin)+npop*(j-1)+1)
                end do

*                call rndend()
                return
	       end if   
              end if
            end do


* Reproduction.
* The i-th couple, garanhao(i), femea(i) (with i=1,...,npop/2) will produce 
* a child. The child inherits the (possibly empty) common part of both 
* parents. The remaining genes (variables in the k-subset) is selected from 
* among (genes unique to the mother) U (genes unique to the father). If the 
* cardinality of the mother's unique genes (= cardinality of the father's 
* unique genes) is t, k-t genes are randomly (uniformly) selected from among
* the symmetric difference, with the further restriction that not ALL genes 
* will come from a single parent (to avoid clones). If the offspring is an 
* identical clone of "nclone" other existing individuals, it will be discarded
* and replaced with a (uniformly) randomly generated k-subset (an adopted 
* child).

            nfilho=ngaranhao
            do kfilho=1,nfilho
              npainaomae=0
              nmaenaopai=0
              do i=1,p
                filho(kfilho,i)=.false.
	       if(pop(garanhao(kfilho),i).and.
     +             .not.pop(femea(kfilho),i)) then
	         npainaomae=npainaomae+1
	         painaomae(npainaomae)=i
	       end if
	       if(.not.pop(garanhao(kfilho),i).and.
     +             pop(femea(kfilho),i)) then
	         nmaenaopai=nmaenaopai+1
	         maenaopai(nmaenaopai)=i
	       end if
	       if(pop(garanhao(kfilho),i).and.
     +             pop(femea(kfilho),i)) then
	           filho(kfilho,i)=.true.
	       end if
              end do
              kpainaomae=randint(1,npainaomae-1)
              call randsk1(npainaomae,kpainaomae,setk,randsk1pp)
              do i=1,npainaomae
                filho(kfilho,painaomae(i))=setk(i)
              end do
              kmaenaopai=npainaomae-kpainaomae
             call randsk1(nmaenaopai,kmaenaopai,setk,randsk1pp)
              do i=1,nmaenaopai
                filho(kfilho,maenaopai(i))=setk(i)
              end do

* Determines the (nvezes) number of times which this genetic makeup exists 
* in the population (including newborn children). If nvezes > nclone, the 
* child is rejected and a new child of unknown parents is adopted.

              nvezes=0
	      kpop=1
	      do while(nvezes.LE.nclone .and. kpop.LE.npop .and. 
     +            npop-kpop+kfilho.GT.nclone-nvezes)
	        j=1
	        igual=.true.
	        do while(j.LE.p .and. igual)
	          if(filho(kfilho,j).NEQV.pop(kpop,j)) then
		     igual=.false.
                   else
                     j=j+1
	          end if
	        end do
	        if(igual) nvezes=nvezes+1
	        kpop=kpop+1
	      end do
	      kfilhoant=1
	      do while(nvezes.LE.nclone .and. kfilhoant.LT.kfilho .and. 
     +            kfilho-kfilhoant.GE.nclone-nvezes)
	        j=1
	        igual=.true.
	        do while(j.LE.p .and. igual)
	          if (filho(kfilho,j).NEQV.filho(kfilhoant,j)) then
		      igual=.false.
                   else
                      j=j+1
	          end if
	        end do
	        if(igual) nvezes=nvezes+1
	        kfilhoant=kfilhoant+1
	      end do
              if(nvezes.GT.nclone) then
              call randsk1(p-ndentro,k-ndentro,setk,randsk1pp)
              if(ndentro.GT.0) then 
                 call dcorrigesk(ndentro,dentro,p,setk,poriginal)
              endif
              do j=1,p
                filho(kfilho,j)=setk(j)
              end do
             end if
            end do

* Calculates each child's value of the criterion, and ranks the children 
* accordingly (descending order). Thus, ordemger(1) will be the child with 
* the largest value of the criterion, ordemger(2)
* the child with the second largest criterion value, etc.

            do kfilho=1,nfilho
              do j=1,p
	        setk(j)=filho(kfilho,j)
              end do
              if (criterio.eq.1) then
		  amax = dobjrm(k,setk,p,poriginal,kmax,s,sq,workmat,
     +   setint,skinput)
                 critvalue = dsqrt(amax/tracos)
              end if
              if (criterio.eq.2) then
                  amax = dobjrv(k,setk,p,poriginal,kmax,s,sq,workmat,
     +   setint,skinput)
                 critvalue = dsqrt(amax/tracosq)
              end if
              if (criterio.eq.3) then
		 amax = dobjgcd(k,setk,p,poriginal,kmax,s,workmat,
     +   nqsi,qsi,valp,setint,vecp,fica,skinput)
                 critvalue = amax/dsqrt(DBLE(nqsi*k))
              end if
              if (criterio.eq.4) then
                 amax=dobjtau2(k,setk,p,poriginal,kmax,s,h,rh,checksg,
     +   numtol,setint,work,egval,skinput,hkinput,ekinput,workmat)
                 critvalue = amax
              end if 
	      if (criterio.eq.5) then
                 amax=dobjxi2(k,setk,p,poriginal,kmax,s,h,rh,checksg,
     +   numtol,setint,work,egval,skinput,hkinput,workmat)
                 critvalue = amax
              end if
	      if (criterio.eq.6) then
                amax=dobjzeta2(k,setk,p,poriginal,kmax,s,h,rh,checksg,
     +   numtol,setint,work,egval,skinput,hkinput,ekinput,workmat)
	        critvalue = amax
              end if
              if (criterio.eq.7) then
* tirei rh
                 amax=dobjccr12(k,setk,p,poriginal,kmax,s,h,checksg,
     +   numtol,setint,work,egval,skinput,hkinput,workmat)
	         critvalue = amax
              end if
	      valueger(kfilho)=critvalue

* If the logical variable "melhorar" is .true., the newborn child will be 
* genetically improved (i.e., subject to the restricted local improvement 
* algorithm) with probability "mutprob" (0.01 by default). 
* WARNING: This option may tend to generate large numbers of clones 
* and slow down computation times considerably if 
* "mutprob" is large. Default value for "melhorar" is .false.

              if(melhorar.and.critvalue.GE.0.0D0) then
               testran = unifrnd()
               if (testran .LT. mutprob) then
                vantes=amax
                call dmelhoramentogen(criterio,p,kmax,setk,vactual,
     +                ndentro,dentro,k,s,sq,nqsi,qsi,valp,vecp,fica,
     +                poriginal,h,rh,skinput,hkinput,ekinput,egval,
     +                work,workmat,randsk1pp,setint,
     +                checksg,numtol,numprb)
                if(amax.GT.vantes) then
                  if (criterio.eq.1) then
                    critvalue = dsqrt(amax/tracos)
                  end if
                  if (criterio.eq.2) then
                    critvalue = dsqrt(amax/tracosq)
                  end if
                  if (criterio.eq.3) then
                    critvalue = amax/dsqrt(DBLE(nqsi*k))
                  end if
                  if (criterio.eq.4) then
                     critvalue = amax  
                  end if
		  if (criterio.eq.5) then
                    critvalue = amax
                  end if
		  if (criterio.eq.6) then
                      critvalue = amax
                  end if
		  if (criterio.eq.7) then
                      critvalue = amax
                  end if
	          valueger(kfilho)=critvalue
                  do j=1,p
                    filho(kfilho,j)=setk(j)
                  end do
                end if
               end if
              end if
 
              aux=valueger(kfilho)
              j=kfilho-1
              do while(aux.GT.valueger(ordemger(j)))
                 ordemger(j+1)=ordemger(j)
	         j=j-1
              end do
              ordemger(j+1)=kfilho
            end do

* Selection of a new generation.
* The new generation is made up of the npop best (highest criterion value) 
* elements from among the parents and their offspring. novager(1,i), (i=1,p) 
* is the best element in the new generation, novager(2,i) (i=1,p) is the 
* second best, and so on.

            kfilho=1
            kpop=1
            knovager=1
            do while(knovager.LE.npop)
              if(kfilho.GT.nfilho) then
                do while(knovager.LE.npop)
	          do i=1,p
	            novager(knovager,i)=pop(kpop,i)
	          end do
	          valuenovager(knovager)=valuepop(kpop)
                  kpop=kpop+1
	          knovager=knovager+1
	        end do
               else
	        if(valueger(ordemger(kfilho)).GT.valuepop(kpop)) then
	          do i=1,p
	            novager(knovager,i)=filho(ordemger(kfilho),i)
	          end do
	          valuenovager(knovager)=valueger(ordemger(kfilho))
	          kfilho=kfilho+1
	          knovager=knovager+1
	         else
	          do i=1,p
	           novager(knovager,i)=pop(kpop,i)
	          end do
	          valuenovager(knovager)=valuepop(kpop)
                  kpop=kpop+1
	          knovager=knovager+1
	        end if
              end if
            end do
   
* The survivors (i.e., the new population) make up the next generation.
            
            do kpop=1,npop
              do i=1,p
                pop(kpop,i)=novager(kpop,i)
              end do
              valuepop(kpop)=valuenovager(kpop)
            end do

            if(valuepop(1).GT.vmaximo) then
              vmaximo=valuepop(1)
            end if
         end do 

* auxiliary vector for a one-line print-out
         naux=0
         do j=1,p
           if(pop(1,j)) then
             naux=naux+1
             auxw(naux)= fica(j)
           end if
         end do

         critvalue=valuepop(1)

*  preparing the output which will be used in R.
          do kpop=1,npop
           jjaux=0
           do i=1,p
             if (pop(kpop,i)) then
               jjaux=jjaux+1
		if (critvalue.LT.0.0D0) then 
                   vars(npop*kmax*(k-kmin)+npop*(jjaux-1)+kpop) = 0
		else
                   vars(npop*kmax*(k-kmin)+npop*(jjaux-1)+kpop) = 
     +           fica(i)
		end if
             end if
           end do  
           valores((k-kmin)*npop+kpop) = valuepop(kpop) 
          end do

* Optional further improvement of the best solution
*         if (improvement) then
         if (improvement.and.valuepop(1).GE.0.0D0) then
           if (criterio.eq.1) vactual=valuepop(1)**2*tracos
           if (criterio.eq.2) vactual=valuepop(1)**2*tracosq
           if (criterio.eq.3) vactual=valuepop(1)*dsqrt(DBLE(nqsi*k))
           if (criterio.eq.4) vactual=valuepop(1)
           if (criterio.eq.5) vactual=valuepop(1)
	   if (criterio.eq.6) vactual=valuepop(1)
           if (criterio.eq.7) vactual=valuepop(1)
           do j=1,p
              setk(j)=pop(1,j)
           end do
           vcorrente = vactual

	   call dmelhoramentogen(criterio,p,kmax,setk,vactual,
     +         ndentro,dentro,k,s,sq,nqsi,qsi,valp,vecp,fica,
     +         poriginal,h,rh,skinput,hkinput,ekinput,egval,
     +         work,workmat,randsk1pp,setint,checksg,numtol,numprb)

           if (vactual .GT. vcorrente) then 
                   naux=0
                   do j=1,p
                     if(setk(j)) then
                       naux=naux+1
                       auxw(naux)= fica(j)
                     end if
                   end do
                   if (criterio.eq.1) then
                       critvalue=dsqrt(vactual/tracos)
                   end if
                   if (criterio.eq.2) then
                       critvalue=dsqrt(vactual/tracosq)
                   end if
                   if (criterio.eq.3) then
                       critvalue = vactual/dsqrt(DBLE(nqsi*k))
                   end if
	           if (criterio.eq.4) then
                       critvalue=vactual
                   end if
		   if (criterio.eq.5) then
                       critvalue=vactual
                   end if
		   if (criterio.eq.6) then
                       critvalue=vactual
                   end if
		   if (criterio.eq.7) then
                       critvalue=vactual
                   end if
            end if      
         end if

* End of eventual improvement of the best solution

         bestval(k-kmin+1)=critvalue
         do j=1,k
            bestvar(kmax*(k-kmin)+j)=auxw(j)
         end do
         end do 


* End of k-loop

         call rndend()
         return
       end

wald.coef<-function(mat,H,indices, tolval=10*.Machine$double.eps, tolsym=1000*.Machine$double.eps)
{
#   Function wald.coef
#   Computes the value of Wald statistic for testing the significance of the omited variables 
#   in a generalized linear model 

#  error checking

#  mat and indices

  if  (sum(!(as.integer(indices) == indices)) > 0) stop("\n The variable indices must be integers")

  p <- dim(mat)[2]
  validmat(mat,p,tolval,tolsym)
    


# checks on r and H

  validnovcrit(mat,criterion="WALD",H,r=1,p,tolval,tolsym)


# Computing the criterion value

      tr <- function(mat){sum(diag(mat))}
      waldallvar <- tr(solve(mat,H))

      wald.1d <- function(mat,H,indices){ waldallvar - tr(solve(mat[indices,indices],H[indices,indices])) }
      dimension<-length(dim(indices))
      if (dimension > 1){
         wald.2d<-function(mat,H,subsets){
             apply(subsets,1,function(indices){
			
			wald.1d(mat,H,indices)})
            }  
             if (dimension > 2) {
               wald.3d<-function(mat,H,array3d){
                   apply(array3d,3,function(subsets){wald.2d(mat,H,subsets)})
                 }
               output<-wald.3d(mat,H=H,indices)
              }
              if (dimension == 2) {output<-wald.2d(mat=mat,H,indices)}
      }

      if (dimension < 2) {output<-wald.1d(mat,H,indices)}
      output

}

eleaps<-function(mat,kmin=length(include)+1,kmax=ncol(mat)-length(exclude)-1,nsol=1,exclude=NULL,include=NULL,
		  criterion="default",pcindices="first_k",timelimit=15,H=NULL,r=0,
		  tolval=1000*.Machine$double.eps,tolsym=1000*.Machine$double.eps,maxaperr=1E-4) 
{

  NUMERICNULL <- 99E99
  INTEGERNULL <- 99999
  
###############################
# auxiliary  variables        #
###############################

	p <- ncol(mat)    				                # Number of original variables
	nexclude <- length(exclude)     		      # Number of excluded variables
	ninclude <- length(include)     		      # Number of included variables
	if (pcindices[1]!="first_k") esp <- TRUE	# The user has specified the set of Principal Components to be used with the GCD criterion
	else esp <- FALSE				                  # The user has not specified the set of Principal Components to be used with the GCD criterion

        
###############################
# general validation of input #
###############################

         initialization(mat,criterion,r)
         if (validation(mat,kmin,kmax,exclude,include,criterion,pcindices,tolval,tolsym,"eleaps") == "singularmat")  singularmat <- TRUE  
         else singularmat <- FALSE         


#############################################################
# validation specific for the input to the eleaps function  #
#############################################################

	if (timelimit <= 0) stop("\n The time limit argument must be a positive real number\n")
        if ((criterion =="CCR1_2") && (r > 3)) stop("\n The 'eleaps' function does not accept, for the CCR1_2 criterion, \n an effects matrix (H) with  rank greater than 3\n")
        
######################################################################
# Parameter validation if the criterion is one of "TAU_2", "XI_2",   #
# "ZETA_2", "CCR1_2" or "WALD"                                       #
######################################################################

	if (criterion =="TAU_2" || criterion =="XI_2" || criterion =="ZETA_2" || criterion =="CCR1_2" || criterion == "WALD" )
	{
	   validnovcrit(mat,criterion,H,r,p,tolval,tolsym)
	   if (r==1 && criterion!="WALD")  criterion <- "XI_2"	# In multivariate linear problems with r=1 all four criteria are equivalent
	}	    						# but searches based on XI_2 or ZETA_2 are computationally more efficient 
        
##########################################
# Initializations for the C++ subroutine #
##########################################

       # Normalize matrices in order to improve numerical accuracy
	
	if (r==0) mat <- mat/max(mat)
	else {
		scl <- sqrt(diag(mat)) 
    sclotprd <- outer(scl,scl)
		mat <- mat/sclotprd
    H <- H/sclotprd
	}  

	# Matrix initializations
	
	if (singularmat==FALSE) Si <- solve(mat)
	else Si <- NUMERICNULL					

	if (criterion == "RV")  S2 <- mat %*% mat
	else S2 <- NUMERICNULL

	if (criterion == "GCD")  {
	  SSpectd <- eigen(mat,symmetric=TRUE)
	  Segval <- SSpectd$values[pcindices]
	  Segvct <- SSpectd$vectors[,pcindices]
  }  else  {
    Segval <- NUMERICNULL
    Segvct <- NUMERICNULL
  }
 
	if (criterion == "TAU_2" || criterion == "ZETA_2" || (criterion == "CCR1_2" && r > 1) ) {
	   E <- mat - H
	   if (singularmat==FALSE) Ei <- solve(E)	
	   else Ei <- NUMERICNULL				
  } else  E <- Ei <- NUMERICNULL

	if (criterion == "XI_2" || criterion == "WALD" || criterion == "ZETA_2" || criterion == "CCR1_2" ) 	{
	  HSpectd <- eigen(H,symmetric=TRUE)
	  if (r > 1) Hegvct <- HSpectd$vectors[,1:r] %*% sqrt(diag(HSpectd$values[1:r])) 
	  else Hegvct <- HSpectd$vectors[,1] * sqrt(HSpectd$values[1]) 
        }
	else  Hegvct <- NUMERICNULL
	if ( (criterion == "XI_2" || criterion == "WALD" || criterion == "CCR1_2" ) && singularmat==FALSE) 
		HegvctTinv <- solve(mat,Hegvct)
	else HegvctTinv <- NUMERICNULL
	if ( (criterion == "ZETA_2" || (criterion == "CCR1_2" && r == 3)) && singularmat==FALSE)
		HegvctEinv <- solve(E,Hegvct) 
	else HegvctEinv <- NUMERICNULL

	if ( ( criterion == "TAU_2" || (criterion == "CCR1_2" && r > 1) ) && singularmat==FALSE)
		Wilksval <- det(E) / det(mat)
	else Wilksval <- NUMERICNULL
	if ( ( criterion == "XI_2" || criterion == "WALD" || criterion == "CCR1_2" ) && singularmat==FALSE)
		HSi <- t(solve(mat,H))
	if ( ( criterion == "XI_2" || (criterion == "CCR1_2" && r > 1) ) && singularmat==FALSE)
		BartPival <- sum(diag(HSi))
	else BartPival <- NUMERICNULL
        if ( criterion == "WALD") Waldval <- sum(diag(HSi))
        else Waldval <- NUMERICNULL
	if ( ( criterion == "ZETA_2" || (criterion == "CCR1_2" && r == 3) ) && singularmat==FALSE) 
		LawHotval <- sum(diag(solve(E,H)))	
	else LawHotval <- NUMERICNULL
	if ( ( criterion == "CCR1_2") && singularmat==FALSE) 
		CCR12val <- as.numeric(eigen(HSi,symmetric=FALSE,only.values=TRUE)$values[1])	
	else CCR12val <- NUMERICNULL
        
	if (criterion == "WALD")   {

###       Convert a min Wald problem into an (artificial) equivalent max XI2 problem
        
		Hegvct <- Hegvct / sqrt(Waldval)
		HegvctTinv <- HegvctTinv / sqrt(Waldval)
		criterion <- "XI_2"
		criterio <- 5
		BartPival <- 1.
		Waldcrt <- TRUE
	} else Waldcrt <- FALSE

############################
# Call to the C subroutine #
############################

	 Cout <- .Call("eleaps",
           as.double(mat),
           as.double(S2),
           as.double(Si),
           as.double(Segval),
           as.double(Segvct),
           as.double(E),
           as.double(Ei),
           as.double(Hegvct),
           as.double(HegvctTinv),
           as.double(HegvctEinv),
           as.double(Wilksval),	
           as.double(BartPival),
           as.double(LawHotval),
           as.double(CCR12val),
	         as.integer(r),
           as.integer(kmin),
           as.integer(kmax),
           as.integer(nsol),
           as.integer(exclude),
           as.integer(include),
           as.integer(nexclude),
           as.integer(ninclude),
           as.character(criterion),
           as.logical(esp),
           as.integer(pcindices),
           as.integer(length(pcindices)),
           as.integer(p),
	         as.double(timelimit),
	         as.double(maxaperr), 	    
	         as.logical(singularmat), 	    
           PACKAGE="subselect"   
        ) 
	names(Cout) <- c("subsets","values","bestvalues","bestsets","found","nomemory")
	if (Cout$nomemory == TRUE) return(NULL)

#######################################
# Preparing and returning the output  #
#######################################
	
#	 output <- prepRetOtp(c(Cout[1:4],call=match.call()),kmin,kmax,nsol,Waldval,"eleaps",Optimal=Cout$found,tl=timelimit) 
	 output <- prepRetOtp(c(Cout[1:4],call=match.call()),kmin,kmax,nsol,Waldcrt,Waldval,"eleaps",Optimal=Cout$found,tl=timelimit) 

	 if (is.null(output)) return(invisible(NA))
	 output   # return(output)
}

validannimp<-function(kmin, kmax, nsol, exclude, nexclude, include, ninclude, initialsol, implog){


##################################################################
# validation of input that is specific to the anneal and improve #
# functions (which admit similar possible initial solutions)     #
##################################################################



##################################################################
# initializations when initial solutions are specified; checking #
# nature of initialsol (and how to interpret it) and for         #
# conflicts with the exclude and include requirements (that must #
# be respected by the initial solutions)                         #
##################################################################

        if  (!is.null(initialsol) & (sum(!(as.integer(initialsol) == initialsol)) > 0)) stop("\n The initial solutions must be specified as integers indicating variable numbers")
        if (is.null(initialsol)) {silog <- FALSE}
        else {silog <- TRUE  # initial solutions have been specified by user          
#########################################################
# checking for the presence of variables that are to be #
# forcefully excluded                                   #
#########################################################

            if ((nexclude != 0) & (sum(exclude == rep(initialsol,rep(length(exclude),length(as.vector(initialsol))))) !=0)) stop("\n the specified initial solutions contain variables that are to be excluded")

#########################################################
# how to deal with various formats of input (of initial #
# solutions) for a single cardinality                   #
#########################################################

          dimsol<-dim(initialsol)
              
          if (length(dimsol) > 3) {
            stop("\n Can't handle arrays of more than 3 dimensions")}
          if (kmin == kmax) {

#############################################################
# inital solution is a vector: must be repeated if nsol > 1 #
#############################################################


               if (is.vector(initialsol)) {
                   if (length(initialsol) != kmax) 
                     {stop("\n The specified initial and final solutions have different cardinalities")}
                   else
                     {if (nsol > 1) {
                     initialsol<-matrix(nrow=nsol,ncol=kmax,rep(initialsol,rep(nsol,kmax)))
                     if (implog==TRUE) {warning("\n a single initial subset necessarily produces nsol identical \n final solutions in the improve algorithm")}}
                    }
               }


##############################
# initial solution is array? #
##############################

           if (is.array(initialsol)) {

###########################
# initialsol is 3-d array #
###########################

           if (length(dimsol) == 3) {
            if (dim(initialsol)[[3]] > 1) stop("\n The input array of initial solutions must have dimensions as \n (1 or nsol) x k x 1 when a single cardinality is requested")
            else initialsol<-matrix(nrow=dim(initialsol)[[1]],ncol=dim(initialsol)[[2]],initialsol)
            }
           else

##################################################
# initialsol is matrix: must be of form nsol x k #
##################################################

                 {if (dim(initialsol)[[2]] != kmax) stop("\n Input matrix of initial solutions must have as many columns as variables in the requested subset")
                  else 
                      {if  (dim(initialsol)[[1]] != nsol) {
                         if (dim(initialsol)[[1]] == 1) {
                          initialsol<-matrix(nrow=nsol,ncol=dim(initialsol)[[2]],rep(initialsol,rep(nsol,kmax))) 
                          if (implog==TRUE) {warning("\n a single initial subset necessarily produces nsol identical \n final solutions in the improve algorithm")}
                         }
                           else {stop("\n The number of initial solutions can only be 1 or nsol (number of final solutions requested)")}}}}
              }}



##########################################################
# how to deal with various formats of input (initialsol) #
# if more than one cardinality is requested              #
##########################################################

             else  # (if kmax > kmin), i.e., more than one cardinality requested

###############################################
# initial solution cannot be a single vector  #
###############################################

              {if (is.vector(initialsol)) stop("\n There must be initial solutions for all cardinalities requested")

               if (is.array(initialsol)) {

###############################
# initial solution 3-d array? #
###############################


                  if (length(dim(initialsol)) == 3) {
                   if ((dim(initialsol)[[3]] != length(kmin:kmax)) | (dim(initialsol)[[2]] != kmax) | ((dim(initialsol)[[1]] != nsol) & (dim(initialsol)[[1]] > 1))) stop("\n The input array of initial solutions must have dimensions as \n nsol x kmax x no. of different cardinalities requested")
                   else 
                    { if (dim(initialsol)[[1]] != nsol) {
                       initialsol<-array(dim=c(nsol,kmax,length(kmin:kmax)),rep(initialsol,rep(nsol,kmax*length(kmin:kmax))))
                      if (implog==TRUE) {warning("\n a single initial subset necessarily produces nsol identical \n final solutions in the improve algorithm")}
                      } 
                  }}
                else 


############################
# if initalsol is a matrix #
############################


                    {
                    if ((dim(initialsol)[[2]] != kmax) | (dim(initialsol)[[1]] != length(kmin:kmax))) stop("\n A matrix of initial solutions for more than one cardinality must be of type no. of cardinalities x kmax")
                    if (nsol > 1) {
                       initialsol<-array(dim=c(nsol,kmax,length(kmin:kmax)),rep(t(initialsol),rep(nsol,kmax*length(kmin:kmax)))) 
                    if (implog==TRUE) {warning("\n a single initial subset necessarily produces nsol identical \n final solutions in the improve algorithm")}
                    }
}}}
                 if ((ninclude != 0) & (sum(include == rep(initialsol,rep(length(include),length(as.vector(initialsol))))) != nsol*length(kmin:kmax)*length(include))) stop("\n Not all the specified initial solutions contain the variables that are to be included") 
}


##################################
#  assigning any changed values  #
##################################

         assign("silog",silog,pos=parent.frame())          
         assign("initialsol",initialsol,pos=parent.frame())

}

validmat<-function(mat,p,tolval,tolsym,allowsingular,algorithm=c("anneal","genetic","improve","eleaps","none"))
{
#   Function validmat
#   Checks the main (covariance/total sums of squares and products)
#   matrix for size, symmetry, positive definiteness, etc.
#   To be called from other functions, not directly by the user.

# valid input?

  algorithm <- match.arg(algorithm) 
  if (!is.matrix(mat)) {
         stop("Data is missing or is not given in matrix form")}
     if (dim(mat)[1] != dim(mat)[2]) {
         mat<-cor(mat)
         warning("Data must be given as a covariance or correlation matrix. \n It has been assumed that you wanted the correlation matrix of the \n data matrix which was supplied.")
       }

# checking for symmetry in the 'total' input matrix

         maxabssym <- max(abs(mat-t(mat)))
         if (maxabssym > tolsym)
           {stop("\n The covariance/total matrix supplied is not symmetric.\n  Symmetric entries differ by up to ",maxabssym,".")}
         else if (maxabssym > .Machine$double.eps) {
              mat <-(mat+t(mat))/2
                    warning("\n The covariance/total matrix supplied was slightly asymmetric: \n symmetric entries differed by up to ",maxabssym,".\n (less than the 'tolsym' parameter).\n It has been replaced by its symmetric part.\n")
}

        eigvals <- eigen(mat,symmetric=TRUE,only.values=TRUE)$values
	repcnumb <- eigvals[p]/eigvals[1] 


  # Positive definiteness

 	if ( repcnumb < -1*tolval )
            stop("\n The covariance/correlation matrix supplied is not positive definite")
 
# checking for ill-conditioned 'total' matrix

         if (repcnumb < tolval) {    
		if (allowsingular==TRUE)  { 
			if (algorithm!="none" && (is.null(attr(mat,"KnownSing")) || attr(mat,"KnownSing")!=TRUE))  {
				if (algorithm == "eleaps") 
					warning(paste("\n The covariance/correlation matrix supplied has reciprocal condition number smaller than \n the specified threshold of ",tolval,".\n Therefore the full variable set might not have a well defined criterion value and the search \n will be restricted to subsets where correlation matrices are well conditioned.\n Setting lower values for the 'tolval' and 'maxaperr' function arguments may force more solutions, but numerical \n accuracy may be compromised.",sep=""))  
				else			
					warning(paste("\n The covariance/correlation matrix supplied has reciprocal condition number smaller than \n the specified threshold of ",tolval,".\n Therefore the full variable set might not have a well defined criterion value and the search \n will be restricted to subsets where correlation matrices are well conditioned.\n Setting lower values for the 'tolval' function argument may force more solutions, but numerical \n accuracy may be compromised.",sep=""))  

			}
        		return("singularmat")
         	}                                    
		if (allowsingular==FALSE)   	
			stop(paste("\n The covariance/correlation matrix supplied has reciprocal condition number \n smaller than the specified threshold of", tolval,".\n Suggestion 1: Setting a lower value of the 'tolval' function argument may force a solution, \n but numerical accuracy  may be compromised. \n Suggestion 2: Try a new function call after excluding  variables responsible for the (real \n or approximate) linear dependences. See help(trim.matrix) for  assistance in this respect. \n Suggestion 3: Try a well-conditioned shrinkage estimate of the covariance/correlation \n matrix instead. See the R package corpcor on CRAN."))  
	}	

        return("validmat")

}

rm.coef<-function(mat, indices)
{

#   Computes the matrix correlation between data matrices and their 
#   regression on a subset of their variables. Expected input is a
#   variance-covariance (or correlation) matrix. 

#  error checking


  if  (sum(!(as.integer(indices) == indices)) > 0) stop("\n The variable indices must be integers")
  if (!is.matrix(mat)) {
         stop("Data is missing or is not given in matrix form")}
     if (dim(mat)[1] != dim(mat)[2]) {
         mat<-cor(mat)
         warning("Data must be given as a covariance or correlation matrix. \n It has been assumed that you wanted the correlation matrix of the \n data matrix which was supplied.")
       }
      tr<-function(mat){sum(diag(mat))}
      rm.1d<-function(mat,indices){
        sqrt(tr((mat %*% mat)[indices,indices] %*% solve(mat[indices,indices]))/tr(mat))
        }
      dimension<-length(dim(indices))
      if (dimension > 1){
         rm.2d<-function(mat,subsets){
             apply(subsets,1,function(indices){rm.1d(mat,indices)})
            }  
             if (dimension > 2) {
               rm.3d<-function(mat,array3d){
                   apply(array3d,3,function(subsets){rm.2d(mat,subsets)})
                 }
               output<-rm.3d(mat,indices)
              }
              if (dimension == 2) {output<-rm.2d(mat,indices)}
      }

      if (dimension < 2) {output<-rm.1d(mat,indices)}
      output
}

genetic<-function(mat, kmin, kmax=kmin, popsize=max(100,2*ncol(mat)), nger=100,
mutate=FALSE, mutprob=0.01, maxclone=5, exclude=NULL, include=NULL,
improvement=TRUE, setseed= FALSE,  criterion="default", pcindices="first_k",
initialpop=NULL, force=FALSE, H=NULL, r=0,tolval=1000*.Machine$double.eps,
tolsym=1000*.Machine$double.eps)
{


#########################################################################################
# auxiliary  variables (includes declarations to avoid "no visible binding" NOTE)       #
#########################################################################################

	p <- ncol(mat)    				# Number of original variables
	nexclude <- length(exclude)     		# Number of excluded variables
	ninclude <- length(include)     		# Number of included variables				 
	if (pcindices[1]!="first_k") esp <- TRUE		# The user has specified the set of Principal Components to be used with the GCD criterion
	else esp <- FALSE				# The user has not specified the set of Principal Components to be used with the GCD criterion
	if (!is.null(initialpop)) pilog <- TRUE		# The user has specified an initial population
	else pilog <- FALSE				# The user has not specified an initial population
	exc <- exclude					# Name for vector of excluded variables after 'validation' routine 
	inc <- include					# Name for vector of included variables after 'validation' routine 

        
###############################
# general validation of input #
###############################

  
        initialization(mat, criterion, r)
        if (validation(mat, kmin, kmax, exclude, include, criterion, pcindices, tolval, tolsym, "genetic") == "singularmat")  singularmat = TRUE  
        else singularmat = FALSE         
        maxnovar = 400
        if ((p > maxnovar) & (force==FALSE)) stop("\n For very large data sets, memory problems may crash the R session. \n To proceed anyways, repeat the function call with \n the argument 'force' set to 'TRUE' (after saving anything important \n from the current session)\n")


######################################################################
# Parameter validation if the criterion is one of "TAU_2", "XI_2",   #
# "ZETA_2" or "CCR1_2"  or "WALD"                                    #
######################################################################

if (criterion == "TAU_2" || criterion == "XI_2" || criterion ==
"ZETA_2" || criterion == "CCR1_2" || criterion == "WALD") validnovcrit(mat,criterion,H,r,p,tolval,tolsym)


##############################################################
# more specific validation of input for the genetic function #
##############################################################

        validgenetic(kmin, kmax, popsize, mutprob, exclude, nexclude, include, ninclude, initialpop)


##############################################
# initializations for the Fortran subroutine #
##############################################

       # Normalize matrices in order to improve numerical accuracy
	
   if (r==0) mat <- mat/max(mat)
   else {
     scl <- sqrt(diag(mat)) 
     sclotprd <- outer(scl,scl)
     mat <- mat/sclotprd
     H <- H/sclotprd
  }  

  if (setseed == TRUE) set.seed(2,kind="default")
  valores<-rep(0.0,length(kmin:kmax)*popsize)
  vars<-rep(0,popsize*length(kmin:kmax)*kmax)
  bestval<-rep(0.0,length(kmin:kmax))
  bestvar<-rep(0,kmax*length(kmin:kmax))
  kabort<-kmax+1
  if (criterio == 3) {
    decespectral<-eigen(mat,symmetric=TRUE)
    valp<-decespectral$values
    vecp<-decespectral$vectors
  }  else {
    valp<-rep(0,p)
    vecp<-matrix(nrow=p,ncol=p,rep(0,p*p))
  }


##################################
# call to the Fortran subroutine #
##################################

  if (criterion == "WALD")   {

###       Convert a min Wald problem into an (artificial) equivalent max "XI2" problem
        
    Waldval <- sum(diag(solve(mat,H)))
    H <- H / Waldval 
    criterion <- "XI_2"
    criterio <- 5
    Waldcrt <- TRUE
  }  else {
    Waldcrt <- FALSE
    Waldval <- NULL
  }  
        
  Fortout <- .Fortran("genetic",as.integer(criterio),as.integer(p),
          as.double(as.vector(mat)),
          as.integer(kmin),as.integer(kmax),as.double(valores),
          as.integer(vars),as.double(bestval),as.integer(bestvar),
          as.integer(nexclude),as.integer(exc),as.integer(ninclude),
          as.integer(inc),as.integer(popsize),as.integer(nger),
          as.integer(maxclone),as.logical(mutate),
          as.double(mutprob),as.logical(improvement),
          as.integer(length(pcindices)),as.integer(pcindices),as.logical(esp),
          as.integer(kabort),as.logical(pilog),
          as.integer(as.vector(initialpop)),as.double(valp),
          as.double(as.vector(vecp)),as.double(as.vector(H)),as.integer(r),
	  as.logical(singularmat),as.double(tolval),logical(1),
	  PACKAGE="subselect")

#######################################
# Preparing and returning the output  #
#######################################

# output <- prepRetOtp(c(Fortout[6:9],call=match.call()),kmin,kmax,popsize,Waldval,"genetic",Numprb=Fortout[[32]]) 
  output <- prepRetOtp(c(Fortout[6:9],call=match.call()),kmin,kmax,popsize,Waldcrt,Waldval,"genetic",Numprb=Fortout[[32]]) 
  kabort <- Fortout[[23]]
  if (is.null(output)) return(invisible(NA))
  maxk <- min(kabort,kmin+nrow(output$bestsets)-1)

  list(subsets=output$subsets[,1:maxk,1:(maxk-kmin+1),drop=FALSE],
		     values=output$values[,1:(maxk-kmin+1),drop=FALSE],
		     bestvalues=output$bestvalues[1:(maxk-kmin+1)],
		     bestsets=output$bestsets[1:(maxk-kmin+1),1:maxk,drop=FALSE],
		     call=output$call)
}

#prepRetOtp <- function(output,kmin,kmax,nsol,Waldval,algorithm=c("anneal","genetic","improve","eleaps"),Numprb=NULL,Optimal=NULL,tl=NULL)
prepRetOtp <- function(output,kmin,kmax,nsol,Waldcrt,Waldval,algorithm=c("anneal","genetic","improve","eleaps"),Numprb=NULL,Optimal=NULL,tl=NULL)
{
	 algorithm <- match.arg(algorithm) 
	 if (algorithm!="eleaps")  {
        	valores <- matrix(nrow=nsol,ncol=length(kmin:kmax),output[[1]])
        	variaveis <- array(output[[2]],c(nsol,kmax,length(kmin:kmax)))
        	bestval <- output[[3]]
        	bestvar <- t(matrix(nrow=kmax,ncol=length(kmin:kmax),output[[4]]))
        	output <- list(subsets=variaveis,values=valores,bestvalues=bestval,bestsets=bestvar,call=output[[5]])
	 }
         dimnames(output$subsets)<-list(paste("Solution",1:nsol,sep=" "),paste("Var",1:kmax,sep="."),paste("Card",kmin:kmax,sep="."))
         dimnames(output$values)<-list(paste("Solution",1:nsol,sep=" "),paste("card.",kmin:kmax,sep=""))
         names(output$bestvalues)<-paste("Card",kmin:kmax,sep=".")
         dimnames(output$bestsets)<-list(paste("Card",kmin:kmax,sep="."),paste("Var",1:kmax,sep="."))

   	 ndim <- kmax-kmin+1
	 dimind <- 1:ndim
	 nosol <- dimind[sapply(dimind,function(dim) all(output$subsets[,,dim]==0))]
	 if (length(nosol)>0)  {

		wrnmsg1 <- paste(algorithm,"was not able to find any reliable non-collinear subset")

		if (algorithm=="eleaps") 

			wrnmsgEl <- paste("within the specified time limit of ",tl," seconds.\nTo search for reliable solutions either increase the value of the function argument 'timelimit',\n or try one of the available meta-heuristics (anneal, genetic or improve).\n")

		if (length(nosol)==ndim)  {
			if (algorithm=="eleaps") warning(paste(wrnmsg1,wrnmsgEl)) 
			else warning(paste(wrnmsg1,".\n",sep="")) 
			return(NULL)
		}
		else {
			if (length(nosol)==1)  wrnmsg2 <- paste("of dimensionality",kmin+nosol-1)
			else  wrnmsg2 <- paste("of dimensionalities",paste(kmin+nosol-1,collapse=" "))
			if (algorithm=="eleaps") warning(paste(wrnmsg1,wrnmsg2,"\n",wrnmsgEl)) 
			else warning(paste(wrnmsg1," ",wrnmsg2,".\n",sep="")) 
			output$subsets <- output$subsets[,,-nosol,drop=FALSE]  
			output$values <- output$values[,-nosol,drop=FALSE]  
			output$bestsets <- output$bestsets[-nosol,,drop=FALSE]  
			output$bestvalues <- output$bestvalues[-nosol]  
		}
	 } 
	 for (dim in 1:length(output$bestvalues))  {
		trialind <- 1:nsol
		nullsols <- trialind[sapply(trialind,function(ind) all(output$subsets[ind,,dim]==0))] 
		if (length(nullsols)>0)  output$values[nullsols,dim] <- 0.  
 	}
	if (Waldcrt)  {
	  criterion <- "WALD"
	  criterio <- 8
	  validvalues <- output$values[output$values > 0.]
	  output$values[output$values > 0.] <- rep(Waldval,length(validvalues)) - validvalues * Waldval
	  output$bestvalues <- rep(Waldval,ndim) - output$bestvalues * Waldval  
	}
	if (algorithm=="eleaps")  {
          if (Optimal == FALSE) 
            warning("eleaps was not able to complete an exact search within the specified time limit of ",tl," seconds, \n and the optimality of the returned solutions cannot be guaranteed.\n To search for better solutions either increase the value of the function argument 'timelimit',\n or try one of the available meta-heuristics (anneal, genetic or improve).\n") 
	} else  if (Numprb==TRUE)  
	  warning("Because of numerical problems caused by strong multicolinearity\n some subsets were excluded from the analysis.\nYou can try to increase the number of subsets to be compared by reducing the value\nof the function argument 'tolval', but the numerical accuracy of results may be compromised.\n")

	 output  #  return(output)
}

#### lmHmat methods ##################

lmHmat <- function(...)  UseMethod("lmHmat")
  
lmHmat.default <- function(x,y,...)
{
  if  ( !is.matrix(x) || ( !is.matrix(y) && !is.vector(y) ) )  stop("Arguments of wrong type")
   n <- nrow(x)
   p <- ncol(x)
   if (is.vector(y)) q <- 1
   else q <- ncol(y)
   if ( (is.matrix(y) && nrow(y) != n) || ( is.vector(y) && length(y) != n )  )
     stop("Argument dimensions do not match")

   Sx <- var(x)
   if (n <= p) attr(Sx,"KnownSing") <- TRUE
   dx <- x - matrix(rep(apply(x,2,mean),n),n,p,byrow=T)
   if (q==1) {
	dy <- y - rep(mean(y),n)
	uy <- dy / sqrt(sum(dy^2))
   }	
   else  {
	dy <- y - matrix(rep(apply(y,2,mean),n),n,q,byrow=T)
   	uy <- svd(dy,nv=0)$u
   }
   h <- t(uy) %*% dx
   H <- t(h) %*% h / (n-1)

  res <- list(mat=Sx,H=H,r=min(p,q),call=match.call())
  res
} 

lmHmat.data.frame <- function(x,y,...)
{
   res <- lmHmat.default(data.matrix(x),as.matrix(y))
   res$call <- match.call()
   res
}

lmHmat.formula <- function(formula,data=NULL,...)
{
   m <- match.call()
   if (is.matrix(eval(m$data,sys.parent()))) m$data <- as.data.frame(data)
   m[[1]] <- as.name("model.frame")
   m <- eval(m,sys.parent())
   Terms <- attr(m,"terms")
   y <- model.extract(m,"response")
   x <- model.matrix(Terms,m)
   xint <- match("(Intercept)",dimnames(x)[[2]],nomatch=0)
   if (xint>0) x <- x[,-xint,drop=F]
   if (is.null(dim(y)) || length(dim(y)==1))
     yind <- which(apply(x,2,function(v) all(v==y)))
   else {
     yind <- NULL
     for (j in 1:ncol(y))
       yind <- c(yind,which(apply(x,2,function(v) all(v==y[,j]))))
   } 
   if (length(yind)>0) x <- x[,-yind]
   res <- lmHmat.default(x,y)
   res$call <- match.call()
   res 
}

lmHmat.lm <- function(fitdlmmodel,...)
  {
    formula <- fitdlmmodel$call$formula
    data <- fitdlmmodel$model
    res <- lmHmat.formula(formula=formula,data=data)
    res$call <- match.call()
    res
  }

 
#### ldaHmat methods ##################

ldaHmat <- function(...) UseMethod("ldaHmat")

ldaHmat.default <- function(x,grouping,...)
{
 if  ( !is.matrix(x) || !is.factor(grouping) ) stop("Arguments of wrong type")
  n <- nrow(x)
  if (n != length(grouping)) stop("Argument dimensions do not match")
  grp <- factor(grouping,levels=unique(grouping)) 
  nk <- table(grp)
  k <- nrow(nk)
  T <- (n-1)*cov(x)
  if (n <= ncol(x)) attr(T,"KnownSing") <- TRUE
  Si <- by(x,grp,cov)
  H <- T
  for (i in 1:k) H <- H - (nk[[i]]-1)*Si[[i]]
  res <- list(mat=T,H=H,r=min(ncol(x),k-1),call=match.call())
  res
}

ldaHmat.data.frame <- function(x,grouping,...)
{
   res <- ldaHmat.default(data.matrix(x),grouping)
   res$call <- match.call()
   res
}

ldaHmat.formula <- function(formula,data=NULL,...)
{
   m <- match.call()
   if (is.matrix(eval(m$data,sys.parent()))) m$data <- as.data.frame(data)
   m[[1]] <- as.name("model.frame")
   m <- eval(m,sys.parent())
   Terms <- attr(m,"terms")
   grouping <- model.extract(m,"response")
   x <- model.matrix(Terms,m)
   xint <- match("(Intercept)",dimnames(x)[[2]],nomatch=0)
   if (xint>0) x <- x[,-xint,drop=F]
   res <- ldaHmat.default(x,grouping)
   res$call <- match.call()
   res
}

#### glhHmat methods ##################

glhHmat <- function(...)  UseMethod("glhHmat")

glhHmat.default <- function(x,A,C,...)
{
   if  ( !is.matrix(x) || !is.matrix(A) || ( !is.matrix(C) && !is.vector(C) ) )   stop("Arguments of wrong type")
   if (is.vector(C))  C <- matrix(C,1,length(C))
   if ( nrow(A) != nrow(x) || ncol(C) != ncol(A) )  stop("Argument dimensions do not match")

   r <- qr(C)$rank
   rA <- qr(A)$rank
   if ( rA <= r) stop("There are not enough linearly independent columns in the desing matrix (A)")
   svdA <- svd(A)
   a <-  t(svdA$u) %*% x 
   E <- t(x) %*% x - t(a) %*% a
   M <- svdA$u[,1:rA] %*% diag(svdA$d[1:rA]^-1) %*% t(svdA$v[,1:rA]) %*% t(C)
   h <- t(svd(M,nv=0)$u) %*% x
   H <- t(h) %*% h
   T <- H+E	
   if (nrow(x) <= ncol(x)) attr(T,"KnownSing") <- TRUE

   res <- list(mat=T,H=H,r=r,call=match.call())
   res
}

glhHmat.data.frame <- function(x,A,C,...)
{
   if (is.vector(C)) res <- glhHmat.default(data.matrix(x),as.matrix(A),C)
   else res <- glhHmat.default(data.matrix(x),as.matrix(A),as.matrix(C))
   res$call <- match.call()
   res
}

glhHmat.formula <- function(formula,C,data=NULL,...)
{
   m <- match.call()
   m$C <- NULL
   if (is.matrix(eval(m$data,sys.parent()))) m$data <- as.data.frame(data)
   m[[1]] <- as.name("model.frame")
   m <- eval(m,sys.parent())
   Terms <- attr(m,"terms")
   x <- model.extract(m,"response")
   A <- model.matrix(Terms,m)
   if (is.vector(C)) res <- glhHmat.default(as.matrix(x),A,C)
   else res <- glhHmat.default(as.matrix(x),A,as.matrix(C))
   res$call <- match.call()
   res
}

#### glmHmat methods ##################

glmHmat <- function(...)  UseMethod("glmHmat")

glmHmat.glm <- function(fitdglmmodel,...)
{
    if ( names(coef(fitdglmmodel)[1]) == "(Intercept)" )  {
    	b <- coef(fitdglmmodel)[-1]
    	Sb <- vcov(fitdglmmodel)[-1,-1]
    }
    else  {	
    	b <- coef(fitdglmmodel)
    	Sb <- vcov(fitdglmmodel)
    }
    FI <- solve(Sb)
    attr(FI,"FisherI") <- TRUE
    h <- solve(Sb,b)
    H <- h %o% h
    list(mat=FI,H=H,r=1,call=match.call())
}

ccr12.coef<-function(mat,H,r,indices,tolval=10*.Machine$double.eps,tolsym=1000*.Machine$double.eps)
{
#   Function ccr1_2.coef
#   Computes the first squared canonical correlation.  
#   This criterion is equivalent to the maximization of Roy first root.
#   Expected input: a variance-covariance (or correlation) matrix, 
#   Effect descrption matrix (H) and its rank (r)

#  error checking

#  mat and indices

  if  (sum(!(as.integer(indices) == indices)) > 0) stop("\n The variable indices must be integers")

  p <- dim(mat)[2]
  validmat(mat,p,tolval,tolsym,allowsingular=TRUE,algorithm="none")


# checks on r and H

  validnovcrit(mat,criterion="CCR1_2",H,r,p,tolval,tolsym)
  
#  Computing the criterion value  

      ccr12.1d<-function(mat,H,r,indices){
		
	Re(eigen(solve(mat[indices,indices],H[indices,indices]))$values[1])
		
       }
      dimension<-length(dim(indices))
      if (dimension > 1){
         ccr12.2d<-function(mat,H,r,subsets){
             apply(subsets,1,function(indices){
			
			ccr12.1d(mat,H,r,indices)})
            }  
             if (dimension > 2) {
               ccr12.3d<-function(mat,H,r,array3d){
                   apply(array3d,3,function(subsets){ccr12.2d(mat,H,r,subsets)})
                 }
               output<-ccr12.3d(mat,H=H,r,indices)
              }
              if (dimension == 2) {output<-ccr12.2d(mat=mat,H,r,indices)}
      }

      if (dimension < 2) {output<-ccr12.1d(mat,H,r,indices)}
      output
}

rv.coef<-function(mat, indices)
{
#    Computes Escoufier's RV-coefficient for the configuration of
#    points defined by n observations of a set of p variables, and by
#    the regression of all variables on a subset of k variables
#    (given by \code{indices}). 

#  error checking

     if  (sum(!(as.integer(indices) == indices)) > 0) stop("\n The variable indices must be integers")
     if (!is.matrix(mat)) {
         stop("Data is missing or is not given in matrix form")}
     if (dim(mat)[1] != dim(mat)[2]) {
         mat<-cor(mat)
         warning("Data must be given as a covariance or correlation matrix. \n It has been assumed that you wanted the correlation matrix of the \n data matrix which was supplied.")
       }
      tr<-function(mat){sum(diag(mat))}
      rv.1d<-function(mat,indices){
             mat2 <- (mat %*% mat)[indices, indices]
             invmatk <- solve(mat[indices, indices])
             sqrt(tr(mat2 %*% invmatk %*% mat2 %*% invmatk)/tr(mat %*% mat))
        }
      dimension<-length(dim(indices))
      if (dimension > 1){
         rv.2d<-function(mat,subsets){
             apply(subsets,1,function(indices){rv.1d(mat,indices)})
            }  
             if (dimension > 2) {
               rv.3d<-function(mat,array3d){
                   apply(array3d,3,function(subsets){rv.2d(mat,subsets)})
                 }
               output<-rv.3d(mat,indices)
              }
              if (dimension == 2) {output<-rv.2d(mat,indices)}
      }

      if (dimension < 2) {output<-rv.1d(mat,indices)}
      output
}

anneal <- function(mat, kmin, kmax=kmin, nsol=1, niter=1000,exclude=NULL, include=NULL, improvement=TRUE,setseed = FALSE, cooling=0.05, temp=1,coolfreq=1, criterion="default", pcindices="first_k", initialsol=NULL,force=FALSE,H=NULL,r=0, tolval=1000*.Machine$double.eps, 
tolsym=1000*.Machine$double.eps)
{


#########################################################################################
# auxiliary  variables (includes declarations to avoid "no visible binding" NOTE)       #
#########################################################################################

	p <- ncol(mat)    				# Number of original variables
	nexclude <- length(exclude)     		# Number of excluded variables
	ninclude <- length(include)     		# Number of included variables				 
	if (pcindices[1]!="first_k") esp <- TRUE	# The user has specified the set of Principal Components to be used with the GCD criterion
	else esp <- FALSE				# The user has not specified the set of Principal Components to be used with the GCD criterion
	if (!is.null(initialsol)) silog <- TRUE		# The user has specified a set of initial solutions
	else silog <- FALSE				# The user has not specified a set of initial solutions
	exc <- exclude					# Name for vector of excluded variables after 'validation' routine 
	inc <- include					# Name for vector of included variables after 'validation' routine 


        
###############################
# general validation of input #
###############################

        initialization(mat, criterion, r)
         if (validation(mat, kmin, kmax, exclude, include, criterion, pcindices, tolval, tolsym, "anneal") == "singularmat")  
		singularmat <- TRUE  
         else singularmat <- FALSE         
        maxnovar = 400
        if ((p > maxnovar) & (force==FALSE)) stop("\n For very large data sets, memory problems may crash the R session. \n To proceed anyways, repeat the function call with \n the argument 'force' set to 'TRUE' (after saving anything important \n from the current session)\n")
        

######################################################################
# Parameter validation if the criterion is one of "TAU_2", "XI_2",   #
# "ZETA_2" or "CCR1_2" or "WALD"                                     #
######################################################################

if (criterion == "TAU_2" || criterion == "XI_2" || criterion ==
"ZETA_2" || criterion == "CCR1_2" || criterion == "WALD") validnovcrit(mat,criterion,H,r,p,tolval,tolsym)


##########################################################################
# more specific validation of input for the anneal and improve functions #
##########################################################################

        implog<-FALSE
        validannimp(kmin, kmax, nsol, exclude, nexclude, include, ninclude, initialsol,implog)

#############################################################
# validation specific for the input to the anneal function  #
#############################################################

        if (!(as.integer(coolfreq) == coolfreq) | (coolfreq < 1)) stop("\n The 'cooling frequency' parameter must be a non-negative integer")
        if (cooling<=0 || cooling >=1) stop("\n values of the 'cooling' parameter must be between 0 and 1")

##########################################
# initializations for Fortran subroutine #
##########################################

       # Normalize matrices in order to improve numerical accuracy
	
	if (r==0) mat <- mat/max(mat)
	else {
		scl <- sqrt(diag(mat)) 
        	sclotprd <- outer(scl,scl)
		mat <- mat/sclotprd
        	H <- H/sclotprd
	}  
        if (setseed == TRUE) {set.seed(2,kind="default")} 
        valores<-rep(0.0,length(kmin:kmax)*nsol)    
        vars<-rep(0,nsol*length(kmin:kmax)*kmax)
        bestval<-rep(0.0,length(kmin:kmax))
        bestvar<-rep(0,kmax*length(kmin:kmax))
        if (criterio == 3) {
          decespectral<-eigen(mat,symmetric=TRUE)
          valp<-decespectral$values
          vecp<-decespectral$vectors
        }
        else {
              valp<-rep(0,p)
              vecp<-matrix(nrow=p,ncol=p,rep(0,p*p))
        }
     

##################################
# call to the Fortran subroutine #
##################################
         
	if (criterion == "WALD")   {

###       Convert a min Wald problem into an (artificial) equivalent max XI2 problem
        
    Waldval <- sum(diag(solve(mat,H)))
    H <- H / Waldval     
	  criterion <- "XI_2"
		criterio <- 5
		Waldcrt <- TRUE
	}
  else {
    Waldcrt <- FALSE
    Waldval <- NULL
  }  

       Fortout<-.Fortran("anneal",as.integer(criterio),as.integer(p),
          as.double(as.vector(mat)),
          as.integer(kmin),as.integer(kmax),as.double(valores),
          as.integer(vars),as.double(bestval),as.integer(bestvar),
          as.integer(nexclude),as.integer(exc),as.integer(ninclude),
          as.integer(inc),as.integer(nsol),as.integer(niter),
          as.logical(improvement),
          as.double(cooling),as.double(temp),
          as.integer(coolfreq),as.integer(length(pcindices)),
          as.integer(pcindices),as.logical(esp),as.logical(silog),
          as.integer(as.vector(initialsol)), as.double(valp),
          as.double(as.vector(vecp)), as.double(as.vector(H)),as.integer(r),
	  as.logical(singularmat),as.double(tolval),logical(1),
	  PACKAGE="subselect")

#######################################
# Preparing and returning the output  #
#######################################

#	output <- prepRetOtp(c(Fortout[6:9],call=match.call()),kmin,kmax,nsol,Waldval,"anneal",Numprb=Fortout[[31]]) 
  output <- prepRetOtp(c(Fortout[6:9],call=match.call()),kmin,kmax,nsol,Waldcrt,Waldval,"anneal",Numprb=Fortout[[31]]) 
  if (is.null(output)) return(invisible(NA))
	output   # return(output)
}

validation<-function(mat, kmin, kmax, exclude, include, criterion, pcindices, tolval, tolsym, algorithm)
{

##########################################################
#  general validation of input for all search functions  #
##########################################################

         if (tolval < 0) stop("\n The 'tolval' argument must be non-negative.")

#################################################
# checking acceptability of criterion requested #
#################################################
         
 	labelsrm<-c("RM","Rm","rm","1",1)
       	labelsrv<-c("RV","Rv","rv","2",2)
       	labelsgcd<-c("GCD","Gcd","gcd","3",3)
	labelstau<-c("TAU_2","Tau_2","tau_2","TAU2","Tau2","tau2","TAU","Tau","tau","WILKS","Wilks","wilks")
	labelsxi<-c("XI_2","Xi_2","xi_2","XI2","Xi2","xi2","XI","Xi","xi","BARTLLET","Bartllet","bartllet")
	labelszeta<-c("ZETA_2","Zeta_2","zeta_2","ZETA2","Zeta2","zeta2","ZETA","Zeta","zeta","HOTELLING","Hotelling","hotelling")
       	labelsccr1<-c("CCR1_2","Ccr1_2","ccr1_2","CCR12","Ccr12","ccr12","CCR2","Ccr2","ccr2","CCR1","Ccr1","ccr1","ROY","Roy","roy")
	labelswald<-c("WALD","Wald","wald")
         	
	if (sum(criterion == c(labelsrm,labelsrv,labelsgcd,labelstau,labelsxi,labelszeta,labelsccr1,labelswald)) == 0) 
		stop("criterion requested is not catered for, or has been misspecified\n")

       	if (sum(criterion == labelsrm) > 0) criterio<-1
       	if (sum(criterion == labelsrv) > 0) criterio<-2
       	if (sum(criterion == labelsgcd) > 0) criterio<-3
       	if (sum(criterion == labelstau) > 0) criterio<-4
       	if (sum(criterion == labelsxi) > 0) criterio<-5
       	if (sum(criterion == labelszeta) > 0) criterio<-6
       	if (sum(criterion == labelsccr1) > 0) criterio<-7
       	if (sum(criterion == labelswald) > 0) criterio<-8
      		
	if (criterio == 1)  criterion <- "RM"
	if (criterio == 2)  criterion <- "RV"
	if (criterio == 3)  criterion <- "GCD"
	if (criterio == 4)  criterion <- "TAU_2"
	if (criterio == 5)  criterion <- "XI_2"
	if (criterio == 6)  criterion <- "ZETA_2"
	if (criterio == 7)  criterion <- "CCR1_2"
	if (criterio == 8)  criterion <- "WALD"

####################################################################
# checking for an input matrix that must be square, of full rank,  #
#    symmetric, positive definite  - calls the 'validmat' routine  #
####################################################################

         p <- dim(mat)[2]

        if (criterion=="TAU_2" || criterion=="XI_2" || criterion=="ZETA_2" || criterion=="CCR1_2")
        {   		
         	if (validmat(mat,p,tolval,tolsym,allowsingular=TRUE,algorithm) == "singularmat")  singularmat <- TRUE  	
         	else singularmat <- FALSE						 			
	}                                                				 			
        else {						 							
		validmat(mat,p,tolval,tolsym,allowsingular=FALSE,algorithm)   						
         	singularmat <- FALSE						 				
	}					 								
        
######################################################################
#  checking consistency of the requested values of kmin, kmax        #
#  (extreme sizes of variable subsets). The validation that kmax<=p  #
#   is made later, considering the no. of excluded variables         #
######################################################################         

         if (!is.numeric(kmin) || !is.numeric(kmax)) stop("\n Arguments kmin and kmax must be numeric.\n Perhaps unnamed arguments in the wrong order?")
         if (kmax < kmin) {
          aux<-kmin
          kmin<-kmax
          kmax<-aux
          warning("the argument kmin should precede the argument kmax.\n Since the value of kmin exceeded that of kmax, they have been swapped \n")}
          if (kmin != floor(kmin)) {
                  kmin<-floor(kmin)
                  warning("\n The value of 'kmin' requested was not an integer. It has been truncated. \n")
            }                  
          if (kmin >= p) {
             kmin<-p-1
             warning("\n The value of kmin requested is equal to or exceeds the number \n of variables. It has been set at p-1. \n")
            }
          if (kmin < 1) {
             kmin <- 1
             warning("\n The value of kmin requested is less than 1. It has been set at 1. \n")
            }
          if (kmax != floor(kmax)) {
                  kmax<-floor(kmax)
                  warning("\n The value of 'kmax' requested was not an integer. It has been truncated. \n")
            }   
         if (kmax >= p) {
         	kmax<-p-1
         	warning("\n The value of kmax requested is equal to or exceeds the number \n of variables. It has been set at p-1. \n")
            }

#############################################################
# checking for consistency of requests to exclude/include   #
#  certain variables.                                       #
#############################################################

         if (sum(duplicated(c(exclude,include))) > 0)
          stop("\n You have requested that the same variable be both included in, and excluded \n from, the subset.\n")
         nexclude<-length(exclude)
         if (nexclude !=0) {
         if (kmax >= p-nexclude) {
                                  kmax<-p-nexclude-1
                                  warning("\n Cardinalities requested are too large for the requested number of excluded \n variables, and kmax has been set at p-nexclude-1 \n")}
         exclude<-sort(exclude)}  
         exc<-c(0,exclude)
         ninclude<-length(include)
         if (ninclude !=0){
         if (kmin <= ninclude) {kmin<-ninclude+1
                                warning("\n Cardinalities requested are too small for the requested number of included \n variables, and kmin has been set to ninclude + 1 \n")}
         include<-sort(include)}
         inc<-c(0,include)
         if (kmax<kmin) stop("\n After trying to adapt to the requests for exclusion and inclusion of \n variables, kmax is now smaller than kmin. There must be a mistake.\n")


######################
# Checking pcindices #
######################


         
# Value always assigned to esp and to non-numeric pcindices to enable passing to parent.frame and to Fortran or C++ routines, for criteria other than GCD

         esp<-FALSE
         if (criterio == 3) { 
                            if (is.null(pcindices)) stop("\n For criterion GCD, argument pcindices must be non-NULL. \n")
                            if (is.numeric(pcindices))  {esp<-TRUE
                                    if  (sum(!(as.integer(pcindices) == pcindices)) > 0) stop("\n The PC indices must be integers.\n")
                                    if (max(pcindices)  >  p) stop("\n PCs of rank larger than the data set were requested. \n")
                                                        }
                            else {if (pcindices[1] != "first_k")
                                    {stop("\n unrecognized value for 'pcindices' argument \n")}}}
         if ((pcindices[1] == "first_k") || is.null(pcindices)) {pcindices<-1:kmax}

#################################
# assigning any changed values  #
#################################
         
         assign("mat",mat,pos=parent.frame())         
         assign("kmax",kmax,pos=parent.frame())         
         assign("kmin",kmin,pos=parent.frame())         
         assign("exclude",exclude,pos=parent.frame())         
         assign("include",include,pos=parent.frame())         
         assign("nexclude",nexclude,pos=parent.frame())         
         assign("ninclude",ninclude,pos=parent.frame())         
         assign("criterio",criterio,pos=parent.frame())   
         assign("criterion",criterion,pos=parent.frame())    
         assign("pcindices",pcindices,pos=parent.frame())         
         assign("p",p,pos=parent.frame())                  
         assign("exc",exc,pos=parent.frame())         
         assign("inc",inc,pos=parent.frame())         
         assign("esp",esp,pos=parent.frame())

         if (singularmat==TRUE) return("singularmat")   
         else return("validmat")			
       }

improve<-function(mat, kmin, kmax=kmin, nsol=1, exclude=NULL,
include=NULL, setseed = FALSE, criterion="default", pcindices="first_k",
initialsol=NULL, force=FALSE, H=NULL,r=0, tolval=1000*.Machine$double.eps,
tolsym=1000*.Machine$double.eps)
{

#########################################################################################
# auxiliary  variables (includes declarations to avoid "no visible binding" NOTE)       #
#########################################################################################
  
	p <- ncol(mat)    				# Number of original variables
	nexclude <- length(exclude)     		# Number of excluded variables
	ninclude <- length(include)     		# Number of included variables				 
	if (pcindices[1]!="first_k") esp <- TRUE		# The user has specified the set of Principal Components to be used with the GCD criterion
	else esp <- FALSE				# The user has not specified the set of Principal Components to be used with the GCD criterion
	if (!is.null(initialsol)) silog <- TRUE		# The user has specified a set of initial solutions
	else silog <- FALSE				# The user has not specified a set of initial solutions
	exc <- exclude					# Name for vector of excluded variables after 'validation' routine 
	inc <- include					# Name for vector of included variables after 'validation' routine 

        
###############################
# general validation of input #
###############################

  
        initialization(mat, criterion, r)
         if (validation(mat, kmin, kmax, exclude, include, criterion, pcindices, tolval, tolsym, "improve") == "singularmat")  singularmat = TRUE  
         else singularmat = FALSE         
        maxnovar = 400
        if ((p > maxnovar) & (force==FALSE)) stop("\n For very large data sets, memory problems may crash the R session. \n To proceed anyways, repeat the function call with \n the argument 'force' set to 'TRUE' (after saving anything important \n from the current session)\n")


######################################################################
# Parameter validation if the criterion is one of "TAU_2", "XI_2",   #
# "ZETA_2" or "CCR1_2"  or "WALD"                                    #
######################################################################

  if (criterion == "TAU_2" || criterion == "XI_2" || criterion == "ZETA_2" || criterion == "CCR1_2" || criterion == "WALD") validnovcrit(mat,criterion,H,r,p,tolval,tolsym)


##########################################################################
# more specific validation of input for the anneal and improve functions #
##########################################################################

  implog<-TRUE
  validannimp(kmin, kmax, nsol, exclude, nexclude, include, ninclude, initialsol, implog)


###########################################
# initializations for Fortran subroutine  #
###########################################

       # Normalize matrices in order to improve numerical accuracy
	
  if (r==0) mat <- mat/max(mat)
  else {
    scl <- sqrt(diag(mat)) 
    sclotprd <- outer(scl,scl)
    mat <- mat/sclotprd
    H <- H/sclotprd
  }  

  if (setseed == TRUE) set.seed(2,kind="default")
  valores<-rep(0.0,(kmax-kmin+1)*nsol)
  vars<-rep(0,nsol*length(kmin:kmax)*kmax)
  bestval<-rep(0.0,length(kmin:kmax))
  bestvar<-rep(0,kmax*length(kmin:kmax))
  if (criterio == 3) {
    decespectral<-eigen(mat,symmetric=TRUE)
    valp<-decespectral$values
    vecp<-decespectral$vectors
  } else {
    valp<-rep(0,p)
    vecp<-matrix(nrow=p,ncol=p,rep(0,p*p))
  }

###############################
# call to Fortran subroutine  #
###############################

  if (criterion == "WALD")   {

###       Convert a min Wald problem into an (artificial) equivalent max XI2 problem
        
    Waldval <- sum(diag(solve(mat,H)))
    H <- H / Waldval     
    criterion <- "XI_2"
    criterio <- 5
    Waldcrt <- TRUE
  }  else {
    Waldcrt <- FALSE
    Waldval <- NULL
  }  
        
  Fortout<-.Fortran("improve",as.integer(criterio),as.integer(p),
          as.double(as.vector(mat)),
          as.integer(kmin),as.integer(kmax),as.double(valores),
          as.integer(vars),as.double(bestval),as.integer(bestvar),
          as.integer(nexclude),as.integer(exc),as.integer(ninclude),
          as.integer(inc),as.integer(nsol),
          as.integer(length(pcindices)),as.integer(pcindices),
          as.logical(esp),as.logical(silog),
          as.integer(as.vector(initialsol)),as.double(valp),
          as.double(as.vector(vecp)),as.double(as.vector(H)),as.integer(r),
	  as.logical(singularmat),as.double(tolval),logical(1),
	  PACKAGE="subselect")

#######################################
# Preparing and returning the output  #
#######################################

#  output <- prepRetOtp(c(Fortout[6:9],call=match.call()),kmin,kmax,nsol,Waldval,"improve",Numprb=Fortout[[26]]) 
  output <- prepRetOtp(c(Fortout[6:9],call=match.call()),kmin,kmax,nsol,Waldcrt,Waldval,"improve",Numprb=Fortout[[26]]) 
  if (is.null(output)) return(invisible(NA))
	output   # return(output)
}

* SUBPROGRApsMS:
* Included in the R/S package "subselect", available on CRAN.
********************************************************************
       subroutine newinicializar(criterio,p,s,svector,sq,nfora,
     +  fora,ndentro,dentro,fica,tracos,tracosq,vecp,poriginal,vecvecp,
     +   h,hvector,rh)
**********************************************************************
* performs some initial chores: converts svector into a matrix s; computes
* the square of that matrix, sq, and the traces of both s (for RM) and sq 
* (for RV); converts the eigenvectors of s (for GCD), calculated within R 
* and passed on as vector vecvecp, into a matrix. All this is done taking 
* into account the variables that are to be 
* forcibly excluded (vector fora) or included (vector dentro).
*
*
* INPUT:
*
* criterio - integer variable indicating the criterion for the quality of 
*            the k-subset.
*    p     - integer variable, giving the number of original variables.
* svector  - double precision vector, giving the full covariance (or 
*            correlation) matrix of the p variables (given as a vector
*            for convenience in passing from R to Fortran). 
*  nfora   - integer variable indicating the number of original variables 
*            that are to be forcefully excluded from the k-subset.
*   fora   - integer vector, indicating the variable numbers associated
*            with the nfora variables that are to be excluded by force.
* ndentro  - integer variable indicating the number of original variables 
*            that are to be forcefully included in the k-subset.
*  dentro  - integer vector, indicating the variable numbers associated
*            with the ndentro variables that are to be included by force.
* vecvecp  - double precision vector, giving the eigenvectors of the 
*            covariance (or correlation) matrix of the p variables (given 
*            as a vector for convenience in passing from R to Fortran).
* hvector  - double precision vector, giving the full effect descrption 
*            matrix (H) of the p variables (given as a vector for
*            convenience in passing from R to Fortran). 
*   rh     - H matrix rank (expected)
*
* OUTPUT:
*
*   p      - integer variable indicating the number of admissible variables,
*            i.e., the original number of variables p minus nfora.
*   s      - double precision array (pxp) giving the covariance (or 
*            correlation) matrix of the admissible variables.
*   sq     - double precision array (pxp) giving the square of matrix s.
*  fica    - integer vector giving the original variable numbers of the 
*            admissible variables.
* tracos   - double precision variable giving the trace of matrix s.
* tracosq  - double precision variable giving the trace of matrix sq.
*  vecp    - double precision array (pxp) giving the eigenvectors of s (in
*            the same column-order as the eigenvalues).
* poriginal - integer variable, indicating the original number of variables
*             (poriginal = p + nfora)
*   h       - double precision array (pxp) giving the full effect descrption 
*            matrix (H)  
*

* general declarations
      integer criterio,i,iaux,j,nfora,ndentro,nfica
      integer p,poriginal
      integer fora(0:nfora),fica(0:p),dentro(0:ndentro)
      double precision s(p,*),sq(p,*),svector(*)
      double precision vecvecp(*)
* declaration only for the RM criterion
      double precision tracos
* declarations only for the RV criterion
      double precision tracosq
* declarations only for the GCD criterion
      double precision vecp(p,*)
* declarations only for the tau2,xi2,zeta2 and ccr12 criteria
      integer rh
      double precision hvector(*),h(p,*)

      external dprodmat

* conversion of svector into a matrix (s); if GCD, initialization of the vecp
* matrix which will hold the eigenvectors of s (computed in R and passed 
* in as vector vecvecp)
       do j=1,p
         DO i= 1,p
           s(i,j) = svector(i+(j-1)*p)
           if (criterio.eq.3) then 
              vecp(i,j) = vecvecp(i+(j-1)*p)
           end if 
         END DO
      end do

       If (rh .GT. 0) then
* conversion of hvector into a matrix (h) 
       do j=1,p
         DO i= 1,p
           h(i,j) = hvector(i+(j-1)*p)
          END DO
        end do
        end if


* Defines matrix sq := s x s. Computes the traces of s (for RM) and 
* sq (for RV), and the eigendecomposition of s (for GCD)


      call dprodmat(p,p,s,p,s,sq)


      if (criterio .eq. 1) then 
         tracos=0.0D0
         DO j=1,p
           tracos=tracos+s(j,j)
         END DO
      end if
      if (criterio .eq. 2) then 
         tracosq=0.0D0
         DO j=1,p
           tracosq=tracosq+sq(j,j)
         END DO
      end if


* Re-defines the matrices s, sq and h (if r>0), excluding the rows/columns that the 
* user has requested be excluded from the final solution. 
* p is redefined as p-nfora, where nfora is the number of variables that 
* are to be excluded. The original variable-number for each of the surviving
* variables is preserved in vector fica(j), j=1,p-nfora.
* Warning: the original matrices s and sq are destroyed.

      poriginal=p
      p=p-nfora
      fora(0)=0
      nfica=0
      fica(0)=0
      do j=0,nfora-1
         do i=fora(j)+1,fora(j+1)-1
            nfica=nfica+1
            fica(nfica)=i
         end do
      end do
      iaux=fora(nfora)
      do j=iaux+1,poriginal
         nfica=nfica+1
         fica(nfica)=j
      end do

      if(nfora .GT. 0) then
         do i=1,p-1
            do j=i+1,p
              s(i,j)=s(fica(i),fica(j))
              s(j,i)=s(i,j)
              sq(i,j)=sq(fica(i),fica(j))
              sq(j,i)=sq(i,j)
              if(rh .GT. 0) then
                h(i,j)=h(fica(i),fica(j))
                h(j,i)=h(i,j)
              end if
            end do
            s(i,i)=s(fica(i),fica(i))
            sq(i,i)=sq(fica(i),fica(i))
		    if(rh .GT. 0) then
		       h(i,i)=h(fica(i),fica(i))
                    end if
         end do
         s(p,p)=s(fica(p),fica(p))
         sq(p,p)=sq(fica(p),fica(p))
         if(rh .GT. 0) then
		    h(p,p)=h(fica(p),fica(p))
          end if 
      end if

* The vector of variables that are forced to be included in the solution 
* is redefined, taking into account that nfora variables (may) have been 
* excluded.

      if((ndentro.GT.0) .and. (nfora.GT.0)) then
         i=1
         j=1
         do while(j.LE.ndentro)
           if(dentro(j).EQ.fica(i)) then
             dentro(j)=i
             j=j+1
           end if
           i=i+1
         end do
      end if
      return
      end
**********************************************************************

*********************************************************************
      subroutine randsk1(n,k,sk,pp)
**********************************************************************
* generates a subset of k random integers from the set {1,2,...,n}. 
* Warning: uses function randint.

*  INPUT: 
*    iseed
*    n   - integer variable, largest integer in the set.
*    k   - integer variable, giving the number of random integers that
*                are to be generated
*    pp  - integer vector of n elements, used to store temporary results
* 
*  OUTPUT: 
*   sk   - logical vector defining the generated subset 
*               (sk(i)=.true. iff i belongs to sk, i=1,...,n)

       integer i,k,n,nalt,randint
       integer pp(*)
       logical sk(*)

       do i=1,n
        sk(i)=.false.
        pp(i)=i
       end do
       do i=1,k
        nalt=randint(i,n)
        sk(pp(nalt))=.true.
        pp(nalt)=pp(i)
       end do
       return 
      end
***********************************************************************

************************************************************************
      integer function randint(esq,dto)
**********************************************************************
* randomly generates a number in the interval [esq,dir].
*
* WARNING: Uses a call to R's random number generator (as described
*  in pages 48 and 50-51 of the "Writing R Extensions" manual, version
*  1.5).
*
*INPUT: 
       
*        esq  - integer variable, giving lower bound of interval.
*        dto  - integer variable, giving upper bound of interval.

       external unifrnd, rndstart, rndend
       
       double precision ran, unifrnd
       integer esq,dto

       call rndstart()
       ran = unifrnd()
       randint=esq+int(ran*(dto-esq+1))
       call rndend()
       return 
      end
*********************************************************************

*********************************************************************
      subroutine dprodmat(n,m,a,r,b,prod)
**********************************************************************
* calculates the matrix product of two matrices
*
* INPUT : n - integer, number of rows of matrix a
*         m - integer, number of columns of matrix a (= number of rows
*                matrix b)
* 
*         a - double precision array (nxm)
*         r - integer, number of columns of matrix b
*         b - double precision array (mxr)
* OUTPUT: 
*   prod - double precision array (nxr), the product a x b.

       INTEGER i,j,l,m,r
       DOUBLE PRECISION a(n,*),b(m,*),prod(n,*),soma

       DO i=1,n
         DO j=1,r
           soma=0.0D0 
        	DO l=1,m
	          soma=soma+a(i,l)*b(l,j)
	        END DO
	        prod(i,j)=soma
         END DO
       END DO
       RETURN
      END
**********************************************************************

**********************************************************************
       subroutine dcorrigesk(ndentro,dentro,p,setk,poriginal)
**********************************************************************
* subroutine randsk1, randomly selects (with uniform 
* distribution) a subset (setk) with cardinality k-ndentro  of the set 
* {1,2,...,p-ndentro}. This routine takes that subset and merges it with 
* the ndentro variables indicated in vector dentro to produce a subset of
* cardinality k=|setk|+ndentro, whose variable numbers are in accordance with 
* the original variable numbers. The vector setk is re-defined accordingly.
* Thus, the two subroutines randsk1and dcorrigesk randomly select a subset
* of cardinality k from the set {1,2,...,p}, ensuring that the ndentro 
* variables specified by dentro are included.
*
* INPUT: 
*
* ndentro - integer variable giving the size of vector dentro.
* dentro  - integer vector giving the variable numbers of variables that are
*           to be forcefully included in the k-subset.
*   p     - number of elements in the full set (excluding 'exclude' variables).
*  setk   - logical vector with setk(j)=.true. iff j belongs to the  
*           subset of {1,2,...,p-ndentro}
* poriginal - number of variables originally (equal to 'p', except when
*              the 'exclude' option has been used
* 
* OUTPUT: 
* 
*  setk   - logical vector with setk(j)=.true. iff j belongs to the  
*           subset of {1,2,...,p}

       integer i,j,naux,ndentro,p,poriginal
       integer dentro(0:ndentro)
       logical setk(*)

       dentro(0)=0
       naux=ndentro
       i=p-ndentro
       j=p
       do while(j.GE.1)
         if(dentro(naux).EQ.j) then
           setk(j)=.true.
	   naux=naux-1
	 else
	   setk(j)=setk(i)
	   i=i-1
         end if
         j=j-1
       end do
       return
      end
**********************************************************************

************************************************************************
	subroutine dannealing(criterio,p,kmax,setk,vactual,ndentro,
     +                dentro,niter,beta,temp,coolfreq,k,s,tracos,sq,
     +                tracosq,nqsi,qsi,valp,vecp,fica,poriginal,
     +                h,rh,skinput,hkinput,ekinput,egval,work,workmat,
     +                randsk1pp,setint,checksg,numtol,numprb)
**********************************************************************

* 
* Applies simulated annealing to a subset setk of the original variables. 
*
*
* INPUT : 
*
*criterio - integer variable indicating the criterion of subset quality
*            requested.
*   p     - integer variable giving the number of original variables (minus
*               ndentro).
* kmax    - integer variable, giving the largest cardinality
*   k     - integer variable, cardinality of subset.
*   s     - double precision matrix of covariances of all p variables.
*   sq    - double precision matrix, the square of s. 
* setk    - logical vector, indicating a given subset of variables. 
*           setk(j)=.true. iff variable j belongs to the subset. Changed
*           on output.
*ndentro  - integer variable, giving the number of variables that are to
*          be forcefully included in the output subset.
* dentro  - integer vector, indicating the numbers associated with the 
*          ndentro variables that are to be forced into the subsets.
* niter   - integer variable, number of simulated annealing iterations 
*           requested for each solution.
*  beta   - double precision variable,(between 0 and 1) indicating the
*           geometric cooling factor for simulated annealing.
* temp   - double precision variable, indicating the initial temperature
*           to e used in the simulated annealing algorithm.
* coolfreq - integer indicating the cooling frequency, i.e., the number 
*           of iterations that are to be carried out for each fixed
*           temperature 
* tracos  - double precision variable, the trace of the covariance (or
*           correlation) matrix s. (For RM criterion only).
* tracosq - double precision variable, the trace of the matrix sq. (For RV
*           criterion only).
*  nqsi   - integer variable, indicating the dimension of the Principal 
*           Subspace with which the subspace spanned by the k-subset is
*            to be compared. (For GCD criterion only).
*  qsi    - integer vector, giving the ranks of the Principal Components 
*           which span the Principal Subspace described above (see nqsi).
*           (For GCD criterion only).
*  valp   - double precision vector giving the eigenvalues of s, in 
*            decreasing order. (For GCD criterion only).
*  vecp   - double precision array (pxp) giving the eigenvectors of s, in
*            the same column-order as the values of valp. (For GCD only).
*  fica   - integer vector giving the original variable numbers of the 
*            admissible variables.
* poriginal - 
*
*   h     - double precision matrix giving the full effect descrption 
*            matrix (H)  
*
*   rh     - integer variable giving the expected rank of H matrix.
* checksg - logical, flag indicating if a test for ill-conditioned 
*	    (E being numerically singular) problems should be implemented
*  numtol - double precision tolerance level for the ratio of the diagonal 
*           elements of E Choleski decomposition

*
* OUTPUT: 
*
* setk    - logical vector indicating the best subset produced by simulated 
*           annealing after the niter iterations. setk(j)=.true. iff variable 
*           j belongs to the subset. 
*vactual  - double precision variable giving the best value of the criterion 
*           produced throughout the niter iterations.
* numprb -  logical flag set to true if numerical problems precluded some
*           (ill-conditioned) subsets to be evaluated, and to false otherwise.   

*  WORKING ARRAYS:
*
*  skinput   - double precision matrix (kmax*kmax) used to store the submatrices S_k. 
*  hkinput   - double precision matrix (kmax*kmax) used to store the submatrices H_k. 
*  ekinput   - double precision matrix (kmax*kmax) used to store the submatrices E_k. 
*  egval     - double precision array (kmax), used to store eigenvalues.
*  work      - double precision array (6*kmax), to be used internaly by LAPACK routines.
*  workmat   - double precision matrix (kmax*kmax) used to store temporary results.
*  randsk1pp - integer array(p) to be used internaly by the randsk1 routine.
*  setint    - integer array(p) used to store the indices of the variables kept in each subset.
        
* general declarations
        integer coolfreq,criterio,i,inaoconv,iter,j,jentra,jsai,k,kcons
        integer kque,ncons,ndentro,nigual,niter,nque,p,poriginal,kmax
        integer dentro(0:ndentro),que(p),cons(p),randint
	logical checksg,numprb
        logical setk(*),setkmax(poriginal),auxlog(p)
        double precision vactual,vmax,vtroca,dir,numtol,beta,citer,temp
        double precision unifrnd,r
        double precision s(poriginal,*)
* declarations specific to the RM and RV criteria
        double precision sq(poriginal,*),tracos,tracosq,dobjrm,dobjrv
* declarations specific to the GCD criterion
        integer qsi(*),fica(0:p),nqsi
        double precision valp(*)
        double precision vecp(poriginal,*),dobjgcd
* declarations specific to the tau2,xi2,zeta2 and ccr12 criteria
        integer rh
        double precision h(poriginal,*)
        double precision dobjtau2,dobjxi2,dobjzeta2,dobjccr12
* declarations of local arrays and matrices
       INTEGER randsk1pp(*),setint(*)
       DOUBLE PRECISION skinput(kmax,*),hkinput(kmax,*)
       DOUBLE PRECISION ekinput(kmax,*),workmat(kmax,*)
       DOUBLE PRECISION work(*),egval(*)

        external unifrnd, rndstart, rndend

* initializing 
        citer = temp
        vtroca = -1.0D0
        dir = 0.0D0
        numprb=.false.

        do j=1,p
         auxlog(j)=.true.
        end do
        do j=1,ndentro
         auxlog(dentro(j))=.false.
        end do

        ncons=0
        nque=0
        do j=1,p
           if(.not.setk(j)) then
              nque=nque+1
	      que(nque)=j
	   else
	      if(auxlog(j)) then
	        ncons=ncons+1
	        cons(ncons)=j
	      end if
           end if
        end do

        vmax=vactual
        do j=1,p
          setkmax(j)=setk(j)
        end do

* more initializing
        iter=0      
        nigual=0
        inaoconv=0
        do while(iter.LT.niter)
         
 
* generates a (uniform) random integer, kque, in {1,2,...,nque}.
* jentra, the variable which is being considered for inclusion in the subset
* setk, is the element in position kque of the vector que 

         kque=randint(1,nque)
         jentra=que(kque)
  
* generates a (uniform) random integer, kcons, in {1,2,...,kcons}.
* jsai, the variable which is being considered for exclusion from the subset
* setk, is the element in position kcons of the vector cons, i.e., an 
* element of setk not belonging to vector dentro

         kcons=randint(1,ncons)
         jsai=cons(kcons)

* calculates the value of the criterion for solution setk\{jsai}U{jentra}

         setk(jsai)=.false.
         setk(jentra)=.true.
         if (criterio.eq.1) then
	    vtroca=dobjrm(k,setk,p,poriginal,kmax,s,sq,workmat,
     +   setint,skinput)
         end if
         if (criterio.eq.2) then
           vtroca=dobjrv(k,setk,p,poriginal,kmax,s,sq,workmat,
     +   setint,skinput)
         end if
         if (criterio.eq.3) then
            vtroca=dobjgcd(k,setk,p,poriginal,kmax,s,workmat,
     +   nqsi,qsi,valp,setint,vecp,fica,skinput)
        end if
         if (criterio.eq.4) then
               vtroca=dobjtau2(k,setk,p,poriginal,kmax,s,h,rh,checksg,
     +   numtol,setint,work,egval,skinput,hkinput,ekinput,workmat)
            end if
	   if (criterio.eq.5) then
               vtroca=dobjxi2(k,setk,p,poriginal,kmax,s,h,rh,checksg,
     +   numtol,setint,work,egval,skinput,hkinput,workmat)
           end if
	   if (criterio.eq.6) then
               vtroca=dobjzeta2(k,setk,p,poriginal,kmax,s,h,rh,checksg,
     +   numtol,setint,work,egval,skinput,hkinput,ekinput,workmat)
           end if		 
	  if (criterio.eq.7) then
* tirei rh
               vtroca=dobjccr12(k,setk,p,poriginal,kmax,s,h,checksg,
     +   numtol,setint,work,egval,skinput,hkinput,workmat)
           end if

         if (checksg.and.criterio.GT.3.and.vtroca.eq.-0.9999D0) then
	   numprb=.true.        
         end if

* if vtroca>vmax, updates vmax and setkmax, which hold the best value and 
* subset found so far.

         if(vtroca.GT.vmax) then
          vmax=vtroca
          do j=1,p
	    setkmax(j)=setk(j)
	  end do
         end if

* The replacement is carried out if vtroca>=vactual or, if not, with 
* probability given by the simulated annealing algorithm, and computed
* by R's Random Number Generator

         call rndstart()
         r = unifrnd()
         call rndend()

         if(vtroca-vactual.GE.0) then 
          dir=0.1D5
         else
          if (criterio.eq.1) then
           dir=exp((dsqrt(vtroca)-dsqrt(vactual))/(dsqrt(tracos)*citer))
          end if
          if (criterio.eq.2) then
          dir=exp((dsqrt(vtroca)-dsqrt(vactual))/(dsqrt(tracosq)*citer))
          end if
          if (criterio.eq.3) then
          dir=exp(((vtroca/dsqrt(DBLE(nqsi*k)))-
     +         (vactual/dsqrt(DBLE(nqsi*k))))/citer)
          end if
	    if (criterio.eq.4) then
          dir=exp((vtroca-vactual)/citer)
          end if
		if (criterio.eq.5) then
          dir=exp((vtroca-vactual)/citer)
          end if
		if (criterio.eq.6) then
          dir=exp((vtroca-vactual)/citer)
          end if
		if (criterio.eq.7) then
          dir=exp((vtroca-vactual)/citer)
          end if
         end if
         if(vtroca.GE.vactual.or.(vtroca.LT.vactual.and.r.LT.dir)) then 
* swap

          vactual=vtroca
          que(kque)=jsai
          cons(kcons)=jentra

           nigual=0
        else 
* reject the swap
           setk(jsai)=.true.
           setk(jentra)=.false.

          nigual=nigual+1 
* counts number of consecutive rejections of a swap
        end if

* Hacking the temperature:
* updates the temperature citer every coolfreq iterations

        if(mod(iter,coolfreq).EQ.0) citer=citer*(1-beta)


* when there are more than p consecutive rejections of a swap and the
* first 20p iterations have not yet been completed, the
* simulated annealing temperature citer will be increased: by a factor 
* of 2 if less than 5p iterations have gone by, a factor of 1.5 if the
* number of iterations is between 5p and 10p, by a factor of 1.1 if
* the current number of iterations is between 10p and 20p. 

       if(nigual.GE.p) then
         if(iter.LE.5*p) then
              citer=citer*2.0D0
	 else
	      if(iter.LE.10*p) then
	        citer=citer*1.5D0
	      else
	        if(iter.LE.20*p) then
	            citer=citer*1.1D0
	        else
		    inaoconv=inaoconv+1
	        end if
	       end if
          end if
          nigual=0
        end if

* inaoconv counts the number of times in which the currently best solution 
* has remained unchanged after the first 20p iterations.

* inaoconv counts the number of consecutive iterations without changes in
* the current solution AFTER THE FIRST 20p ITERATIONS. If this becomes too
* large (the value chosen here was 500), the algorithm stops (a further change
* is unlikely, given the low temperature after so many iterations).

         if (inaoconv.GT.500) iter=niter
         iter=iter+1
      end do
 
* defines the algorithm's solution, vactual, as the best solution obtained
* at any stage of the niter iterations. 

       vactual=vmax
       do i=1,p
          setk(i)=setkmax(i)
       end do

       return
      end
**********************************************************************

**********************************************************************
	subroutine dmelhoramentogen(criterio,p,kmax,setk,vactual,ndentro,
     +                dentro,k,s,sq,nqsi,qsi,valp,vecp,fica,poriginal,
     +                h,rh,skinput,hkinput,ekinput,egval,work,workmat,
     +                randsk1pp,setint,checksg,numtol,numprb)

**********************************************************************
* This subroutine (which is only called by the "improve" subroutine, 
* but also by the other two main routines - "genetic" and "anneal" -  if 
* the logical variable "improvement" is set to .true.) seeks to improve an 
* initial k-subset by a modified local search algorithm, the details of 
* which are as follows. The variables not belonging to this initial subset 
* are placed in a queue ("que"). This subroutine explores the possibility 
* of replacing a variable in the subset with a variable from the queue. 
* More precisely, a variable j is selected and removed from the queue. Each 
* variable i in the subset is, in turn, replaced by variable j and the 
* resulting values of the criterion are computed. If the best of these k 
* new criterion values exceeds the subset's original criterion value, the 
* current solution is updated accordingly. In this case, the variable which
* leaves the subset is added to the queue, but only if it has not previously 
* been in the queue (i.e., no variable can enter the queue twice). The 
* algorithm proceeds until the queue is emptied. 
*
* INPUT: 
*criterio - integer variable indicating the criterion of subset quality
*            requested.
*   p     - integer variable, number of original variables. 
*  kmax   - integer variable, giving the largest cardinality
*           of the subsets of variables that are wanted.
* setk    - logical vector, indicating a given subset of variables. 
*           setk(j)=.true. iff variable j belongs to the subset. Changed
*           on exit. 
*vactual  - double precision variable giving the best value of the criterion 
*           for the initial solution. Changed on exit.
*ndentro  - integer variable, giving the number of variables that are to
*          be forcefully included in the output subset.
* dentro  - integer vector, indicating the numbers associated with the 
*          ndentro variables that are to be forced into the subsets.
*   k     - integer, cardinality of subset setk.
*   s     - double precision matrix of covariances of all p variables.
*   sq    - double precision matrix, the square of s.
*  nqsi   - integer variable, indicating the dimension of the Principal 
*           Subspace with which the subspace spanned by the k-subset is
*            to be compared. (For GCD criterion only).
*  qsi    - integer vector, giving the ranks of the Principal Components 
*           which span the Principal Subspace described above (see nqsi).
*           (For GCD criterion only).
*  valp   - double precision vector giving the eigenvalues of s, in 
*            decreasing order. (For GCD criterion only).
*  vecp   - double precision array (pxp) giving the eigenvectors of s, in
*            the same column-order as the values of valp. (For GCD only).
*  fica   - integer vector giving the original variable numbers of the 
*            admissible variables.
* poriginal - 
*
*   h     - double precision matrix giving the full effect descrption 
*            matrix (H)  
*
*   rh     - integer variable giving the expected rank of H matrix.
* checksg - logical, flag indicating if a test for ill-conditioned 
*	    (E being numerically singular) problems should be implemented
*  numtol - double precision tolerance level for the ratio of the diagonal elements of E Choleski decomposition.
*
*
* OUTPUT: 
*
* setk    - logical vector indicating the best subset after the modified local
*           search, with setk(j)=.true. iff variable j belongs to the subset. 
*vactual  - double precision variable giving the best value of the criterion 
*           after the modified local search.
* numprb -  logical flag set to true if numerical problems precluded some
*           (ill-conditioned) subsets to be evaluated, and to false otherwise.   

*  WORKING ARRAYS:
*
*  skinput   - double precision matrix (kmax*kmax) used to store the submatrices S_k. 
*  hkinput   - double precision matrix (kmax*kmax) used to store the submatrices H_k. 
*  ekinput   - double precision matrix (kmax*kmax) used to store the submatrices E_k. 
*  egval     - double precision array (kmax), used to store eigenvalues.
*  work      - double precision array (6*kmax), to be used internaly by LAPACK routines.
*  workmat   - double precision matrix (kmax*kmax) used to store temporary results.
*  randsk1pp - integer array(p) to be used internaly by the randsk1 routine.
*  setint    - integer array(p) used to store the indices of the variables kept in each subset.


* general declarations
      integer criterio,k,kmax,j,jconsmax,jentra,jmax,jsai
      integer ncons,ndentro,nque,p,poriginal
      integer dentro(0:ndentro),que(p),cons(p)
      logical setk(*),auxlog(p),esteveque(p)
      logical checksg,numprb
      double precision s(poriginal,*)
      double precision vactual,vtroca,vtrocamax,numtol
* declarations specific to the RM and RV criteria
      double precision sq(poriginal,*),dobjrm,dobjrv
* declarations specific to the gcd criteria
        integer qsi(*),fica(0:p),nqsi
        double precision valp(*),vecp(poriginal,*),dobjgcd
* declarations specific to the tau2,xi2,zeta2 and ccr12 criteria
        integer rh
        double precision h(poriginal,*)
        double precision dobjtau2,dobjxi2,dobjzeta2,dobjccr12
* declarations of local arrays and matrices
       INTEGER randsk1pp(*),setint(*)
       DOUBLE PRECISION skinput(kmax,*),hkinput(kmax,*)
       DOUBLE PRECISION ekinput(kmax,*),workmat(kmax,*)
       DOUBLE PRECISION work(*),egval(*)

* initializations

      vtroca = -1.0D0
      jmax = 0 
      jconsmax = 0

      do j=1,p
        auxlog(j) = .true.
      end do
      do j=1,ndentro
        auxlog(dentro(j))=.false.
      end do

      ncons=0
      nque=0
      DO j=1,p
         IF (.not.setk(j)) THEN
           nque=nque+1
	   que(nque)=j
	   esteveque(j)=.true.
         ELSE
	   esteveque(j)=.false.
	   IF (auxlog(j)) then
             ncons=ncons+1
             cons(ncons)=j
	   END IF
         END IF
      END DO

* emptying the queue "que"
       DO WHILE(nque.GT.0 .and. ncons.GT.0)
       jentra=que(nque)
       nque=nque-1
       vtrocamax=0.0D0
       DO j=1,ncons
        jsai=cons(j)
	setk(jsai)=.false.
	setk(jentra)=.true.
        if (criterio.eq.1) then
		vtroca=dobjrm(k,setk,p,poriginal,kmax,s,sq,workmat,
     +   setint,skinput)
        end if
        if (criterio.eq.2) then
           	vtroca=dobjrv(k,setk,p,poriginal,kmax,s,sq,workmat,
     +   setint,skinput)
        end if
        if (criterio.eq.3) then
		 vtroca=dobjgcd(k,setk,p,poriginal,kmax,s,workmat,
     +   nqsi,qsi,valp,setint,vecp,fica,skinput)
        end if
        if (criterio.eq.4) then
               vtroca=dobjtau2(k,setk,p,poriginal,kmax,s,h,rh,checksg,
     +   numtol,setint,work,egval,skinput,hkinput,ekinput,workmat)
           end if
	   if (criterio.eq.5) then
               vtroca=dobjxi2(k,setk,p,poriginal,kmax,s,h,rh,checksg,
     +   numtol,setint,work,egval,skinput,hkinput,workmat)
           end if
	   if (criterio.eq.6) then
               vtroca=dobjzeta2(k,setk,p,poriginal,kmax,s,h,rh,checksg,
     +   numtol,setint,work,egval,skinput,hkinput,ekinput,workmat)
           end if		 
	  if (criterio.eq.7) then
* tirei rh
               vtroca=dobjccr12(k,setk,p,poriginal,kmax,s,h,checksg,
     +   numtol,setint,work,egval,skinput,hkinput,workmat)
           end if		 

        if (checksg.and.criterio.GT.3.and.vtroca.eq.-0.9999D0) then
	   	numprb=.true.        
        end if
	
	IF (vtroca.GT.vtrocamax) THEN
           vtrocamax=vtroca
           jmax=jsai
           jconsmax=j
	END IF
	setk(jsai)=.true.
	setk(jentra)=.false.
       END DO
       IF (vtrocamax.GT.vactual) THEN
        vactual=vtrocamax
        setk(jmax)=.false.
	setk(jentra)=.true.
	cons(jconsmax)=jentra
        IF (.not.esteveque(jmax)) THEN
           nque=nque+1
           que(nque)=que(1)
           que(1)=jmax
           esteveque(jmax)=.true.
	END IF
       END IF
       END DO
       RETURN
       END
**********************************************************************

**********************************************************************
      function dobjrm(k,setk,p,poriginal,kmax,s,sq,ski,setint,skinput)
**********************************************************************
* This function computes the "variable part" of the RM criterion. By "variable
* part" is meant that which changes from one given k-subset to the other 
* (the final value of RM is only computed in the main subroutine, for 
* the selected solutions, in order to spare some computations in this
* subroutine, which is called extensively). 
*
* WARNING : This function calls the LAPACK routine DPOSV.
*
*  OUTPUT:
*
* The trace of the matrix inv(S_k) x (S**2)_k, where S_k 
* is the submatrix of the covariance (or correlation) matrix for the
* variables in subset setk, (S**2)_k is its equivalent for the square of
* that covariance/correlation matrix, and "inv" stands for matrix inverse.
*
*  INPUT: 
*
*   k     - integer, cardinality of subset setk. 
* setk    - logical vector, indicating a given subset of variables. 
*           setk(j)=.true. iff variable j belongs to the subset. Changed
*           on output.
*   p     - integer variable, number of original variables.
*   s     - double precision matrix of covariances of all p variables.
*   sq    - double precision matrix, the square of s.
* poriginal - 
*  kmax   - integer variable, giving the physical dimensionality of the 
*           square matrices ski and skinput
              
*  WORKING ARRAYS:
*
*  setint    - integer array used to store the indices of the variables 
*              kept in subset setk.
*  skinput   - double precision matrix used to store the submatrix S_k. 
*  ski       - double precision matrix used to store inverse of the submatrix S_k. 

       integer i,info,j,k,kmax,p,poriginal
       integer setint(*)
       logical setk(*)
       character(len=1) laux
       double precision s(poriginal,*),sq(poriginal,*)
       double precision ski(kmax,*),skinput(kmax,*),dobjrm

       external dposv

* initializations and call to LAPACK routine DPOSV
* matrix skinput will store the submatrix S_k. Matrix s will initially
* be identity, fed to the LAPACK routine DPOSV, from which it will emerge
* as the inverse of S_k. 

       do i=1,p
          setint(i)=i
       end do

       i=0
       do j=1,p
         if(setk(j)) then
           i=i+1
           setint(i)=j
         end if
       end do

       do i=1,k-1
           do j=i+1,k
              skinput(i,j)=s(setint(i),setint(j))
              skinput(j,i)=skinput(i,j)
              ski(i,j)=0.0D0
              ski(j,i)=0.0D0
           end do
           skinput(i,i)=s(setint(i),setint(i))
           ski(i,i)=1.0D0
         end do

          skinput(k,k)=s(setint(k),setint(k))
          ski(k,k)=1.0D0

          laux='L'
          info=0
          call dposv(laux,k,k,skinput,kmax,ski,kmax,info)

* computes the trace of inv(S_k) x (S**2)_k

          dobjrm=0.0D0
	  do i=1,k
	    do l=1,k
		  dobjrm=dobjrm+ski(i,l)*sq(setint(l),setint(i))
            end do
	  end do
	  return
	end   
**********************************************************************

**********************************************************************
      function dobjrv(k,setk,p,poriginal,kmax,s,sq,ski,setint,skinput)
**********************************************************************
* This function computes the "variable part" of the RV criterion. By "variable
* part" is meant that which changes from one given k-subset to the other 
* (the final value of RV is only computed in the main subroutine, for 
* the selected solutions, in order to spare some computations in this
* subroutine, which is called extensively). 
*
* WARNING : This function calls the LAPACK routine DPOSV.
* 
* OUTPUT:
* The trace of the matrix (inv(S_k) x (S**2)_k)**2, where S_k 
* is the submatrix of the covariance (or correlation) matrix for the
* variables in subset setk, (S**2)_k is its equivalent for the square of
* that covariance/correlation matrix, and "inv" stands for matrix inverse.
*
*  INPUT: 
*
*   k     - integer, cardinality of subset setk. 
* setk    - logical vector, indicating a given subset of variables. 
*           setk(j)=.true. iff variable j belongs to the subset. Changed
*           on output.
*   p     - integer variable, number of original variables.
*   s     - double precision matrix of covariances of all p variables.
*   sq    - double precision matrix, the square of s.
* poriginal -
*  kmax   - integer variable, giving the physical dimensionality of the 
*           square matrices ski and skinput
              
*  WORKING ARRAYS:
*
*  setint    - integer array used to store the indices of the variables 
*              kept in subset setk.
*  skinput   - double precision matrix used to store the submatrix S_k. 
*  ski       - double precision matrix used to store inverse of the submatrix S_k. 

       integer i,info,j,k,kmax,p,poriginal
       integer setint(*)
       logical setk(*)
       character(len=1) laux
       double precision s(poriginal,*),sq(poriginal,*)
       double precision ski(kmax,*),skinput(kmax,*)
       double precision soma1,soma2,soma,dobjrv

      external dposv

* initializations and call to LAPACK routine DPOSV
* matrix skinput will store the submatrix S_k. Matrix s will initially
* be identity, fed to the LAPACK routine DPOSV, from which it will emerge
* as the inverse of S_k. 
	  
       do i=1,p
          setint(i)=i
       end do


	  i=0
	  do j=1,p
	    if(setk(j)) then
		  i=i+1
       	          setint(i)=j
	    end if
	  end do

	  do i=1,k-1
	    do j=i+1,k
                  skinput(i,j)=s(setint(i),setint(j))
		  skinput(j,i)=skinput(i,j)
	          ski(i,j)=0.0D0
                  ski(j,i)=0.0D0
            end do
            skinput(i,i)=s(setint(i),setint(i))
            ski(i,i)=1.0D0
	  end do
	  skinput(k,k)=s(setint(k),setint(k))
          ski(k,k)=1.0D0

           laux='L'
           info=0
          call dposv(laux,k,k,skinput,kmax,ski,kmax,info)

* computes the trace of (inv(S_k) x (S**2)_k)**2

          dobjrv=0.0D0
          do i=1,k-1
            do j=i+1,k
              soma1=0.0D0
              soma2=0.0D0
	      do l=1,k
	        soma1=soma1+ski(i,l)*sq(setint(l),setint(j))
	        soma2=soma2+ski(j,l)*sq(setint(l),setint(i))
	      end do
	      dobjrv=dobjrv+soma1*soma2
            end do
          end do
          dobjrv=dobjrv*2
          soma=0.0D0
          do i=1,k
              soma=0.0D0
              do l=1,k
	 soma=soma+ski(i,l)*sq(setint(l),setint(i))
              end do
              dobjrv=dobjrv+soma**2.0D0
           end do
       return
       end   
********************************************************************


********************************************************************
      function dobjgcd(k,setk,p,poriginal,kmax,s,ski,nqsi,qsi,valp,
     +   setint,vecp,fica,skinput)
**********************************************************************
* This function computes the "variable part" of the GCD criterion. By "variable
* part" is meant that which changes from one given k-subset to the other 
* (the final value of GCD is only computed in the main subroutine, for 
* the selected solutions, in order to spare some computations in this
* subroutine, which is called extensively).  Specifically, this subroutine
* calculates the trace of the matrix (Sqsi_k) x inv(S_k), where S_k 
* is the submatrix of the covariance (or correlation) matrix for the
* variables in subset setk, "inv" stands for matrix inverse, and Sqsi_k
* 
* WARNING : This function calls the LAPACK routine DPOSV.
*
*
*  OUTPUT:
*    sum_{m in qsi} (valp_m vecp_m^t inv(S_k) vecp_m), where S_k is
*    the submatrix of the covariance/correlation matrix relevant for the 
*    k variables in subset setk, "inv" stands for matrix inverse, valp_m
*    stands for the eigenvalue of the full covariance/correlation 
*    matrix S, associated with the m-th rank (as defined in the set qsi), and
*    vecp_m is the corresponding SUB-eigenvector, defined as the subvector
*    of the eigenvector associated with valp_m, whose elements correspond to
*    positions of the k variables in setk.
*
*  INPUT: 
*
*  nqsi   - integer variable, indicating the dimension of the Principal 
*           Subspace with which the subspace spanned by the k-subset is
*            to be compared.
*  qsi    - integer vector, giving the ranks of the Principal Components 
*           which span the Principal Subspace described above (see nqsi).
*  valp   - double precision vector giving the eigenvalues of s, in 
*            decreasing order.
*  vecp   - double precision array (pxp) giving the eigenvectors of s, in
*            the same column-order as the values of valp.
*   k     - integer, cardinality of subset setk. 
* setk    - logical vector, indicating a given subset of variables. 
*           setk(j)=.true. iff variable j belongs to the subset. Changed
*           on output.
*   p     - integer variable, number of original variables.
*   s     - double precision matrix of covariances of all p variables. 
*  fica   - integer vector giving the original variable numbers of the 
*            admissible variables (those that are not forcefully excluded
*            from the subset by the user).
* poriginal -
*  kmax   - integer variable, giving the physical dimensionality of the 
*           square matrices ski and skinput
              
*  WORKING ARRAYS:
*
*  setint    - integer array used to store the indices of the variables 
*              kept in subset setk.
*  skinput   - double precision matrix used to store the submatrix S_k. 
*  ski       - double precision matrix used to store inverse of the submatrix S_k. 


      integer i,info,j,k,kmax,l,m,nqsi,p,poriginal
      integer setint(*),qsi(*)
      integer fica(0:p)
      logical setk(*)
      character(len=1) laux
      double precision s(poriginal,*),skinput(kmax,*),ski(kmax,*)
      double precision valp(*),vecp(poriginal,*)
      double precision dobjgcd,aux0,aux

      external dposv

* initilizations and call to LAPACK routine DPSOV
* matrix skinput will store the submatrix S_k. Matrix s will initially
* be identity, fed to the LAPACK routine DPOSV, from which it will emerge
* as the inverse of S_k.
  
       do i=1,p
          setint(i)=i
       end do

	  i=0
	  do j=1,p
	    if(setk(j)) then
		  i=i+1
   	      setint(i)=j
	    end if
	  end do

	  do i=1,k-1
	    do j=i+1,k
                  skinput(i,j)=s(setint(i),setint(j))
		  skinput(j,i)=skinput(i,j)
	          ski(i,j)=0.0D0
                  ski(j,i)=0.0D0
            end do
            skinput(i,i)=s(setint(i),setint(i))
            ski(i,i)=1.0D0
	  end do
	  skinput(k,k)=s(setint(k),setint(k))
          ski(k,k)=1.0D0

           laux='L'
           info=0
           call dposv(laux,k,k,skinput,kmax,ski,kmax,info)

* calculates sum_{m in qsi} (valp_m vecp_m^t inv(S_k) vecp_m)

          dobjgcd=0.0D0
	  do m=1,nqsi
	    aux=0.0D0
	    do i=1,k
		  aux0=0.0D0
		  do l=1,k
		    aux0=aux0+ski(i,l)*vecp(fica(setint(l)),qsi(m))
		  end do
                  aux0=aux0*vecp(fica(setint(i)),qsi(m))
    	          aux=aux+aux0	  
	    end do
            dobjgcd=dobjgcd+aux*valp(qsi(m))
	  end do
	  return
	  end   
************************************************************************


************************************************************************
       subroutine checksingl2nrm(k,origmat,ldmat,workmat,egval,work,
     +  numtol,numsing)
************************************************************************
* This function computes checks the singularity of the symmetric square 
* positive-definite (or positive-semidefinite) matrix origmat based on 
* its l2-norm condition number (ratio of largest to smallest eigenvalues).
* WARNING : This function calls the LAPACK routine DSYEV.

*  OUTPUT:
*
*   numsing - integer flag, set to 1 is origmat is numerically singular and 
*              to 0 otherwise

*  INPUT: 
*
*   k       - integer, conceptual dimension of matrix origmat. 
*  origmat  - double precision low-triangle of the positive definite matrix 
*             (or positive-semidefinite) whose singularity is being tested.
*  ldmat    - integer, leading (physical) dimension of the matrix origmat. 
*  numtol   - double precision tolerance level for the ratio of the
*             origmat eigenvalues.

*  WORKING ARRAYS:
*
*  workmat  - double precision matrix with the same dimensions as origmat. 
*  egval    - double precision array, used to store the eigenvalues of origmat.
*  work     - double precision array, to be used internaly by the DSYEV routine.
   
       integer k,info,lwork,numsing,ldmat
       character(len=1) jobz,laux
       double precision work(*),egval(*)
       double precision origmat(ldmat,*),workmat(ldmat,*)
       double precision numtol

       external dsyev

	numsing = 1 

* Create a copy of matrix origmat in order to keep the original data	

	do i =1,k
	  do j=1,i
		workmat(i,j) = origmat(i,j)
	   end do
	end do	

* call to LAPACK routine dsyev

	   jobz='N'
	   laux='L'
           info=0
	   lwork=6*k
  
           call dsyev(jobz,laux,k,workmat,ldmat,egval,work,lwork,info)

* check the singularity of matrix workmat and return 1 if numerically 
* singular, and 0 otherwise 

	if (info .NE. 0) then 
	  return	
	else
	  if (egval(k) .lt. numtol) then 
		return
	  else
		if (egval(1)/egval(k) .lt. numtol) then 
			return
	  	else
			numsing = 0
		end if
	  end if
	end if

	return
	end

************************************************************************

************************************************************************
       function dobjtau2(k,setk,p,poriginal,kmax,s,h,rh,checksg,numtol,
     +   setint,work,egval,skinput,hkinput,ekinput,workmat)
************************************************************************
* This function computes the tau_2 criterion.
* WARNING : This function calls the LAPACK routine DSYGV.
*
*  OUTPUT:
*
*   The tau^2 index of "effect magnitude".  
*   This criterion  is equivalent to the minimization of 
*   Wilks lambda statistic. tau_2 = 1 - lamlba^(1/l) Where l=min(rh,k)
*   lambda = det(E)/det(T) Where E=T-H.
*   We can also obtain lambda = d1*d2*...*dl, where d1, d2, ... dl are the 
*   eigen values of ET^-1
*
*  INPUT: 
*
*   k     - integer, cardinality of subset setk. 
* setk    - logical vector, indicating a given subset of variables. 
*           setk(j)=.true. iff variable j belongs to the subset. Changed
*           on output.
*   p     - integer variable, number of original variables.
*   s     - double precision low-triangle of the matrix of covariances of all p variables.
* poriginal - 
*   h     - double precision low-triangle of a symmetric positive semi-definite 
*           (or positive definite) matrix 
*   rh    - integer, h matrix rank
* checksg - logical, flag indicating if a test for ill-conditioned 
*	    (T being numerically singular) problems should be implemented
*  numtol - double precision tolerance level for the ratio of the diagonal elements
*           of T Choleski decomposition
*  kmax   - integer variable, giving the physical dimensionality of the 
*           square matrices skinput, hkinput, ekinput and workmat
              
*  WORKING ARRAYS:
*
*  setint    - integer array used to store the indices of the variables 
*              kept in subset setk.
*  skinput   - double precision matrix used to store the submatrix T_k. 
*  hkinput   - double precision matrix used to store the submatrix H_k. 
*  ekinput   - double precision matrix used to store the submatrix E_k. 
*  egval     - double precision array used to store the eigenvalues of E_k (T_k)^(-1).
*  work      - double precision array to be used internaly by LAPACK routines.
*  workmat   - double precision matrix to be used internaly by the checksingl2nrm routine.
    
       integer i,info,itype,j,k,lwork,p,poriginal,kmax,numsingular
       integer rh,rhaux
       integer setint(*)
       logical checksg
       logical setk(*)
       character(len=1) laux,jobz
       double precision numtol,lambda,dobjtau2
       double precision s(poriginal,*),h(poriginal,*)
       double precision skinput(kmax,*),hkinput(kmax,*),ekinput(kmax,*)
       double precision work(*),egval(*),workmat(kmax,*)

       external dsygv

* initializations and call to LAPACK routine dsygv
* matrix skinput will store the submatrix S_k. 
* matrix ekinput will store the submatrix E_K = S_k - H_k. 

       do i=1,p
          setint(i)=i
       end do
       i=0
       do j=1,p
         if(setk(j)) then
           i=i+1
           setint(i)=j
         end if
       end do
       do i=1,k
           do j=1,i
              skinput(i,j)=s(setint(i),setint(j))
           end do
        end do

	if (checksg) then
	   call checksingl2nrm(k,skinput,kmax,workmat,egval,work,
     +  numtol,numsingular)

	if (numsingular.eq.1) then
		dobjtau2 = -0.9999D0
		return
	   end if
        end if

       do i=1,k
           do j=1,i
              hkinput(i,j)=h(setint(i),setint(j))
              ekinput(i,j)=skinput(i,j)-hkinput(i,j)
           end do
        end do

	if (checksg) then
	   call checksingl2nrm(k,ekinput,kmax,workmat,egval,work,
     +  numtol,numsingular)

	if (numsingular.eq.1) then
		dobjtau2 = -0.9999D0
		return
	   end if
        end if

        itype=1
	jobz='N'
	laux='L'
        info=0
	lwork=6*k
  
        call dsygv(itype,jobz,laux,k,ekinput,kmax,skinput,kmax,
     +    egval,work,lwork,info)
	  
	lambda=1.0D0
	do i=1,k
	     lambda=lambda*egval(i)
        end do
	rhaux=rh
	if (rh.GT.k) then 
	      rhaux=k
	end if 
        dobjtau2= 1.0D0 - lambda ** (1.0D0/DBLE(rhaux))
        return
     
        end
************************************************************************

   
************************************************************************
       function dobjxi2(k,setk,p,poriginal,kmax,s,h,rh,checksg,numtol,
     +   setint,work,egval,skinput,hkinput,workmat)
************************************************************************
* This function computes the xi_2 criterion.
* WARNING : This function calls the LAPACK routine DSYGV.
*
*  OUTPUT:
*
*   The criterion xi_2 of "effect magnitude".  
*   This criterion  is equivalent to the maximization of Bartlett-Pillai 
*   trace statistic U = tr(HT^-1),  xi_2 = U/l, Where l=min(rh,k)
*  
*
* INPUT: 
*
*   k     - integer, cardinality of subset setk. 
* setk    - logical vector, indicating a given subset of variables. 
*           setk(j)=.true. iff variable j belongs to the subset. Changed
*           on output.
*   p     - integer variable, number of original variables.
*   s     - double precision low-triangle of the matrix of covariances of all p variables.
* poriginal - 
*   h     - double precision low-triangle of a positive semi-definite 
*           (or positive definite) matrix
*   rh    - integer, h matrix rank
* checksg - logical, flag indicating if a test for ill-conditioned 
*	    (T being numerically singular) problems should be implemented
*  numtol - double precision tolerance level for the ratio of the diagonal elements
*           of T Choleski decomposition
*  kmax   - integer variable, giving the physical dimensionality of the 
*           square matrices skinput, hkinput and workmat
              
*  WORKING ARRAYS:
*
*  setint    - integer array used to store the indices of the variables 
*              kept in subset setk.
*  skinput   - double precision matrix used to store the submatrix T_k. 
*  hkinput   - double precision matrix used to store the submatrix H_k. 
*  egval     - double precision array used to store the eigenvalues of H_k (T_k)^(-1).
*  work      - double precision array to be used internaly by LAPACK routines.
*  workmat   - double precision matrix to be used internaly by the checksingl2nrm routine.

   
       integer i,info,itype,j,k,lwork,p,poriginal,kmax,numsingular
       integer rh,rhaux
       integer setint(*)
       logical checksg
       logical setk(*)
       character(len=1) laux,jobz
       double precision numtol,dobjxi2
       double precision s(poriginal,*),h(poriginal,*)
       double precision skinput(kmax,*),hkinput(kmax,*)
       double precision work(*),egval(*),workmat(kmax,*)

       external dsygv

* initializations and call to LAPACK routine dsygv
* matrix skinput will store the submatrix S_k. 
* matrix hkinput will store the submatrix H_k. 

       do i=1,p
          setint(i)=i
       end do

       i=0
       do j=1,p
         if(setk(j)) then
           i=i+1
           setint(i)=j
         end if
       end do
       do i=1,k
           do j=1,i
              skinput(i,j)=s(setint(i),setint(j))
           end do
        end do

	if (checksg) then
	   call checksingl2nrm(k,skinput,kmax,workmat,egval,work,
     +  numtol,numsingular)
	   if (numsingular .eq. 1) then
		dobjxi2 = -0.9999D0
		return
	   end if
        end if

       do i=1,k
           do j=1,i
              hkinput(i,j)=h(setint(i),setint(j))
           end do
        end do

        itype=1
	jobz='N'
	laux='L'
        info=0
        lwork=6*k

        call dsygv(itype,jobz,laux,k,hkinput,kmax,skinput,kmax,
     +  egval,work,lwork,info)

        dobjxi2=0.0D0
	do i=1,k
	    dobjxi2=dobjxi2+egval(i)
        end do
	rhaux=rh
	if (rh.GT.k) then 
	    rhaux=k
	end if 
	dobjxi2=dobjxi2/DBLE(rhaux)  

        return
	end   
************************************************************************

************************************************************************
       function dobjzeta2(k,setk,p,poriginal,kmax,s,h,rh,checksg,numtol,
     +   setint,work,egval,skinput,hkinput,ekinput,workmat)
************************************************************************
* This function computes the zeta_2 criterion.
* WARNING : This function calls the LAPACK routine DSYGV.
*
*  OUTPUT:
*
*   The criterion zeta_2 of "effect magnitude".  
*   This criterion  is equivalent to the maximization of Hotteling-Lawley
*   trace statistic V = tr(HE^-1),  zeta_2 = V/(V+l), Where l=min(rh,k)
*
*  INPUT: 
*
*   k     - integer, cardinality of subset setk. 
* setk    - logical vector, indicating a given subset of variables. 
*           setk(j)=.true. iff variable j belongs to the subset. Changed
*           on output.
*   p     - integer variable, number of original variables.
*   s     - double precision low-triangle of the matrix of covariances of all p variables.
* poriginal - 
*   h     - double precision low-triangle of a positive semi-definite 
*          (or positive definite) matrix.
*   rh    - integer, h matrix rank
* checksg - logical, flag indicating if a test for ill-conditioned 
*	    (E being numerically singular) problems should be implemented
*  numtol - double precision tolerance level for the ratio of the diagonal elements
*           of E Choleski decomposition
*  kmax   - integer variable, giving the physical dimensionality of the 
*           square matrices skinput, hkinput, ekinput and workmat
              
*  WORKING ARRAYS:
*
*  setint    - integer array used to store the indices of the variables 
*              kept in subset setk.
*  skinput   - double precision matrix used to store the submatrix S_k. 
*  hkinput   - double precision matrix used to store the submatrix H_k. 
*  ekinput   - double precision matrix used to store the submatrix E_k. 
*  egval     - double precision array used to store the eigenvalues of H_k (E_k)^(-1).
*  work      - double precision array to be used internaly by LAPACK routines.
*  workmat   - double precision matrix to be used internaly by the checksingl2nrm routine.

    
       integer i,info,itype,j,k,lwork,p,poriginal,kmax,numsingular
       integer rh,rhaux
       integer setint(*)
       logical checksg
       logical setk(*)
       character(len=1) laux,jobz
       double precision numtol,lambda,dobjzeta2
       double precision s(poriginal,*),h(poriginal,*)
       double precision skinput(kmax,*),hkinput(kmax,*),ekinput(kmax,*)
       double precision work(*),egval(*),workmat(kmax,*)

       external dsygv

* initializations and call to LAPACK routine dsygv
* matrix skinput will store the submatrix S_k. 
* matrix ekinput will store the submatrix E_K = S_k - H_k. 

       do i=1,p
          setint(i)=i
       end do
       i=0
       do j=1,p
         if(setk(j)) then
           i=i+1
           setint(i)=j
         end if
       end do
       do i=1,k
           do j=1,i
              skinput(i,j)=s(setint(i),setint(j))
              hkinput(i,j)=h(setint(i),setint(j))
              ekinput(i,j)=skinput(i,j)-hkinput(i,j)
           end do
        end do

	if (checksg) then
	   call checksingl2nrm(k,ekinput,kmax,workmat,egval,work,
     +  numtol,numsingular)

	if (numsingular.eq.1) then
		dobjzeta2 = -0.9999D0
		return
	   end if
        end if

        itype=1
	jobz='N'
	laux='L'
        info=0
	lwork=6*k
  
        call dsygv(itype,jobz,laux,k,hkinput,kmax,ekinput,kmax,
     +    egval,work,lwork,info)

        dobjzeta2=0.0D0
        do i=1,k
	    dobjzeta2 = dobjzeta2 + egval(i)
        end do
	   
	rhaux=rh
	if (rh.GT.k) then 
	    rhaux=k
	end if 
	dobjzeta2=dobjzeta2/(dobjzeta2+DBLE(rhaux))  

        return
	end   
************************************************************************


************************************************************************
       function dobjccr12(k,setk,p,poriginal,kmax,s,h,checksg,numtol,
     +   setint,work,egval,skinput,hkinput,workmat)
************************************************************************
* This function computes the ccr1_2 criterion.
* WARNING : This function calls the LAPACK routine DSYGV.
*
*  OUTPUT:
*
*   The criterion ccr1_2 
*   This criterion  is equivalent to the maximization of Roy first
*   root (lambda1), first eigen value of HE^-1, where E=T-H.
*   ccr1_2 = lambda1/(1+lambda1). 
*  INPUT: 
*
*   k     - integer, cardinality of subset setk. 
* setk    - logical vector, indicating a given subset of variables. 
*           setk(j)=.true. iff variable j belongs to the subset. Changed
*           on output.
*   p     - integer variable, number of original variables.
*   s     - double precision low-triangle of the matrix of covariances 
*           of all p variables.
*   h     - double precision low-triangle of a symmetric positive semi-definite 
*           (or positive definite) matrix. 
* poriginal - 
* checksg - logical, flag indicating if a test for ill-conditioned 
*	    (E being numerically singular) problems should be implemented
*  numtol - double precision tolerance level for the ratio of the diagonal elements
*           of E Choleski decomposition
*  kmax   - integer variable, giving the physical dimensionality of the 
*           square matrices skinput, hkinput, ekinput and workmat
              
*  WORKING ARRAYS:
*
*  setint    - integer array used to store the indices of the variables 
*              kept in subset setk.
*  skinput   - double precision matrix used to store the submatrix T_k = H_k + E_k. 
*  hkinput   - double precision matrix used to store the submatrix H_k. 
*  egval     - double precision array used to store the eigenvalues of H_k (E_k)^(-1).
*  work      - double precision array to be used internaly by LAPACK routines.
*  workmat   - double precision matrix to be used internaly by the checksingl2nrm routine.

       integer i,info,itype,j,k,lwork,p,poriginal,kmax,numsingular
       integer setint(*)
       logical checksg
       logical setk(*)
       character(len=1) laux,jobz
       double precision numtol,dobjccr12
       double precision s(poriginal,*),h(poriginal,*)
       double precision skinput(kmax,*),hkinput(kmax,*)
       double precision work(*),egval(*),workmat(kmax,*)
        
      external dsygv

* initializations and call to LAPACK routine dsygv
* matrix skinput will store the submatrix S_k. 
* matrix hkinput will store the submatrix H_k.

       do i=1,p
          setint(i)=i
       end do
       i=0
       do j=1,p
         if(setk(j)) then
           i=i+1
           setint(i)=j
         end if
       end do
       do i=1,k
           do j=1,i
              skinput(i,j)=s(setint(i),setint(j))
           end do
        end do

	if (checksg) then
	   call checksingl2nrm(k,skinput,kmax,workmat,egval,work,
     +  numtol,numsingular)
	   if (numsingular .eq. 1) then
		dobjccr12 = -0.9999D0
		return
	   end if
        end if

       do i=1,k
           do j=1,i
              hkinput(i,j)=h(setint(i),setint(j))
           end do
        end do
          
        itype=1
	jobz='N'
	laux='L'
        info=0
	lwork=6*k

        call dsygv(itype,jobz,laux,k,hkinput,kmax,skinput,kmax,
     +    egval,work,lwork,info)
      
	dobjccr12=egval(k)

        return
	end   

xi2.coef<-function(mat,H,r,indices, tolval=10*.Machine$double.eps, tolsym=1000*.Machine$double.eps)
{
#   Function xi_2.coef
#   Computes the xi^2 index of "effect magnitude".  
#   This criterion is equivalent to the maximization of Bartlett-Pillai trace statistic.
#   Expected input: a variance-covariance (or correlation) matrix, 
#   Effect descrption matrix (H) and its rank (r)

#  error checking

#  mat and indices

  if  (sum(!(as.integer(indices) == indices)) > 0) stop("\n The variable indices must be integers")

  p <- dim(mat)[2]
  validmat(mat,p,tolval,tolsym,allowsingular=TRUE,algorithm="none")
    


# checks on r and H

  validnovcrit(mat,criterion="XI_2",H,r,p,tolval,tolsym)


# Computing the criterion value

      tr<-function(mat){sum(diag(mat))}

      xi2.1d<-function(mat,H,r,indices){
		l <- min(r,length(indices))
	tr(solve(mat[indices,indices],H[indices,indices]))/l	
       }
      dimension<-length(dim(indices))
      if (dimension > 1){
         xi2.2d<-function(mat,H,r,subsets){
             apply(subsets,1,function(indices){
			
			xi2.1d(mat,H,r,indices)})
            }  
             if (dimension > 2) {
               xi2.3d<-function(mat,H,r,array3d){
                   apply(array3d,3,function(subsets){xi2.2d(mat,H,r,subsets)})
                 }
               output<-xi2.3d(mat,H=H,r,indices)
              }
              if (dimension == 2) {output<-xi2.2d(mat=mat,H,r,indices)}
      }

      if (dimension < 2) {output<-xi2.1d(mat,H,r,indices)}
      output
}

\documentclass[a4paper]{article}
\usepackage{amsmath, amssymb}
\usepackage{color}
\usepackage[latin1]{inputenc}


%\VignetteIndexEntry{subselect} 

\setlength{\textwidth}{16cm}
\setlength{\textheight}{21cm}
\setlength{\evensidemargin}{0cm}
\setlength{\oddsidemargin}{0cm}

\newcommand{\bvec}{\ensuremath{\mathrm{\mathbf{b}}}}
\newcommand{\A}{\ensuremath{\mathrm{\mathbf{A}}}}
\newcommand{\C}{\ensuremath{\mathrm{\mathbf{C}}}}
\newcommand{\E}{\ensuremath{\mathrm{\mathbf{E}}}}
\newcommand{\F}{\ensuremath{\mathrm{\mathbf{F}}}}
\newcommand{\Hmat}{\ensuremath\mathrm{\mathbf{H}}}
\newcommand{\I}{\ensuremath{\mathrm{\mathbf{I}}}}
\newcommand{\M}{\ensuremath{\mathrm{\mathbf{M}}}}
\renewcommand{\P}{\ensuremath{\mathrm{\mathbf{P}}}}
\renewcommand{\S}{\ensuremath{\mathrm{\mathbf{S}}}}
\newcommand{\T}{\ensuremath{\mathrm{\mathbf{T}}}}
\newcommand{\U}{\ensuremath{\mathrm{\mathbf{U}}}}
\newcommand{\X}{\ensuremath{\mathrm{\mathbf{X}}}}
\newcommand{\Y}{\ensuremath{\mathrm{\mathbf{Y}}}}
\newcommand{\umn}{\ensuremath{\mathrm{\mathbf{1}_n}}}

\newcommand{\SsubGp}{\mbox{${\mathrm{\mathbf{S}}}_{\{\cal G\}}$}}
\newcommand{\tr}{\ensuremath{\mathrm{tr}}}

\title{The \texttt{subselect} R package}
\author{Jorge Cadima, Jorge Orestes Cerdeira, Pedro Duarte Silva, Manuel Minhoto}
\begin{document}


\maketitle


\begin{center}
\textbf{Version 0.16.1}
\end{center}



\hrule

\vskip 0.5cm

\abstract{The \texttt{subselect} package addresses the issue of variable
selection in different statistical contexts, among which exploratory data 
analyses; univariate or multivariate linear models; generalized linear 
models; principal components analysis; linear discriminant analysis, 
canonical correlation analysis. 
Selecting variable subsets requires the definition of a numerical
criterion which measures the quality of any given variable
subset as a surrogate for the full set of variables. The current version 
of the \texttt{subselect} package provides eight different  
criteria. For each available criterion, the package provides a function 
that computes the criterion value of any given subset. More significantly, 
the package provides efficient search functions that seek the 
best subsets of any given size, for a specified criterion.
}


\vskip 0.5cm

\hrule

\tableofcontents 


\vskip 0.5cm

\section{Introduction}


\subsection{The variable selection problem}

Selecting a subset of variables which can be used as a surrogate for a
full set of variables, without major loss of quality, is a problem
that arises in numerous contexts. 

Multiple linear regression has been the classical statistical setting 
for variable selection and the greedy algorithms of backward elimination, 
forward selection and stepwise selection have long been used
to choose a subset of linear predictors. However, their limitations are 
well known  and more efficient search algorithms have also been devised,
such as Furnival and Wilson's Leaps and Bounds Algorithm (reference
\cite{Furnival:74}).   

The \texttt{subselect} package addresses the issue of variable
selection in different statistical contexts, among which:
\begin{itemize}
\item exploratory data analyses;
\item univariate or multivariate linear models;
\item generalized linear models;
\item principal components analysis;
\item linear discriminant analysis;
\item canonical correlation analysis.
\end{itemize}


\subsection{Measuring the quality of a subset}

Selecting variable subsets requires the definition of a numerical
\textbf{criterion} which measures the quality of any given variable
subset. In a univariate multiple linear regression, for example, possible
measures of the quality of a subset of predictors are the coefficient
of determination $R^2$,  the $F$ statistic in a goodness-of-fit test, 
its corresponding $p$-value or Akaike's Information Criterion (AIC), 
to give a few examples.  

The \texttt{subselect} package assumes that all potential variable
subsets can be ranked according to a well-defined numerical criterion
that is relevant for the problem at hand, and that \textit{the ultimate 
goal is to select the best subsets for any given cardinality}, i.e., 
the subsets of any given size which have the highest criterion values, 
for a given criterion.

The criteria currently available in the package will be further
discussed in Section \ref{s criteria}. For each available criterion,
the package provides a function that computes the criterion value of any given 
subset. More importantly, the package provides efficient functions 
to search for the best subsets of any given size.

\subsection{Selecting the best subsets}

Identifying the best variable subsets of a given cardinality is 
-- for many criteria -- a computationally intensive problem with
datasets of even moderate size, as is well known in the standard
linear regression context. 

The \texttt{subselect} package has a function (\texttt{eleaps})
implementing Duarte Silva's adaptations (references \cite{Pedro:01}
and \cite{Pedro:02}) of Furnival and Wilson's Leaps and Bounds
Algorithm \cite{Furnival:74} for 
variable selection, using the criteria considered in this
package. This function is very fast for datasets with up to
approximately $p=30$ variables and is guaranteed to identify the $m$  best
subsets of any size $k$ from $1$ to $p$, with $m$ in the range from 
$1$ to $\binom{p}{k}$. It is \textbf{the  recommended
function for variable selection with small or moderately-sized data
sets} and is discussed in subsection \ref{ss eleaps}.   

For \textbf{larger data sets} (roughly $p>35$) the \texttt{eleaps} function is
no longer computationally feasible and alternative search heuristics
are needed. The \texttt{subselect} package provides \textbf{three functions
with different search algorithms}: 
\begin{description}
\item[\texttt{anneal}] a simulated annealing-type search algorithm; 
\item[\texttt{genetic}] a genetic algorithm; and 
\item[\texttt{improve}] a modified local improvement algorithm.
\end{description}
These algorithms are described in reference \cite{CCM:04} and are 
discussed in Section \ref{s algoritmos}. They perform better
than the greedy-type algorithms, such as the standard stepwise
selection algorithms widely used in linear regression.

All four search functions invoke code written in either C++
(\texttt{eleaps}) or Fortran (\texttt{anneal},  \texttt{genetic} and
\texttt{improve}), to speed up computation times.
The search functions are further discussed in Section \ref{s algoritmos}.


\subsection{Installing and loading \texttt{subselect}} 


The \texttt{subselect} package is on CRAN (cran.r-project.org) and can
be installed like any other CRAN package, namely using R's
\texttt{install.packages} facility:


\begin{verbatim}
> install.packages(''subselect'')
\end{verbatim}

This vignette assumes that the \texttt{subselect} package has already
been installed in the user's system. 
Once installed, \texttt{subselect} must be loaded into an R session:

<<>>=
library(subselect) 
@ 

A preliminary view of the available functions and of their purpose 
can, as usual, be obtained via:

\begin{verbatim}
> library(help=subselect)
\end{verbatim}



\section{Criteria for variable selection} 
\label{s criteria}

Currently, the package accepts \textbf{eight different criteria}
measuring the quality of any given variable subset. These 
criteria can be (simplistically) classified into three groups: 
(i) criteria that are useful in an exploratory analysis or principal 
component analysis of a data set;  (ii) criteria that 
are model-based and are relevant in contexts that can be framed as a
multivariate linear model  (multivariate linear regressions, MANOVAs,
linear discriminant analysis, canonical correlation analysis, etc.); and 
(iii) criteria that are useful for variable selection in the context 
of a generalized linear model. 

\subsection{Criteria for exploratory data analyses and PCA}

The three criteria relevant for exploratory data analyses  have been 
previously suggested in the literature (see references \cite{CadJol:01} 
and \cite{CCM:04} for a fuller discussion). They can all be
framed as matrix correlations \cite{Ramsay:84} involving an $n\times p$ 
data matrix (where $p$ is the number of variables and $n$ the number of
observations on those variables) 
and projections of its columns on some subspace of $\mathbb{R}^n$.
In the \texttt{subselect} package, these three criteria are called: 

\begin{description}
\item[RM] The square of this matrix correlation is the
proportion of total variance (inertia) that is preserved 
if the $p$ variables are orthogonally projected onto the subspace 
spanned by a given $k$-variable subset. \emph{Choosing the $k$-variable
subset that maximizes this criterion is therefore akin to what is 
done in a Principal Component Analysis, with the difference that 
the optimal $k$-dimensional subspace is chosen only from among
the  $\binom{p}{k}$  subspaces spanned by $k$ of the $p$
variables}. This is the second of McCabe's  four criteria for what he
terms \textbf{\textit{principal variables}} \cite{McCabe:84}. See Subsection \ref{sss RM} for more details.

\item[GCD] Yanai's \emph{Generalized Coefficient of Determination} 
\cite{Ramsay:84} is  a measure of the closeness of two subspaces that
is closely related to the \emph{gap metric} \cite{Golub:96}. 
More precisely, it is the average of the squared canonical 
correlations between two sets of variables spanning each of the 
subspaces \cite{Ramsay:84}. In this context, these subspaces
are  the subspace spanned by $g$ Principal Components of 
the full, $p$-variable, data set; and the subspace spanned by a 
$k$-variable subset of those $p$ variables. By default the PCs 
considered are the \emph{first} $g$ PCs, and the number of variables
and PCs considered is the same ($k=g$), but other options can be 
specified by the user. \emph{Maximizing the GCD
corresponds to chosing the $k$ variables that span a subspace that is
as close as possible to the principal subspace spanned by the $g$ 
principal components that were specified}. See Subsection \ref{sss GCD} for more details.

\item[RV] Escoufier's RV-criterion \cite{RobEsc:76}  measures the
similarity of the $n$-point configurations defined by the $n$ rows of two
(comparable) data matrices, allowing for translations of the origin, rigid
rotations and global changes of scale. In this case, the two matrices
that are compared are the original data matrix $\X$ and the matrix
$\P_k\X$ which results from projecting the $p$ columns of $\X$ onto
a $k$-dimensional subspace spanned by $k$ of the $p$ original
variables. \emph{Maximizing this criterion, among all $k$-variable subsets,
corresponds to choosing the $k$ variables that span the subspace which 
best reproduces the original $n$-point configuration, allowing for 
translations of the origin, rigid rotations and global changes of scale}. 
See Subsection \ref{sss RV} for more details.
\end{description}

The package functions that compute the values of these indices 
(described below) all have the following two input \emph{arguments}:

\begin{description}
\item[\texttt{mat}] denotes the covariance or correlation matrix of 
the full data set;
\item[\texttt{indices}] is the vector, matrix or 3-d array of integers
giving sets of integers identifying the variables in the subsets of interest.
\end{description}

The function for the $GCD$ coefficient has an additional argument,  
described in subsection \ref{sss GCD}.

\subsubsection{The RM coefficient}
\label{sss RM}

\paragraph{Motivation and definition}

\textbf{ }\newline

It is well-known that the first $k$ principal components of an $n\times p$ data set are the $k$ linear combinations of the $p$ original variables which span the subspace that maximizes the proportion of total variance retained, if the data are orthogonally projected onto that subspace. A similar problem, but considering only the subspaces that are \emph{spanned by $k$-variable subsets of the $p$ original variables}, can be formulated \cite{CadJol:01} as follows: determine the $k$-variable subset which maximizes the square of the RM coefficient, 
%the matrix correlation between the $n\times p$ data
%matrix $\X$ and $\P_k\X$, where $\P_k$ is the $n\times n$ matrix of
%orthogonal projections on the subspace spanned by some subset ${\cal K}$ of $k$
%among the $p$ variables:
\begin{equation}
\label{eq RM}
RM = \mathrm{corr}(\X,\P_k \X) =
\sqrt{\frac{\mathrm{tr}(\X^t \P_k \X)}{\mathrm{\mathrm{tr}(\X^t \X)}}}=
\sqrt{\frac{\sum\limits_{i=1}^{p}\lambda_i (r_m)_i^2}{\sum\limits_{j=1}^{p}\lambda_j}}
 = \sqrt{\frac{\mathrm{tr}([\S^2]_{({\cal K})}\S_{\cal K}^{-1})}{\mathrm{tr}(\S)}} ,
\end{equation}
with \texttt{corr} denoting the matrix correlation; \texttt{tr} the matrix trace; and where
\begin{itemize}
\item $\X$ is the full (column-centered or possibly standardized) data matrix; 
\item $\P_k$ is the matrix of  orthogonal projections on the subspace 
spanned by a given $k$-variable subset;  
\item $\S=\frac{1}{n}\X^t\X$ is the $p\times p$ covariance (or correlation) matrix of the full data set;
\item ${\cal K}$ denotes the index set of the $k$ variables in the variable subset; 
\item $\S_{\cal K}$ is the $k\times k$ principal submatrix of matrix $\S$ which 
  results from retaining the \k rows/columns whose indices belong to ${\cal K}$;
\item $[\S^2]_{({\cal K})}$ is the $k\times k$ principal submatrix of $\S^2$ 
obtained by retaining the \k rows/columns associated with set ${\cal K}$.
\item $\lambda_i$ stands for the $i$-th largest
eigenvalue of the covariance (or correlation) matrix defined by $\X$;
\item $r_m$ stands for the multiple correlation between the
$i$-th principal component of the full data set and the $k$-variable subset.
\end{itemize}

The values of the RM coefficient will lie in the interval $[0,1]$, with 
larger values indicating a higher proportion of explained variance. Note that it is not relevant whether 
sample (co)variances (with denominator $n\!-\!1$), or even the sums of squares 
and products matrices, are used instead of matrix $\S$ as defined above: 
RM is insensitive to multiplication of $\S$ by a constant.

\paragraph{The \texttt{rm.coef} function}

\textbf{ }\newline

The \texttt{subselect} package provides the function \textbf{rm.coef} which
computes the RM coefficient, given a positive definite matrix 
(function argument \texttt{mat}, corresponding to $\S$ in 
the final expression of equation (\ref{eq RM})) and a vector
with $k$ indices (function argument \texttt{indices}, corresponding to 
${\cal K}$ in (\ref{eq RM})):

\begin{verbatim}
> rm.coef(mat,indices)
\end{verbatim}

The \textbf{rm.coef} function uses the final expression in equation (\ref{eq RM}). 
In the standard setting, matrix $\S$ is a covariance or correlation matrix for the full data set,
but it may be any positive definite matrix, such as a matrix of non-central second moments. 


\paragraph{Examples}

\textbf{ }\newline

As an example of the use of the \textbf{rm.coef} function, let us compute the 
value of the RM coefficient for the two petal 
measurements in Fisher's \emph{iris} data (variables 3 and 4, respectively, 
\texttt{Petal.Length} and \texttt{Petal.Width}; see \texttt{help(iris)} 
in a standard R session for more information on this data set). This can be
obtained as follows, using \textbf{rm.coef}'s only two arguments:

<<>>=
rm.coef(mat=var(iris[,-5]),indices=c(3,4))

@ 
The square of this value,
<<>>=
rm.coef(var(iris[,-5]),c(3,4))^2

@ 
gives the proportion of total variability that is preserved if the
four morphometric variables were orthogonally projected  onto the
subspace spanned by the petal measurements. It is quite often the case that this value
is not very much smaller than the proportion of total variability that
is accounted for by the same number of principal components \cite{CadJol:01}.

If more than one subset of a given cardinality is desired, the \texttt{indices} 
argument should be given as a matrix whose rows provide the indices of the variables 
in each subset. For example, the RM coefficients for the three-variable 
subsets of the \emph{iris} data, given by variables $\{1,2,3\}$ and $\{1,2,4\}$ 
are requested by the command:

<<>>=
rm.coef(var(iris[,-5]), indices=matrix(nrow=2,ncol=3,byrow=TRUE,c(1,2,3,1,2,4)))
@

The argument \texttt{indices} can also be a three-dimensional array, if subsets 
of different cardinalities are desired. In this case, the third dimension of 
\texttt{indices} is associated with different cardinalities. This option 
is especially useful when applying the \texttt{rm.coef} function to the output of the 
search algorithms (see Section \ref{s algoritmos}), if more than one cardinality 
has been requested. 

An example for the less frequent case, where the request is built from scratch, is 
now given. It computes the
value of RM for two 1-variable subsets and two 3-variable subsets:

<<>>=
subsets  <- array(data=c(3,2,0,0,0,0,1,1,2,2,3,4), dim=c(2,3,2))
colnames(subsets) <- paste("V",1:3,sep="")
rownames(subsets) <- paste("Solution",1:2)
dimnames(subsets)[[3]]<-paste("Size",c(1,3))
subsets
rm.coef(var(iris[,-5]),indices=subsets)
@ 

The output places the values for each cardinality in a different column.
Notice how the missing variable indices for the lower cardinalities are given as zeroes,

%{\color{red}[FALTA: SE MUDAR O OUTPUT PARA NA, ESTAS FUNCOES TAMBEM TEM DE MUDAR]}.


\subsubsection{The GCD coefficient}
\label{sss GCD}


\paragraph{Motivation and definition}

\textbf{ }\newline

In the standard setting for the \texttt{subselect} package, given a $p$-variable 
data set and a subset of $g$ of its principal components, the GCD criterion 
is a measure of similarity between the  principal subspace spanned by the $g$ 
specified principal components and the subspace spanned by a given 
$k$-variable subset of the original variables \cite{CadJol:01}.

The GCD is the matrix correlation between  the matrix  $\P_k$ of  orthogonal 
projections on the subspace  
spanned by a given $k$-variable subset and  the matrix  $\P_g$ of
orthogonal projections on the subspace spanned by the $g$ given
principal components of the full data set \cite{CadJol:01}: 
  
\begin{equation}
  \label{eq GCD}
GCD  = \mathrm{corr}(\P_k , \P_g) = 
\frac{\mathrm{tr}(\P_k\cdot \P_g)}{\sqrt{k\cdot g}} =
 \frac{1}{\sqrt{k g}} \sum\limits_{i\in{\cal
G}}(r_m)_i^2 = \tr([\SsubGp]_{({\cal K})}\S_{\cal
K}^{-1})/\sqrt{gk},
\end{equation}
where 
\begin{itemize}
\item  ${\cal K}$, $\S$,  $\S_{\cal K}$ and  $(r_m)_i$ are all defined
as above (subsection \ref{sss RM});
\item ${\cal G}$ denotes the index set of the $g$ principal components 
in the PC subset; 
\item $\SsubGp$ is the $p\times p$ matrix of rank $g$ that results from 
retaining only the $g$ terms in the spectral decomposition of $\S$ that 
are associated with the PC indices in the set ${\cal G}$;
\item $[\SsubGp]_{({\cal K})}$ is the $k\times k$ principal submatrix 
of $\SsubGp$ that results from retaining only the rows/columns whose 
indices are in ${\cal K}$.
\end{itemize}

The values of the GCD coefficient are in the interval $[0,1]$, with 
larger values indicating greater similarity between the subspaces.

\paragraph{The \texttt{gcd.coef} function}

\textbf{ }\newline

The \textbf{gcd.coef} function computes the GCD coefficient for a given
positive definite matrix (function argument \texttt{mat}, corresponding 
to matrix $\S$ in the final expression of equation (\ref{eq GCD})), 
a given vector of $k$ variable indices (function argument \texttt{indices}, 
corresponding to ${\cal K}$ in  (\ref{eq GCD})),  
and a given vector of $g$ PC indices (function argument \texttt{pcindices},
corresponding to ${\cal G}$ in  (\ref{eq GCD})):

\begin{verbatim}
> gcd.coef(mat,indices,pcindices)
\end{verbatim}

If the \texttt{pcindices} argument is not specified, by default it 
is set to the first $k$ PCs, where $k$ is the size of the variable 
subset defined by argument \texttt{indices}.

The value of the GCD is computed 
using the final expression in equation (\ref{eq GCD}). Matrix $\S$ (the 
input argument \texttt{mat}) is usually the covariance or correlation 
matrix for the full $p$-variable data set, but there may be other contexts where 
the final expression in (\ref{eq GCD}) makes sense.

\paragraph{Examples}

\textbf{ }\newline

The value of the GCD coefficient for the first two covariance-matrix PCs in  
Fisher's \emph{iris} data and the  two petal measurements
can be obtained as following:

<<>>=
gcd.coef(var(iris[,-5]),ind=c(3,4),pcind=c(1,2))

@ 

The two sepal measurements span a subspace that is much closer to the principal plane:

<<>>=
gcd.coef(var(iris[,-5]),ind=c(1,2))

@ 


As with the RM coefficient  the value of the GCD coefficient 
for multiple subsets can be requested by providing the argument \texttt{mat} as a matrix
(if the subsets are all of the same cardinality) or a three-dimensional array (for subsets 
of different cardinalities) (see subsection \ref{sss RM} for examples). In this case, 
however, the PC indices requested via the argument \texttt{pcindices}
must use the same vector for each function call.


\subsubsection{The RV coefficient}
\label{sss RV}

\paragraph{Motivation and definition}

\textbf{ }\newline

As already stated, Escoufier's RV-criterion \cite{RobEsc:76}  measures the
similarity of the $n$-point configurations defined by the $n$ rows of two
(comparable) data matrices, allowing for translations of the origin, rigid
rotations and global changes of scale. The criterion is defined, in this 
context, as the matrix correlation 
between the matrices  $\X\X^t$ and $\P_k\X\X^t\P_k$ (where $\X$ and
$\P_k$ are defined as above) \cite{CCM:04}:

\begin{equation}
  \label{eq RV}
\mathrm{RV}=\mathrm{corr}(\X\X^t,\P_{k}\X\X^t\P_{k})=
\frac{1}{\sqrt{\tr(\S^2)}}\cdot\sqrt{\tr
\left(\left[\S^2\right]_{({\cal K})}\left[\S_{\cal K}\right]^{-1}\right)^2},
\end{equation}
($\S$, $\S_{\cal K}$ and $\left[\S^2\right]_{({\cal K})}$ are defined as in
section \ref{sss RM}).

Possible values for the RV coefficient lie in the interval $[0,1]$, with 
larger values indicating more similarity between the two $n$-point configurations.

\paragraph{The \texttt{rv.coef} function}

\textbf{ }\newline

The value of the RV coefficient, for a given positive definite matrix 
(function argument \texttt{mat}, corresponding 
to matrix $\S$ in the final expression of equation (\ref{eq RV})) and 
a given vector of $k$ variable indices (function argument \texttt{indices}, 
corresponding to ${\cal K}$ in equation (\ref{eq RV})), can be
computed using the \textbf{rv.coef} function: 

\begin{verbatim}
> rv.coef(mat,indices)
\end{verbatim}


\paragraph{Examples}

\textbf{ }\newline

The \texttt{farm} data set (included in this package), has $n=99$ individuals,
observed in each of $p=62$ variables. The RV coefficient
reveals that the $99-$point configuration of the standardized data in
$\mathbb{R}^{62}$ is fairly similar to the  configuration that results
from the orthogonal projection of those $99$ points on the subspace
spanned by variables number 2, 37, 57 and 59:

<<>>=
data(farm)
rv.coef(cor(farm),ind=c(2,37,57,59))
@ 

For two different variable subsets of size 4, the argument \texttt{indices} 
is given as a matrix, whose rows indicate the variable indices for 
each subset. For example, the RV-coefficients of the four-variable subsets 
$\{2,12,56,59\}$ and  $\{2,3,11,59\}$, are requested by the command:

<<>>=
rv.coef(cor(farm), indices=matrix(nrow=2,ncol=4,byrow=TRUE,c(2,12,56,59,2,3,11,59)))
@ 


\subsection{Criteria for a Multivariate Linear model context}
\label{ss linear model criteria}

Different statistical methods arise within the general
framework of a multivariate linear model \cite{Huberty:94}:
$$\X = \A \pmb{\Psi} + \U ,$$ 
where $\X$ is an $n\times p$ data matrix of
original variables, $\A$ is a known ($n\times q$) design matrix, 
$\pmb{\Psi}$ is a ($q\times p$) matrix of unknown parameters 
and $\U$ is a ($n\times p$) matrix of error vectors. 
Particular cases in this setting include, among others,
[Multivariate] Analysis of Variance ([M]ANOVA),  Linear 
Discriminant Analyis (LDA) and Canonical Correlation
Analysis (CCA). Here we will be particularly concerned with 
contexts where a selection of subsets of $\X$ 
is warranted, the classical cases being LDA and CCA. 

In these statistical methods, variable subsets are often  
assessed according to their contribution to the violation of an 
additional reference hypothesis, $H_0 \,:\, \C \, \pmb{\Psi} = 0$, 
where $\C$ is a known coefficient matrix of rank $r$ \cite{Huberty:94}.

For example, in LDA $\X$ is a matrix of $n$ observations, divided 
by $q$ groups, on $p$ attributes; $\A$ is a matrix of group indicators; 
and $\pmb{\Psi}$ is a matrix of group-specific population means. In this 
case, the rows of $\C$ specify $q-1$ contrasts, stating the equality 
of population means across groups. Hence, $r = min(p,q-1)$, and any index 
of the extent of $H_0$ violations can be interpreted as a measure 
of group separation.   

In Canonical Correlation Analysis, $\A = \left[ \umn \, \Y \right]$ 
where  $\umn $ is a vector of ones, the columns of $\X$ and $\Y$ 
are $n$ observations on two different sets of variables,
the reference hypothesis, 
$H_0: \C \ \pmb{\Psi} = \left[0 \, \I_q \right] \left[\pmb{\Psi}_0^t \, 
  \pmb{\Psi}_{\Y}^t \right]^t = 0 $, 
states that the two sets are uncorrelated, and 
$r = min\left(rank(\X),rank(\Y)\right)$. Indicators for the violation of 
$H_0$ are measures of linear association between the $\X$ and $\Y$ variable 
sets. We note that in this context only the variables associated 
with the $\X$ group are selected, while the $\Y$ group of variables 
remains fixed.

When $\Y = y$ consists of a single (response) variable this problem 
becomes the traditional variable selection problem in Linear Regression, 
usually modelled as 
$y = \left[1_n \, \X \right] \left[\beta_0 \ \pmb{\beta}_X \right]^t + \pmb{\epsilon} $. 
While \texttt{subselect} can also be used with reasonable results 
in multivariate regression models 
$\Y =\left[1_n \, \X \right] \left[\pmb{\beta_0} \, \pmb{\beta}_X \right]^t + \pmb{\epsilon}$,
with $\Y$ having more than one column, the symmetric association indices  
considered in \texttt{subselect} were not designed to measure the 
quality of $\Y$ predictions. In multivariate prediction problems 
other (non-symmetric) measures such as those discussed in Rencher 
\cite{Rencher:95} and McQuarrie and Tsai \cite{McQuarrieTsai:98} may 
be more relevant. In univariate regression all these measures are 
monotone functions of the traditional coefficient of determination, 
which is also the index considered by \texttt{subselect}.

  
It is well known that, under classical Gaussian assumptions, test 
statistics for $H_0$ are given by several increasing functions of 
the $r$ positive eigenvalues of a product matrix $\T^{-1}\Hmat$, 
with $\T$ and $\Hmat$ the total and effect matrices of Sum of 
Squares and Cross Product (SSCP) deviations associated with $H_0$, such that
$$ \T = \Hmat + \E,$$
where matrix $\E$ is the matrix of residual SSCP.  The former SSCP matrices are given by 
$\T = \X^t(\I_p - \P_{\omega}) \X$ and $\Hmat = \X^t(\P_{\Omega} -
\P_{\omega}) \X$,  where $\I_p$ is an identity matrix, $\P_{\Omega} =
\A(\A^t\A)^-\A^t$ and  
$$\P_{\omega} = \A(\A^t\A)^-\A^t -
\A(\A^t\A)^-\C^t[\C(\A^t\A)^-\C^t]^-\C(\A^t\A)^-\A^t,$$ 
are projection matrices on the spaces spanned by the columns of $\A$
(space $\Omega$) and by the linear combinations of these columns that
satisfy the reference hypothesis (space $\omega$). In these
formulas $\M^t$ denotes the transpose of matrix $\M$, and $\M^-$ a
generalized inverse. Furthermore, whether
or not the classical assumptions hold, the same eigenvalues can be
used to define descriptive indices that measure an "effect"
characterized by the violation of $H_0$.

\subsubsection{Four criteria}

In the \texttt{subselect} package four effect-size indices, which 
are monotone functions of the traditional test statistics, are 
used as comparison criteria. These indices are called:

\begin{description} 
\item[Ccr12] The $ccr_1^2$  index is a function of the traditional 
\emph{Roy first root} test statistic \cite{Roy:39}, $\lambda_1$, for
the violation of  the linear hypothesis of the form 
$H_0\,:\, \C \pmb{\Psi} = 0$. 
%Specifically, $$ccr_1^2 = \frac{\lambda_1}{1+\lambda_1}.$$
\emph{Maximizing this criterion is equivalent to maximizing
Roy's first root}  (see Subsection \ref{sss ccr12} for more details).
\item[Tau2] The $\tau^2$ index is a  function of the traditional \emph{Wilks'
Lambda} statistic  \cite{Wilks:32}.
%, defined as $\Lambda=\frac{det(\E)}{det(\T)}$: $$\tau^2= 1-\Lambda^{(1/r)}.$$
\emph{Maximizing this criterion is equivalent to minimizing Wilks' Lambda} (see Subsection \ref{sss tau2}).
\item[Xi2] The $\xi^2$ coefficient is a function of the
\emph{Bartlett-Pillai trace} test statistic  \cite{Pillai:55}
%, $P=\tr(\Hmat\T^{-1})$: $$\xi^2 = \frac{P}{r}.$$
\emph{Maximizing this criterion is equivalent to maximizing
the Bartlett-Pillai statistic} (see Subsection \ref{sss xi2}).
\item[Zeta2] The index $\zeta^2$ is a function of the traditional
\emph{Lawley-Hotelling trace} test statistic  \cite{Lawley:38} \cite{Hotelling:51}.
%,  $V=\tr(\Hmat\E^{-1})$: $$\zeta^2= \frac{V}{V+r}.$$
\emph{Maximizing this criterion is equivalent to maximizing
the Lawley-Hotelling statistic} (see Subsection \ref{sss zeta2}).
\end{description} 

Cramer and Nicewander \cite{CramerNicewander:79} 
introduced these indices in the context of CCA, and Huberty \cite{Huberty:94} 
discusses their use in the context of LDA and MANOVA. 
\textit{In all four cases, the indices are defined so that their 
values lie between $0$ and $1$, and the larger the index value, the better 
the variable subset} as a surrogate for the full data set.  
In the specific case of a multiple linear regression (with a single
response variable), all four indices are equal to the standard
coefficient of determination, $R^2$. However, in multivariate problems with $r > 1$
the indices differ because they place different emphasis on canonical directions
associated with the effect under study. In particular, $ccr_1^2$ only considers the
first canonical direction, $\xi^2$ weights all $r$ directions in a balanced way, and
$\tau^2$, $\zeta^2$ are intermediate indices that emphasize the main
directions (see reference \cite{Pedro:01} for further details).  

The four package functions that compute the values of these indices are 
called \texttt{*.coef}, where the asterisk denotes the above criterion name. 
These functions have \textit{six input arguments}: 
\begin{description}
\item[mat]  denotes the Total SSCP matrix $\T$;
\item[H] denotes the effects SSCP matrix  of relevance for
the particular case at hand (see subsection \ref{sss Hmat} for more details);
\item[r] denotes the rank of the $\Hmat$ matrix (except in degenerate cases - 
see the \texttt{help} files for each \texttt{*.coef} function);
\item[indices] is the vector, matrix or 3-d array of integers
that identify the variables in the subsets of interest; 
\item[tolval \texttt{and} tolsym] are parameters used
to check for ill-conditioning of the matrix \texttt{mat},
and the symmetry of matrices \texttt{mat} and $\Hmat$ (see the
  \texttt{help} files for more details).  
\end{description}

\subsubsection{Creating the SSCP matrices} 
\label{sss Hmat} 

The relevant Sum of Squares and Cross-Products Effect matrix $\Hmat$
is specific to each context in which model-based variable selection
arises. Computing these matrices can sometimes be time-consuming and
the direct application of the formulas described in the previous
subsection can lead to large rounding errors. The \texttt{subselect} 
package provides  helper functions which, for standard contexts, 
create both the SSCP Effects matrix $\Hmat$ and the SSCP Total
matrix $\T$, using sound numerical procedures based on
singular value decompositions, 
%{\color{red}[JC: tu e' que sabes  como fizeste, se foi com a dvs, acho bem referir]}
as well as computing the presumed rank $r$ of $\Hmat$. 
%There is also a more 
%general helper function (\texttt{glhHmat}) for the general 
%multivariate linear hypothesis  case.

The output from these functions can be used as input for
both the functions that compute the model-based criteria of quality
for a given variable subset (discussed in the subsections below) and
for the search functions that seek the best subsets of variables
for a given data set (see Section \ref{s algoritmos}). 

For the multivariate linear model context, \texttt{subselect} 
provides three such helper functions. For all three functions, 
the output is a list of four objects: 
\begin{description}
\item[mat] is the relevant Total SSCP matrix $\T$ divided by
$n-1$ (where $n$ is the number of rows in the data matrix);
\item[H] is the relevant Effect SSCP matrix divided by $n-1$;
%{\color{red}PEDRO:  Acho que sim, mas sugiro que , se nao 
%fizer mal, se deixe como esta'. Caso contrario teremos de 
%re-escrever varias coisas.  Disseste:'' Wouldn't it be cleaner 
\item[r] is the rank of matrix $\Hmat$;
\item[call] is the function call which generated the output.
\end{description}

The fact that \texttt{mat} and \texttt{T} are not defined as the 
standard SSCP matrices, but rather divided by $n-1$ is of no consequence,  
since multiplying T and H by a common scalar does not affect the criteria values.  

The three helper functions currently available for the multivariate 
linear model context are the following:


\paragraph{The \textbf{lmHmat} function for linear regression and CCA}

\textbf{ }\newline

The function \textbf{lmHmat} creates the necessary SSCP matrices for 
a \textbf{\emph{linear regression}} or \emph{\textbf{canonical correlation
analysis}} context. In a (possibly multivariate response) linear
regression context, it is assumed that the response variables are
fixed and a subset of the predictor variables is being assessed. 
In a canonical correlation analysis context, it is
assumed that one of the two groups of variables is fixed, and it is a
subset of the second group of variables  that is being assessed 
(variable selection in both groups will currently have to be done via
a 2-step approach). 
This function takes by default the following arguments:
\begin{description}
  \item[x] is a matrix containing the full set
of predictor variables, in the regression context, or the group of
variables in which a variable subset is to be chosen in the CCA context. 
  \item[y] is a matrix or vector containing the set of
fixed response variables, in the regression context, or the set of
fixed variables in the CCA context.
\end{description}

There is an S3 \emph{method} for arguments \texttt{x} and 
\texttt{y} of class  \texttt{data.frame}, as well as methods 
for classes \texttt{formula} and \texttt{lm}, that replace 
the input arguments by:
\begin{description}
  \item[formula] a  standard linear model formula  $y \sim
x1 + x2 + ...$ defining the model relation between response and
predictor variables (in the regression context) or fixed and
assessed variables in the CCA context. 
  \item[data] a data frame from which variables specified in formula 
are preferentially to be taken. 
\end{description}
or
\begin{description}
  \item[fitdlmmodel] an object of class \texttt{lm}, as produced by R's
  \texttt{lm} function. 
\end{description}

In this context, the output object \texttt{mat} is 
%{\color{red} proportional to -- if mat and H are not divided by n-1}
the covariance
matrix  $T_x$ of the $x$ variables; object \texttt{H} is 
%{\color{red} proportional to?}
the covariance matrix  $H_{x\vert y}$ of the projections of the $x$
variables on the space spanned by the $y$ variables;  and $r$ is the
expected rank of the $\Hmat$ matrix which, under the assumption of
linear independence, equals the minimum between the number of
variables in the x and y sets (the true rank of $\Hmat$ can be
different if the linear independence condition fails). See the
\texttt{lmHmat} help file for more information.

\paragraph{Example.} An example of the use of the helper function
\texttt{lmHmat} involves the \texttt{iris} data set. The goal is to
study the \textbf{(univariate response) multiple linear regression} of variable
\texttt{Sepal.Length} (the first variable in the \texttt{iris} data
frame) on the three remaining predictors.  

<<>>=
lmHmat(x=iris[,2:4], y=iris[,1])
@

\paragraph{The \textbf{ldaHmat} function for linear discriminant analysis}

\textbf{ }\newline

This function takes by default the following arguments:
\begin{description}
  \item[x] is a matrix containing the discriminating
variables, from which a subset is being considered. 
  \item[grouping] is a factor specifying the class to which each
observation belongs.
\end{description}

There are S3 \emph{methods} for 
\begin{itemize}
\item class \texttt{data.frame} input argument \texttt{x};
\item input object of class \texttt{formula}.
\end{itemize}
With these methods, the input arguments for \texttt{ldaHmat} can be given as:
\begin{description}
  \item[formula] a formula of the form $grouping \sim x1 + x2 + ...$
where the $x$ variables denote the discriminating variables. 
  \item[data] a data frame from which variables specified in formula are preferentially to be taken. 
\end{description}

In this context, the output objects \texttt{mat} and \texttt{H} are
the standard total ($\T$) and between-group ($\Hmat$) SSCP matrices of
Fisher's linear discriminant analysis; and output object \texttt{r} is the
rank of the between-group matrix $\Hmat$, which equals the minimum 
between the number of discriminators and the number of groups minus
one (although the true rank of $\Hmat$ can be different, if the discriminators
are linearly dependent).

\paragraph{Examples}

A simple example of the use of function \texttt{ldaHmat} again
involves the \texttt{iris} data set. We seek to discriminate
the three iris species, using the four morphometric variables as
discriminators.

<<>>=
ldaHmat(x=iris[,1:4], grouping=iris$Species)
@
%$  

In the same example, the function could have been invoked as:

<<>>=
attach(iris)
ldaHmat(Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width)
detach(iris)
@
%$


\paragraph{The \textbf{glhHmat} function for a general linear
hypthesis context}  

\textbf{ }\newline 

The function \textbf{glhHmat} creates the appropriate SSCP matrices 
for any problem that can be described as an instance of the 
\textbf{\emph{general multivariate linear model}} $\X = \A \pmb{\Psi} +
\U$, \emph{with a \textbf{reference hypothesis}} $H_0: \C \pmb{\Psi} =
\mathbf{0}$, where $\pmb{\Psi}$ is a matrix of unknown parameters and $\C$
is a known rank-$r$ coefficient matrix. 
By default, this function takes the following arguments:
\begin{description}
  \item[x] is a matrix containing the response variables. 
  \item[A] is a design matrix specifying a linear model in which $\X$ is the response.
  \item[C] is a matrix or vector containing the coefficients of the reference hypothesis. 
\end{description}

There is an S3 \emph{method} for input of class \texttt{data.frame} 
in which x and \A~ are defined as data frames. A further method accepts 
input of class \texttt{formula}, with input arguments:
\begin{description}
  \item[formula] a formula of the form $X \sim A1 + A2 + ...+ Ap$
where the terms of the right hand side specify the relevant columns of the design matrix. 
  \item[C] a matrix or vector containing the coefficients of the reference hypothesis. 
  \item[data] a data frame from which variables specified in formula are preferentially to be taken. 
\end{description}

In this context, the $\T$ and $\Hmat$ matrix have the generic form
given at the beginning of this subsection and \texttt{r} is the
rank of $\Hmat$, which equals the rank of $\C$ (the
true rank of $\Hmat$ can be different from \texttt{r} if the $\X$
variables are linearly dependent). 

\paragraph{Example.} 

The following example creates the Total and Effects SSCP matrices, 
$\T$ and $\Hmat$, for an analysis of the data set \texttt{crabs} in  
the MASS package. This data set records physical measurements on 200
specimens of \textit{Leptograpsus variegatus} crabs observed on the
shores of  Western Australia. The crabs are classified by two factors,
both with two levels each: \texttt{sex}  and \texttt{sp} (crab species,
as defined by its colour: blue or orange). The measurement
variables include the carapace length (\texttt{CL}),  
the carapace width (\texttt{CW}), the size of the frontal lobe  
(\texttt{FL}) and the rear width (\texttt{RW}). 
We assume that there is an interest in 
comparing the subsets of these variables measured in their original  
and logarithmic scales.   In particular, we assume that it is wished 
to create the $\T$ and $\Hmat$ matrices associated with an analysis 
of the effect of the \texttt{sp} factor after controlling for 
sex. Only the \texttt{formula}, \texttt{C} and \texttt{data} arguments
are explicitly given in this function call.


<<>>=
library(MASS)
data(crabs)
lFL <- log(crabs$FL) ; lRW <- log(crabs$RW); lCL <- log(crabs$CL); lCW <- log(crabs$CW)
C <- matrix(0.,nrow=2,ncol=4)
C[1,3] = C[2,4] = 1.
C
Hmat5 <- glhHmat(cbind(FL,RW,CL,CW,lFL,lRW,lCL,lCW) ~ sp*sex,C=C,data=crabs)
Hmat5
@ 


\subsubsection{The $ccr_1^2$ (ccr12) coefficient and Roy's first root statistic}
\label{sss ccr12}

\paragraph{Motivation and definition}

\textbf{ }\newline

The $ccr_1^2$ coefficient is an increasing function of Roy's first root test
statistic for the reference hypothesis in the standard multivariate
linear model. Roy's first root is the largest eigenvalue of $\Hmat\E^{-1}$, where
$\Hmat$ is the Effect matrix and $\E$ is the Error (residual) matrix. 
The index $ccr_1^2$ is related to Roy's first root $\lambda_1$ by: 
$$ccr_1^2 =\frac{\lambda_1}{1+\lambda_1}.$$ 

\paragraph{The \texttt{ccr12.coef} function}

\textbf{ }\newline

The \texttt{subselect} package provides the function \textbf{ccr12.coef} which
computes the $ccr_1^2$ coefficient, given the variance or total SSCP matrix for the full data set, \texttt{mat}, the effects SSCP matrix \texttt{H}, the expected rank of the $\Hmat$ matrix, \texttt{r}, and a vector \texttt{indices} with the indices of the variable subset that is being considered. These arguments may be defined with the helper functions described in Subsection \ref{sss Hmat}. A standard function call looks like:

\begin{verbatim}
> ccr12.coef(mat, H, r, indices)
\end{verbatim}

For further arguments and options, see the function \texttt{help} page.


\paragraph{Example}

\textbf{ }\newline

The following example in the use of function \texttt{ccr12.coef} in
the context of a (univariate response) Multiple Linear Regression uses 
the \texttt{Cars93} data set from the MASS library. Variable 5 (average price)
is regressed on 13 other variables. The goal is to compare subsets of
these 13 variables according to their ability to predict car
prices. The helper function \texttt{lmHmat} creates the relevant input
to test the value of the $ccr_1^2$ criterion for the subset of the
fourth, fifth, tenth and eleventh predictors.

<<>>=
library(MASS)
data(Cars93)
CarsHmat <- lmHmat(x=Cars93[c(7:8,12:15,17:22,25)],y=Cars93[5])
ccr12.coef(mat=CarsHmat$mat, H=CarsHmat$H, r=CarsHmat$r, indices=c(4,5,10,11))
@
%$

\subsubsection{The $\tau^2$ (tau2) coefficient and Wilk's Lambda} 
\label{sss tau2} 

\paragraph{Motivation and definition}

\textbf{ }\newline

The Tau squared index $\tau^2$ is a decreasing function of the 
standard Wilk's Lambda statistic for the multivariate linear model and
its reference hypothesis. The Wilk's lambda statistic ($\Lambda$) is 
given by:   
$$\Lambda=\frac{det(\E)}{det(\T)},$$ 
where $\E$ is the Error (residual) SSCP matrix and $\T$ is the Total
SSCP matrix.   The index $\tau^2$ is related to the Wilk's Lambda
statistic by:  
$$\tau^2 = 1 - \Lambda^{1/r},$$ 
where $r$ is the rank of  the Effect SSCP matrix $\Hmat$.

\paragraph{The \texttt{tau2.coef} function}

\textbf{ }\newline

Function \textbf{tau2.coef} is similar to its counterpart \texttt{ccr12.coef} described in the previous Subsection. 
A standard function call looks like:

\begin{verbatim}
> tau2.coef(mat, H, r, indices)
\end{verbatim}


\paragraph{Example}

\textbf{ }\newline

A very simple example of the use of the $\tau^2$ criterion with the 
Linear Discriminant Analysis example for the \emph{iris} data set, 
using all four morphometric variables to discriminate the three species. 
The subset consisting of variables 1 and 3 is then considered as a surrogate for all four variables.
 
<<>>=
irisHmat <- ldaHmat(iris[1:4],iris$Species)
tau2.coef(irisHmat$mat,H=irisHmat$H,r=irisHmat$r,c(1,3))
@ 

\subsubsection{The $\xi^2$ (xi2) coefficient and the Bartlett-Pillai statistic}
\label{sss xi2}

\paragraph{Motivation and definition}

\textbf{ }\newline

The Xi squared index is an increasing function of the traditional
Bartllet-Pillai trace test statistic. The Bartlett-Pillai trace $P$ is
given by:  
$P=tr(\Hmat\T^{-1})$ where $\Hmat$ is the Effects SSCP matrix and $\T$
is the Total SSCP matrix.  The Xi squared index $\xi^2$  is related 
to the Bartllet-Pillai trace by: 
$$\xi^2 =\frac{P}{r},$$ 
where $r$ is the rank of $\Hmat$.
 
\paragraph{The \texttt{xi2.coef} function}

\textbf{ }\newline

Function \textbf{xi2.coef} is similar to the previous criterion functions described in this Section. 
A standard function call looks like:

\begin{verbatim}
> xi2.coef(mat, H, r, indices)
\end{verbatim}



\paragraph{Example}

\textbf{ }\newline

The same example considered in Subsection \ref{sss tau2}, only 
this time using the $\xi^2$ index of $\tau^2$.

<<>>=
irisHmat <- ldaHmat(iris[1:4],iris$Species)
xi2.coef(irisHmat$mat,H=irisHmat$H,r=irisHmat$r,c(1,3))
@

\subsubsection{The $\zeta^2$ (zeta2) coefficient and the Lawley-Hotelling statistic}
\label{sss zeta2} 

\paragraph{Motivation and definition}

\textbf{ }\newline

The Zeta squared index $\zeta^2$ is an increasing function of the 
traditional Lawley-Hotelling trace test statistic. The 
Lawley-Hotelling  trace is given by $V=tr(\Hmat\E^{-1})$ where $\Hmat$ 
is the Effect SSCP matrix and $\E$ is the Error SSCP matrix.  
 The index $\zeta^2$ is related to the Lawley-Hotelling  trace by:   
$$\zeta^2 =\frac{V}{V+r} ,$$ 
where $r$ is the rank of $\Hmat$. 

\paragraph{The \texttt{zeta2.coef} function}

\textbf{ }\newline


Again, function \textbf{zeta2.coef} has arguments similar to those of the other related functions in this Section. 
A standard function call looks like:

\begin{verbatim}
> zeta2.coef(mat, H, r, indices)
\end{verbatim}



\paragraph{Example}

\textbf{ }\newline

Again, the same example as in the previous two subsections:

<<>>=
irisHmat <- ldaHmat(iris[1:4],iris$Species)
zeta2.coef(irisHmat$mat,H=irisHmat$H,r=irisHmat$r,c(1,3))
@  


\subsection{Criterion for generalized linear models}  
\label{ss glm}  

\paragraph{Motivation and definition}

\textbf{ }\newline

Variable selection in the context of generalized linear models is
typically based on the minimization of statistics that test the
significance of the excluded variables.  In particular, the likelihood
ratio, Wald and Rao statistics, or some monotone function of those
statistics, are often proposed as comparison criteria for variable
subsets of the same dimensionality. All these statistics are
asympotically equivalent and, given suitable assumptions,
can be converted into information criteria, such as the AIC, which 
also compare  subsets of different dimensionalities 
(see references \cite{Lawless:78} and \cite{Lawless:87}  for further details). 

Among these criteria, Wald's statistic has some computational
advantages because it can always be derived from the same  maximum
likelihood and Fisher  information estimates (concerning the full
model). In particular, let $W_{all}$ be the value of the Wald statistic
testing the significance of the full covariate vector, $\bvec$ and
$\F$ be the coefficient and Fisher   information estimates,
and $\Hmat$ be an auxiliary rank-one matrix given by $\Hmat =  \F
\bvec \bvec^t \F$. It follows that the value of Wald's statistic for
the   excluded variables ($W_{exc}$) in a given subset  is given by  
$W_{exc} = W_{all} - tr (\F_{indices}^{-1} \Hmat_{indices})$, where 
$\F_{indices}$ and $\Hmat_{indices}$ are the portions of the $\F$ and
$\Hmat$ matrices associated with the selected variables. 

The \texttt{subselect} package provides a function \texttt{wald.coef}
that computes the value of Wald's statistic, testing the significance
of the excluded  variables, in the context of variable subset
selection in generalized linear models (see Subsection \ref{sss wald} for more details). 

\subsubsection{A helper function for the GLM context} 
\label{sss helper GLM} 

As with the multivariate linear model context, creating the Fisher
Information matrix and the auxiliary matrix $\Hmat$ described above
may be time-consuming. The \texttt{subselect} package provides the
helper function  \textbf{glmHmat} which accepts  a glm object
(\texttt{fitdglmmodel}) to retrieve an estimate of  Fisher's Information.
($\F$) matrix together with an auxiliary rank-one positive-definite
matrix ($\Hmat$), such that the positive eigenvalue of $\F^{-1}\Hmat$
equals the value of Wald's statistic  for testing the global
significance of the fitted model. These matrices may be used as input
to the function that computes the Wald criterion, as well as to the
variable selection search routines described in Section \ref{s
algoritmos}. 

As an example of the use of this helper function, in the context of
binary response  logistic regression models, consider the last 100
observations of the \emph{iris} data set (retaining only observations
for the \emph{versicolor} and \emph{virginica} species). Assume that
the goal is to judge subsets of the four morphometric variables (petal
and sepal lengths and widths), in models with the binary response
variable given by an indicator variable of the two remaining species.  
The helper function \texttt{glmHmat} will produce the Fisher Information  
matrix (output object \texttt{mat}), the auxiliary $\Hmat$ function
(output object \texttt{H}), the function \texttt{call}, and set the 
rank of output object \texttt{H} to $1$,  as follows: 

<<>>=
iris2sp <- iris[iris$Species != "setosa",]
modelfit <- glm(Species ~ Sepal.Length + Sepal.Width + Petal.Length +
Petal.Width, data=iris2sp, family=binomial)  
Hmat <- glmHmat(modelfit)
Hmat
@ %$



\subsubsection{Function \texttt{wald.coef} and the Wald coefficient}    
\label{sss wald}

The function \textbf{wald.coef} computes the value ($W_{exc}$)    
of the Wald statistic, as described above. 
The function takes as input arguments:
\begin{description} 
  \item[mat] An estimate of Fisher's information matrix $\F$ for the  
  full model variable-coefficient estimates; 
  \item[H] A matrix product of the form $\Hmat =  \F \bvec \bvec^t \F$ 
  where $\bvec$ is a vector of variable-coefficient estimates; 
   \item[indices] a numerical vector, matrix or 3-d array of integers 
  giving the indices of the variables in the subset. If a matrix is 
  specified, each row is taken to represent a different 
  \emph{k}-variable subset. If a 3-d array is given, it is assumed 
  that the third dimension corresponds to different cardinalities. 
  \item[tolval \texttt{and} tolsym] are parameters used in checks for  
   ill-conditioning and positive-definiteness of the Fisher Information and 
   the auxiliary ($\Hmat$) matrices (see the \texttt{wald.coef} help file for further details).  
\end{description} 

The values of arguments \texttt{mat} and \texttt{H} can be created  
using the \texttt{glmHmat} helper function, described in subsection \ref{sss helper GLM}.   


\paragraph{Example}

\textbf{ }\newline

An example of variable selection in the context of binary response 
regression models can be given using the same \texttt{crabs} data set  
from the MASS package that was already discussed in subsection \ref{sss Hmat}.  
The logarithms and original physical measurements 
of the \textit{Leptograpsus variegatus} crabs considered in the MASS crabs   
data set are used to fit a logistic model where each crab's sex forms 
the response variable. The quality of the variable subset made up by the  
first, sixth and seventh predictor variables is measured via the Wald coefficent.   

<<>>= 
library(MASS) 
lFL <- log(crabs$FL)
lRW <- log(crabs$RW)
lCL <- log(crabs$CL)
lCW <- log(crabs$CW)
logrfit <- glm(sex ~ FL + RW + CL + CW  + lFL + lRW + lCL + lCW,data=crabs,family=binomial)
lHmat <- glmHmat(logrfit) 
wald.coef(lHmat$mat,lHmat$H,indices=c(1,6,7),tolsym=1E-06)  
@  

 
It should be stressed that, contrary to the criteria considered in the     
previous problems, $W_{exc}$ is not bounded  above by 1 and $W_{exc}$   
is a \textit{decreasing} function of subset quality.    

\section{Search algorithms}
\label{s algoritmos}

Given any data set for which the variable selection problem is 
relevant and a criterion that measures how well any 
given variable subset approximates the full data set, the 
problem of \emph{finding the best $k$-variable subsets for that     
criterion} arises naturally. Such problems are computationally 
intensive for the criteria considered in this package. 
A complete search, among all $k$-variable subsets is a task which  
quickly becomes unfeasible even for moderately-sized data 
sets unless $k$ is very small, or very large, when compared with $p$.  

It may happen that
a criterion has special properties which render enumeration
methods possible for some moderate-sized data sets. Furnival and
Wilson's \cite{Furnival:74} Leaps and bounds algorithm did this in the context
of subset selection in Linear Regression, and package co-author Duarte Silva 
(see references \cite{Pedro:01} and \cite{Pedro:02})
has discussed the application of this algorithm to various
methods of Multivariate Statistics. The package function \textbf{eleaps} implements 
these algorithms for the criteria discussed above, using C++ code 
(see Section \ref{ss eleaps}). This function is very fast for small or 
moderately sized datasets (with roughly $p<30$ variables). For larger data sets
(roughly $p>35$ variables) it becomes computationally unfeasible and alternative 
search heuristics become necessary.

Traditional heuristics for the variable selection problem in the context
of linear regression, such as the backward elimination, forward selection or 
stepwise selection algorithms belong to a general class of algorithms called 
\emph{greedy}. Their limitations are well-known. The search algorithms considered 
in this package do not fall in this category. They are of three types: a 
simulated annealing algorithm (implemented by
the \textbf{anneal} function and discussed in section \ref{ss anneal}); a 
genetic algorithm (function \textbf{genetic}, discussed in section \ref{ss genetic}) 
and a modified local search algorithm (function \textbf{improve}, discussed in 
section \ref{ss improve}). These algorithms were discussed in \cite{CCM:04}. 
The three corresponding package functions use Fortran code for computational efficiency.


\subsection{Common input arguments}
\label{ss input}

The four search functions described above share several input arguments. These are:

\begin{description}
  \item[mat] a covariance/correlation, information or sums of squares and products
    matrix of the variables from which the k-subset is to be selected (equivalent 
    to the homonymous argument in the criteria functions discussed in 
    Section \ref{s criteria}). 
  \item[kmin] the cardinality of the smallest subset that is sought.
  \item[kmax] the cardinality of the largest subset that is sought.
  \item[exclude] a vector of variables (referenced by their row/column
  numbers in matrix \texttt{mat}) that are to be forcibly excluded from
  the subsets considered.
  \item[include] a vector of variables (referenced by their row/column
  numbers in matrix \texttt{mat}) that are to be forcibly included in
  the subsets considered.
  \item[criterion] A character variable, indicating which criterion
  is to be used in judging the quality of the subsets. These are discussed
  in Section \ref{s criteria}. The default
  criterion is \texttt{rm} for exploratory and PCA analysis, \texttt{tau2} in
  the multivariate linear model context, and \texttt{Wald} for generalized linear models.
  \item[pcindices] (for the GCD criterion only) is a vector of ranks of 
  Principal Components that are to be used for comparison with the 
  k-variable subsets. The default value is the character string
  \texttt{first\_k}, which associates PCs 1 to \emph{k} to each
  cardinality \emph{k}  requested by the user.
  \item[H] Effect description matrix. Not used with the RM, RV or GCD
    criteria, hence the NULL default value.
  \item[r] Rank of the effects  matrix, $\Hmat$. Not used with the
    RM, RV or GCD criteria. 
  \item[tolval] a parameter to fine-tune the approach to ill-conditioned 
    \texttt{mat} arguments (see the search function help files for more details).
  \item[tolsym] a parameter to fine-tune the approach to non-symmetric 
    \texttt{mat} and $\Hmat$ arguments (see the search function help files for more details).
\end{description}

Each search function has additional specific input arguments that are discussed 
in the subsequent sections and in the appropriate help files.


\subsection{Common output objects}
\label{ss output}

The four search functions have a \textit{common output structure}: a \texttt{list} with 
five components:

\begin{description}
   \item[subsets] An $m\times kmax \times length(kmin:kmax)$ 3-dimensional 
   array, where $m$ is the number of different solutions that are to be 
   produced for each cardinality (see the subsequent subsections for details). 
   For each cardinality (dimension 3) and each solution
   (dimension 1) the list of variables (referenced by their row/column
   numbers in matrix \texttt{mat}) in the subset is given (dimension 2). 
    For cardinalities  smaller than \texttt{kmax}, the extra final positions are set
   to zero.
%{\color{red}MUDAR PARA NA?}). 
   These output objects can be directly 
   passed to the \texttt{*.coef} functions discussed in Section \ref{s criteria}, 
   in order to compute the performance of these subsets with criteria other than 
   the one used to select them; 
   \item[values] An $m\times  length(kmin:kmax)$ matrix, giving for each cardinality 
   (columns), the criterion values  of the best $m$ (rows) subsets selected according to 
   the chosen criterion;
   \item[bestvalues] A $length(kmin:kmax)$ vector giving
   the overall best values of the criterion for each cardinality;
   \item[bestsets] A $length(kmin:kmax) \times kmax$ matrix, giving, for each 
   cardinality (rows), the variables (referenced by their row/column numbers in 
   matrix \texttt{mat}) in the best $k$-variable subset (can also be fed to the 
   \texttt{*.coef} functions).
   \item[call] The function call which generated the output.
\end{description}

%This output is of \textbf{class} \emph{subselect} (which inherits from class \texttt{list}). 
%Methods for this class are discussed in Section \ref{s methods}.



\subsection{The \texttt{eleaps} function: an efficient complete search}
\label{ss eleaps}

For each cardinality $k$ (with $k$ ranging from \texttt{kmin} to \texttt{kmax}),
% \texttt{eleaps} performs a branch and bound search for the best \texttt{nsol}-variable 
\texttt{eleaps} ("Extended Leaps and Bounds") performs a branch and bound search for 
the best \texttt{nsol}-variable 
subsets (\texttt{nsol} being a user-specified function argument), according to a 
specified criterion. The function \texttt{eleaps} implements Duarte Silva's 
adaptation for various multivariate analysis contexts (references 
\cite{Pedro:01} and \cite{Pedro:02}) of Furnival and 
Wilson's Leaps and Bounds Algorithm (reference \cite{Furnival:74}) for 
variable selection in Regression Analysis. If the search is not completed within 
a user defined time limit (input argument \texttt{timelimit}), \texttt{eleaps} 
exits with a warning message, returning the best (but not necessarly optimal)
solutions found in the partial search performed.

In order to improve computation times, the bulk of computations are
carried out by C++ routines. Further details about the Algorithm can 
be found in references \cite{Pedro:01} and \cite{Pedro:02} and in the 
comments  to the C++ code (in the package's \texttt{src} directory). 
A discussion of the criteria considered can be found in  
Section \ref{s criteria} above. The function checks for ill-conditioning of the 
input matrix (see the \texttt{eleaps} help file for details).

\paragraph{Examples}

\textbf{ }\newline

For illustration of use, we now consider five examples.

\begin{description}
\item[Example 1] deals with a small data set provided in the standard distributions 
of R. The \texttt{swiss} data set is a 6-variable data set with  Swiss 
fertility and socioeconomic indicators (1888). Subsets of variables of all cardinalities 
are sought, since the \texttt{eleaps} function sets, by default,
\texttt{kmin} to 1 and \texttt{kmax} to one less than the number of columns in the 
input argument \texttt{mat}. The function call requests the best three
subsets of each cardinality, using the RM criterion (subsection \ref{sss RM}). 

<<>>=
data(swiss)
eleaps(cor(swiss),nsol=3, criterion="RM")
@ 

In this example, it is not necessary to explicitly specify the criterion RM, 
since it is the default criterion for any function call which does not explicitly set 
the \texttt{r} input argument (except in a GLM context - see Section \ref{ss glm}).

\item[Example 2] illustrates the use of the \texttt{include} and \texttt{exclude} 
arguments that are common to all the search functions provided by the package \texttt{subselect}. 
Here, we request only 2- and 3- dimensional subsets that exclude
variable number 6 and include variable number 1. 
For each cardinality, three solutions are requested (argument \texttt{nsol}). 
The criterion requested is the GCD (see subsection \ref{sss GCD}) and the 
subspace used to gauge our solutions is the principal 
subspace spanned by the first three principal components of the full data set 
(argument \texttt{pcindices}). Our solutions will be the 2- and 3-variable subsets that 
span subspaces that are closest to this 3-d principal subspace.
 
<<>>=
data(swiss)
swiss.gcd <- eleaps(cor(swiss),kmin=2,kmax=3,exclude=6,include=1,nsol=3,criterion="gcd",pcindices=1:3)
swiss.gcd
@ 

\emph{The output of this function call can be used as input for a function call requesting 
the values of the chosen solutions under a different criterion}. For example, the 
values of the RM coefficient for the above solutions are given by:

<<>>=
rm.coef(mat=cor(swiss), indices=swiss.gcd$subsets)
@ 


\item[Example 3] involves a \textbf{Linear Discriminant Analysis} example with a very 
small data set, which has already been discussed in subsection \ref{sss tau2}.
We consider the Iris data and three groups, defined by species (setosa, 
versicolor and virginica). The goal is to select the 2- and 3-variable
subsets that are optimal for the linear discrimination (as measured 
by the \texttt{ccr12} criterion, \emph{i.e.}, by Roy's first root statistic).
 
<<>>=
irisHmat <- ldaHmat(iris[1:4],iris$Species)
eleaps(irisHmat$mat,kmin=2,kmax=3,H=irisHmat$H,r=irisHmat$r,crit="ccr12")
@ 

\item[Example 4] \label{ex Cars eleaps}
involves the MASS package \texttt{Cars93} data set
that was  already discussed in subsection \ref{sss ccr12}. The context
here is a \textbf{Canonical
Correlation Analysis}. Two groups of variables within the Cars93
data set are compared: the $X$ group (variables 7,8, 12 to 15, 17 to 22 and 25) 
and the $Y$ group (variables 4 and 6). The latter variables are respectively the
car prices for a basic (Min.Price) and premium (Max.Price) version, of the car models. 
The goal is to select 4- to 6-variable subsets of the 13-variable $\X$ group that 
are optimal in terms of preserving the canonical correlations, according to the
\texttt{zeta2} criterion (Warning: the $\Y$ group with the 2-price variables  is kept 
intact; subset selection is carried out in the $\X$ group only).
The \texttt{tolsym} parameter is used to relax the symmetry
requirements on the effect matrix $\Hmat$ which, for numerical reasons,
is slightly asymmetric. Since corresponding off-diagonal entries of
matrix $\Hmat$ are different, but by less than \texttt{tolsym}, $\Hmat$ is 
replaced  by its symmetric part: $(\Hmat+\Hmat^t)/2$.

<<>>=
library(MASS)
data(Cars93)
Cars93.xgroup <- Cars93[,c(7:8,12:15,17:22,25)]
CarsHmat <- lmHmat(Cars93.xgroup,Cars93[,c(4,6)])
colnames(Cars93[,c(4,6)])
colnames(Cars93.xgroup)
#colnames(CarsHmat$mat)
Cars.eleaps <- eleaps(CarsHmat$mat, kmin=4, kmax=6, H=CarsHmat$H, r=CarsHmat$r, crit="zeta2", tolsym=1e-9)
Cars.eleaps$bestvalues
Cars.eleaps$bestsets
@ 
%$

\item[Example 5.] A final example involves the use of the \texttt{eleaps} function for variable   
selection in the  context of a \textbf{generalized linear model}, more precisely, of a logistic  
regression model. We consider the last 100 observations of 
the \emph{iris} data set (versicolor and virginica species) and seek the best variable  
subsets for the model with the indicator variable for the other two species as  
the binary response variable.  

<<>>=
iris2sp <- iris[iris$Species != "setosa",]
logrfit <- glm(Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width,iris2sp,family=binomial)
Hmat <- glmHmat(logrfit)
eleaps(Hmat$mat, H=Hmat$H, r=Hmat$r, criterion="Wald", nsol=3)
@ 

It should be stressed that, unlike other criteria in the
\texttt{subselect} package, the Wald criterion is not bounded above by
1 and is a \emph{decreasing} function of subset quality, so that the
3-variable subsets do, in fact, perform better than their smaller-sized
counterparts.


\end{description}



\subsection{The \texttt{anneal} function: a simulated annealing algorithm}
\label{ss anneal}

Given a full data set, the \textbf{anneal} function uses a Simulated Annealing
algorithm to seek a $k$-variable subset which is
optimal, as a surrogate for the full set, with respect to a given criterion.  
The algorithm is described in detail in \cite{CCM:04}.

In brief, for each of the  solutions requested by the user (via the
\texttt{nsol} function argument), 
an initial $k$-variable subset (for k ranging from
\texttt{kmin} to \texttt{kmax}) of a full set of $p$  variables is
randomly selected and passed on to a Simulated Annealing algorithm. 
The algorithm then selects a random subset in the neighbourhood of the
current subset (neighbourhood of a subset S being defined as the
family of all $k$-variable subsets which differ from S by a
single variable), and decides whether to replace the current subset 
according to the Simulated Annealing rule, i.e., either (i) always,
if the alternative subset's criterion value is larger; or (ii)
with probability $\exp^{\frac{ac-cc}{t}}$ if the alternative subset's
criterion value ($ac$) is smaller than that of the current
solution ($cc$), where $t$ (the temperature parameter) decreases
throughout the iterations of the algorithm. For each cardinality $k$,
the stopping criterion for the algorithm is the number of iterations,  
which is controlled by the user (function argument \texttt{niter}). 
Also controlled by the user are the 
initial temperature (argument \texttt{temp}), the rate of geometric
cooling of the temperature (argument \texttt{cooling}) and the  
frequency with which the temperature is cooled, as measured by 
argument \texttt{coolfreq}, the number of iterations after which the
temperature is multiplied by 1-\texttt{cooling}.

Optionally, the best $k$-variable subset produced by the simulated
annealing algorithm may be used as input in a restricted local search
algorithm, for possible further improvement (this option is controlled
by the logical argument \texttt{improvement}  which, by default, is
TRUE). The user may force variables to be included and/or  excluded
from the $k$-variable subsets (arguments \texttt{include} and
\texttt{exclude}), and may specify initial solutions
(argument  \texttt{initialsol}). 

\paragraph{Computational effort}

\textbf{ } \newline
For each cardinality $k$, the total
number of calls to the procedure which computes the criterion values
is $nsol \times niter + 1$. These calls are the dominant computational
effort in each iteration of the algorithm.   In order to improve
computation times, the bulk of computations is carried out by a
Fortran routine. Further details about the Simulated Annealing
algorithm can  be found in reference \cite{CCM:04} and in the comments
to the Fortran code (in the \texttt{src} subdirectory for this
package).  


\paragraph{The \texttt{force} argument}

\textbf{ } \newline
For datasets with a very large number of variables
(currently $p$ > 400), it is  necessary to set the \texttt{force}
argument to TRUE for the function to run, but this may cause a session
crash if there is not enough memory available. The function checks for
ill-conditioning of the input matrix (see details  in the
\texttt{anneal} help file). 


\paragraph{Reproducible solutions}

\textbf{ } \newline
The \texttt{anneal} algorithm is a random algorithm, so that solutions
are, in general,  different each time the algorithm is run. For
reproducible solutions, the logical  argument \texttt{setseed} should
be set to TRUE during a session and left as TRUE in subsequent
function   calls where it is wished to reproduce the results. 


\paragraph{Uniqueness of solutions}

\textbf{ } \newline
The requested \texttt{nsol} solutions are \emph{not necessarily different
solutions}. The \texttt{nsol} solutions are computed separately, so
that (unlike what happens with the \texttt{eleaps} function), the same 
variable subset may appear more than once among the \texttt{nsol}
solutions. 

\paragraph{Examples}

\textbf{ } \newline
Four \textbf{examples} of usage of the \texttt{anneal} function are now given. 

\begin{description}
\item[Example 1] is a very simple example, with a small data set with very few iterations
of the algorithm, using the RM criterion (although for a data set of this size it is 
best to use the \texttt{eleaps} complete search, described in section \ref{ss eleaps}).


<<>>=
data(swiss)
anneal(cor(swiss),kmin=2,kmax=3,nsol=4,niter=10,criterion="RM")
@ 

\item[Example 2] uses the 62-variable farm data set, included in this package (see \texttt{help(farm)} for details).

<<>>=
data(farm)
anneal(cor(farm), kmin=6, nsol=5, criterion="rv")
@ 

Since the \texttt{kmax} argument was not specified, the \texttt{anneal} function by default 
assigns it the same value as \texttt{kmin}. Notice that there may be repeated subsets among 
the 5 solutions produced by the function call.

\item[Example 3] \label{ex CysticFibrosis anneal}
involves subset selection in the context of a
\textbf{(univariate)  Multiple Linear Regression}. The data set \texttt{cystfibr},
included in the ISwR package, contains lung function data for cystic fibrosis
patients (7-23 years old). The data consists of 25 observations on 10 variables.
The objective is to predict the variable \texttt{pemax} (maximum expiratory pressure) from
relevant patient characteristics.  A best subset of linear predictors is sought, 
using the \texttt{tau2} criterion  which, in the case of a univariate linear regression, is
just  the standard Coefficient of Determination, $R^2$. 

<<>>=
library(ISwR)
cystfibrHmat <- lmHmat(pemax ~ age+sex+height+weight+bmp+fev1+rv+frc+tlc, data=cystfibr) 
colnames(cystfibrHmat$mat)
cystfibr.tau2 <- anneal(cystfibrHmat$mat, kmin=4, kmax=6, H=cystfibrHmat$H, r=cystfibrHmat$r, crit="tau2")  
cystfibr.tau2$bestvalues  
cystfibr.tau2$bestsets  
@
 

The algorithm underlying the \texttt{anneal} function is a random algorithm, whose 
solutions are not necessarily reproducible. It may happen that the solutions for different cardinalities 
are not nested. This illustrates that the algorithm can produce different results 
from the standard greedy algorithms, such as forward selection or backward elimination.
 
That the value of the $\tau^2$ coefficient in the context of a     
univariate linear regression is the coefficient of determination can   
be confirmed via R's standard \texttt{lm} function:  

<<>>= 
summary(lm(pemax ~ weight+bmp+fev1+rv, data=cystfibr))$r.squared
@ 

The other three  multivariate linear hypothesis
criteria discussed in Section \ref{ss linear model criteria} also give
the value of $R^2$ in a univariate multiple regression, as is
illustrated by the following command, which requests the
value of the $\xi^2$ criterion for the above solutions.

<<>>=
xi2.coef(mat=cystfibrHmat$mat, indices=cystfibr.tau2$bestsets,  H=cystfibrHmat$H, r=cystfibrHmat$r)
@ 
%$

\item[Example 4] considers variable selection in the context of a 
\textbf{logistic regression} model. We consider the last 100 
observations of the \emph{iris} data set (that is, the observations 
for the versicolor and virginica species) and seek the best 1- to 
3-variable subsets for the logistic regression that uses
the four morphometric variables to model the probability of each of 
these two species. 

<<>>= 
data(iris)
iris2sp <- iris[iris$Species != "setosa",]
logrfit <- glm(Species ~ Sepal.Length + Sepal.Width + Petal.Length + Petal.Width,iris2sp,family=binomial)
Hmat <- glmHmat(logrfit)
iris2p.Wald <- anneal(Hmat$mat,1,3,H=Hmat$H,r=1,nsol=5,criterion="Wald")
iris2p.Wald$bestsets
iris2p.Wald$bestvalues
@
\end{description}
%$


\subsection{The \texttt{genetic} function: a genetic algorithm}
\label{ss genetic}


Given a full data set,  a Genetic Algorithm
algorithm seeks a k-variable subset which is
optimal, as a surrogate for the full set, with respect to a given criterion.  
The algorithm is described in detail in \cite{CCM:04}.

In brief, for each cardinality $k$ (with $k$ ranging from the function
arguments \texttt{kmin} to \texttt{kmax}),
an initial population of  $k$-variable subsets is randomly
selected from a full set of $p$  variables. The size of this initial
population is specified by the function argument \texttt{popsize}
(\texttt{popsize}=100 by default). 
In each iteration, \texttt{popsize}/2 couples
are formed from among the population and each couple generates a child
(a new $k$-variable subset)
which inherits properties of its parents (specifically, it inherits
all variables common to both parents and a random selection of
variables in the symmetric difference of its parents' genetic makeup).
Each offspring may optionally undergo a mutation in the form of a
local improvement algorithm  (see subsection \ref{ss improve}),
with a user-specified probability. Whether or not mutations occur is
controlled by the logical variable \texttt{mutate} (which is FALSE by
default), and the respective probability is given by the argument 
\texttt{mutprob}.  The parents
and offspring are ranked according to their criterion value, and the
best \texttt{popsize} of these k-subsets  will make up the next
generation, which is used as the current population in the subsequent
iteration. 

The stopping rule for the algorithm is the number of generations,
which is specified by the function argument \texttt{nger}.

Optionally, the best \emph{k}-variable subset produced by the Genetic
Algorithm may be passed as input to a restricted local improvement
algorithm, for possible further improvement (see subsection \ref{ss
improve}).  

The user may force variables to be included and/or excluded from the
\emph{k}-subsets (function arguments \texttt{include} and \texttt{exclude}), 
and may specify an initial population (function argument \texttt{initialpop}).


The function checks for ill-conditioning of the input matrix (see the
\texttt{genetic} help file for details).


\paragraph{Genetic diversity}

\textbf{ } \newline
For this algorithm to run, it needs genetic diversity in the
population (i.e., a large number of different variables in the
variable subsets that are being considered). This means that the
function will not run on data sets with a small number of variables, $p$. 
In an attempt to ensure this genetic diversity, the
function has an optional argument \texttt{maxclone}, which is an  
integer variable specifying the maximum number of identical replicates 
(clones) of individuals (variable subsets) that is acceptable in the population. 
 However, even \texttt{maxclone}=0 does not guarantee that there are
no repetitions: only the offspring  of couples are tested for
clones. If any such clones are rejected, they are replaced by a
k-variable subset chosen at random, without any further clone tests. 
%See Example \ref{exemp 1 genetic}. 

\paragraph{Computational effort}

\textbf{ } \newline
For each cardinality \emph{k}, the total number of calls to the
procedure which computes the criterion 
values is $popsize + nger \times popsize/2$. These calls are the
dominant computational effort in each iteration of the algorithm.  
In order to improve computation times, the bulk of computations are
carried out by a Fortran routine. Further details about the Genetic
Algorithm can be found in \cite{CCM:04} and in the comments to the 
Fortran code (in the \texttt{src} subdirectory for this package).  

\paragraph{The \texttt{force} argument}

\textbf{ } \newline
For datasets with a very
large number of variables (currently $p$ > 400), it is 
necessary to set the \texttt{force} argument to TRUE for the function to
run, but this may cause a session crash if there is not enough memory
available. 


\paragraph{Reproducible solutions}

\textbf{ } \newline
The \texttt{genetic} algorithm is a random algorithm, so that solutions
are, in general,  different each time the algorithm is run. For
reproducible solutions, the logical  argument \texttt{setseed} should
be set to TRUE during a session and left as TRUE in subsequent
function   calls where it is wished to reproduce the results. 

\paragraph{Uniqueness of solutions}

\textbf{ } \newline
The requested \texttt{nsol} solutions are \emph{not necessarily different
solutions}. The \texttt{nsol} solutions are computed separately, so
that (unlike what happens with the \texttt{eleaps} function), the same 
variable subset may appear more than once among the \texttt{nsol}
solutions.   


\paragraph{Examples}

\textbf{ } \newline
Two \textbf{examples} of use of the \texttt{genetic} function are now given:

\begin{description}
\item[Example 1.] \label{exemp 1 genetic} 
Consider the 62-variable Portuguese \texttt{farm} dataset, already discussed in
subsection \ref{sss RV} (page \pageref{sss RV}). We seek a 10-variable
subset which spans a subspace as close as possible to the first 10-d
principal subspace, so that criterion GCD (subsection \ref{sss GCD})
is relevant. By default, in the \texttt{genetic} function (as in the
\texttt{anneal} and \texttt{improve} function, but unlike the
\texttt{eleaps} function), the default value of \texttt{kmax} is set
equal to the value of \texttt{kmin}, and so does not need to be
specified in this case.

<<>>=
farm.gcd <- genetic(cor(farm), kmin=10, crit="gcd")
farm.gcd$bestsets
farm.gcd$bestvalues
@ 

Since the argument \texttt{popsize} was not specified, the default
population size ($100$) was used. As stated above, not all 100
solutions are necessarily different. To select only the different
solutions R's function \texttt{unique} may be used:

<<>>=
unique(farm.gcd$subsets)
@ 

If more than just a few different solutions are desired,    
it may be possible to increase the diversity of solutions by using the 
\texttt{maxclone} argument. The following example compares the number 
of different solutions obtained using the default value of 
\texttt{maxclone}=5 and a user-specified value 
\texttt{maxclone}=0. Since the \texttt{subsets} output object is a 3-d 
array, \texttt{unique} also produces a 3-d object, whose first 
dimension gives the number of different solutions. 

<<>>=
dim(unique(genetic(cor(farm), kmin=10, crit="gcd")$subsets))
dim(unique(genetic(cor(farm), kmin=10, maxclone=0, crit="gcd")$subsets))
@ 

\item[Example 2.] We consider subset selection in the context of a  
\textbf{Canonical Correlation Analysis}. The data set in this example 
is the same as in Example 1 above.  
Two groups of variables within the farm data set are compared: the 
$X$ group (all variables except the first four) and the $Y$ group 
(variables 1 to 4).  The goal is to select 4- to 6-variable 
subsets of the 58-variable $X$ group that  are optimal in terms of 
preserving the canonical correlations, according to the \texttt{zeta2} 
criterion. 
(Warning: the 4-variable $Y$ group is kept  intact; subset 
selection is carried out in the $X$  group only).  
As can be seen from the results, even a four-variable $X$ subgroup 
essentially reproduces the canonical correlations with the $Y$ group of variables
%The \texttt{tolsym} 
%parameter is used to relax the symmetry requirements on the effect 
%matrix $\Hmat$ which, for numerical reasons, is slightly 
%asymmetric. Since corresponding off-diagonal entries of matrix $\Hmat$ 
%are different, but by less than \texttt{tolsym}, $\Hmat$ is  replaced 
%by its symmetric part: $(\Hmat+\Hmat^t)/2$.  

<<>>= 
data(farm) 
farm.xgroup <- farm[,-c(1,2,3,4)]
farmHmat <- lmHmat(farm.xgroup,farm[,1:4])
colnames(farmHmat$mat)
farm.gen <- genetic(farmHmat$mat, kmin=4, kmax=6, H=farmHmat$H, r=farmHmat$r,crit="zeta2", maxclone=0, popsize=150)
farm.gen$bestvalues
farm.gen$bestsets
@ 
\end{description}

WARNING: The column numbers given by the last command are relative only to the $X$ group of variables, and so must be read off the list of variable names produced by the \texttt{colnames} command above (which are the same as those that would be printed out by the command \texttt{colnames(farm.xgroup)}). 



\subsection{The \texttt{improve} function: a restricted improvement algorithm}
\label{ss improve}

The function \textbf{improve} implements a Restricted Local
Improvement search for an optimal $k$-variable subset. Given a set of
variables, the algorithm seeks a $k$-variable subset which is
optimal, as a surrogate for the whole set, with respect to a given
criterion.  

In brief, for each  solution requested by the user (the number of
which is set by the \texttt{nsol} function argument),
an initial $k$-variable subset (for $k$ ranging from 
argument \texttt{kmin} to argument \texttt{kmax}) of a full set of $p$
variables is 
randomly selected and the variables not belonging to this subset are
placed in a queue. The possibility of replacing a variable in the
current $k$-subset with a variable from the queue is then explored.  
More precisely, a variable is selected, removed from the queue, and the 
$k$ values of the criterion which would result from swapping this selected
variable with each variable in the current subset are computed. If the
best of these values improves the current criterion value, the current
subset is updated accordingly. In this case, the variable which leaves
the subset is added to the queue, but only if it has not previously
been in the queue (i.e., no variable can enter the queue twice). The
algorithm proceeds until the queue is emptied  (see reference
\cite{CCM:04} for more details).   

The user may force variables to be included and/or excluded (arguments
\texttt{include} and \texttt{exclude}) from the
$k$-variable subsets, and may specify initial solutions (using the
\texttt{initialsol} argument).


The function checks for ill-conditioning of the input matrix (see the
\texttt{improve} help function for more details).

\paragraph{Computational effort}

\textbf{ } \newline
For each cardinality k, the total number of calls to the procedure
which computes the criterion values is O($nsol \times k \times p$). 
These calls are the dominant computational
effort in each iteration of the algorithm.  
In order to improve computation times, the bulk of computations are
carried out in a Fortran routine. Further details about the algorithm can
be found in Reference \cite{CCM:04} and in the comments to the Fortran code (in
the \texttt{src} subdirectory for this package). 


\paragraph{The \texttt{force} argument}

\textbf{ } \newline
For datasets with a very
large number of variables (currently $p$ > 400), it is 
necessary to set the \texttt{force} argument to \texttt{TRUE} for the
function to run, but this may cause a session crash if there is not
enough memory available. 


\paragraph{Reproducible solutions}

\textbf{ } \newline
As with the \texttt{anneal} and \texttt{genetic} algorithm, so too the
\texttt{improve} algorithm is a random algorithm. Solutions 
are, in general,  different each time the algorithm is run. For
reproducible solutions, the logical  argument \texttt{setseed} should
be set to TRUE during a session and left as TRUE in subsequent
function   calls where it is wished to reproduce the results. 

\paragraph{Uniqueness of solutions}

\textbf{ } \newline
The requested \texttt{nsol} solutions are \emph{not necessarily different
solutions}. The \texttt{nsol} solutions are computed separately, so
that (unlike what happens with the \texttt{eleaps} function), the same 
variable subset may appear more than once among the \texttt{nsol}
solutions. 


\paragraph{Examples}

\textbf{ } \newline
Two \textbf{examples} of use of the \texttt{improve} function are now given:

\begin{description}
\item[Example 1.] A very simple example, using the standardized \texttt{swiss} data
set from R's general distribution (see \texttt{help(swiss)} for more
information on this data set, which was also used in the first example
of Section \ref{ss eleaps}). The purpose is to determine the best 
2-variable set, so that orthogonally projecting the 47-point (standardized) 
configuration on the
subspace spanned by those two  variables would provide a configuration
as similar as possible (allowing for rigid rotations, translation of
the origin and global changes of scale) to the original 47-point
configuration defined by all six (standardized) variables in the data set. Two
different requests are made, the first without any restrictions, while
the second forces the inclusion of variable 1, and the exclusion of
variable 6, from the selected subsets.

<<>>=
swiss.imp1 <- improve(mat=cor(swiss),kmin=2,kmax=3,nsol=4,criterion="GCD")
swiss.imp2 <- improve(cor(swiss),2,3,nsol=4,criterion="GCD",include=c(1),exclude=6)
swiss.imp1$bestvalues
swiss.imp1$bestsets
swiss.imp2$bestvalues
swiss.imp2$bestsets
@ 


\item[Example 2.] A second very simple example in a \textbf{Linear
Discriminant Analysis} setting. Here we consider the same situation 
as in the example for the discussion of the \texttt{ldaHmat} fundtion,
in subsection \ref{sss Hmat}. The goal is to select the 2- and 3-variable
subsets that are optimal for the linear discrimination of the three
\emph{iris} species in Fisher's \emph{iris} data (as measured 
by the \texttt{ccr12} criterion).

<<>>=
data(iris)
irisHmat <- ldaHmat(iris[1:4],iris$Species)
improve(irisHmat$mat,kmin=2,kmax=3,H=irisHmat$H,r=irisHmat$r,crit="ccr12")
@ 

\item[Example 3] is a more involved example, for a \textbf{MANOVA}  
context. It follows up on the example used when introducing the 
\textbf{glhHmat} function, in subsection \ref{sss Hmat}. More 
precisely, in this data set 200 crabs are classified by two factors, 
\texttt{sex}  and \texttt{sp}, with two levels each. There are also 
measurement variables, in both their original  
and logarithmic scales. The goal is to detect the variables that are 
prominent in an analysis of the effect of the \texttt{sp} factor after 
controlling for sex. the initial commands in this example were already 
discussed in subsection \ref{sss Hmat}. 


<<>>= 
library(MASS)
data(crabs)
lFL <- log(crabs$FL) ; lRW <- log(crabs$RW); lCL <- log(crabs$CL); lCW <- log(crabs$CW)
C <- matrix(0.,nrow=2,ncol=4)
C[1,3] = C[2,4] = 1.
C
Hmat5 <- glhHmat(cbind(FL,RW,CL,CW,lFL,lRW,lCL,lCW) ~ sp*sex,C=C,data=crabs)
improve(mat=Hmat5$mat, kmin=4, nsol=3, H=Hmat5$H, r=Hmat5$r, crit="xi2",tolsym=1e-06)
@ 
\end{description}


%\section{Methods}
%\label{s methods}

%{\color{red}FALTA}

\bibliographystyle{plain}
\bibliography{subbib}

\end{document}

\name{ldaHmat}
\alias{ldaHmat}
\alias{ldaHmat.default}
\alias{ldaHmat.formula}
\alias{ldaHmat.data.frame}
\title{Total and Between-Group Deviation Matrices in Linear Discriminant Analysis}
\description{Computes total and between-group matrices of Sums of Squares and Cross-Product (SSCP) deviations in linear discriminant analysis. These matrices may be used as input to the variable selection search routines \code{\link{anneal}}, \code{\link{genetic}} \code{\link{improve}} or \code{\link{eleaps}}.}  

\usage{

\method{ldaHmat}{default}(x,grouping,...)

\method{ldaHmat}{data.frame}(x,grouping,...)

\method{ldaHmat}{formula}(formula,data=NULL,...)

}
\arguments{
  \item{x}{A matrix or data frame containing the discriminators for
which the SSCP matrix is to be computed.} 
  \item{grouping}{A factor specifying the class for each observation.}
  \item{formula}{A formula of the form \code{'groups ~ x1 + x2 + ...'}
That is, the response is the grouping factor and the right hand side
specifies the (non-factor) discriminators.} 
  \item{data}{Data frame from which variables specified in 'formula'
are preferentially to be taken.}  
  \item{...}{further arguments for the method.}
}
  
\value{
A list with four items:
  \item{mat}{The total SSCP matrix}
  \item{H}{The between-groups SSCP matrix}
  \item{r}{The expected rank of the H matrix which equals the minimum
between the number of discriminators and the number of groups minus
one. The true rank of H can be different from r if the discriminators
are linearly dependent.} 
  \item{call}{The function call which generated the output.}
}
\seealso{\code{\link{anneal}}, \code{\link{genetic}}, \code{\link{improve}}, \code{\link{eleaps}}.} 
\examples{
##--------------------------------------------------------------------

## An example with a very small data set. We consider the Iris data
## and three groups, defined by species (setosa, versicolor and
## virginica). 

data(iris)
irisHmat <- ldaHmat(iris[1:4],iris$Species)
irisHmat

##$mat
##             Sepal.Length Sepal.Width Petal.Length Petal.Width
##Sepal.Length   102.168333   -6.322667     189.8730    76.92433
##Sepal.Width     -6.322667   28.306933     -49.1188   -18.12427
##Petal.Length   189.873000  -49.118800     464.3254   193.04580
##Petal.Width     76.924333  -18.124267     193.0458    86.56993

##$H
##             Sepal.Length Sepal.Width Petal.Length Petal.Width
##Sepal.Length     63.21213   -19.95267     165.2484    71.27933
##Sepal.Width     -19.95267    11.34493     -57.2396   -22.93267
##Petal.Length    165.24840   -57.23960     437.1028   186.77400
##Petal.Width      71.27933   -22.93267     186.7740    80.41333

##$r
##[1] 2

##$call
##ldaHmat.data.frame(x = iris[1:4], grouping = iris$Species)




}
\keyword{manip}

\name{glmHmat}
\alias{glmHmat}
\alias{glmHmat.glm}
\title{Input matrices for subselect search routines in generalized linear models}

\description{glmHmat uses a glm object (fitdglmmodel) to build an estimate of 
Fisher's Information (FI) matrix together with an auxiliarly rank-one positive-defenite 
matrix (H), such that the positive eigenvalue of \eqn{FI^{-1} H} equals the value of Wald's statistic 
for testing the global significance of fitdglmmodel. These matrices may be used as input to the
variable selection search routines \code{\link{anneal}}, \code{\link{genetic}} 
\code{\link{improve}} or \code{\link{eleaps}}, usign the minimization of Wald's statistic
as criterion for discarding variables. }

\details{
Variable selection in the context of generalized linear models is typically based
on the minimization of statistics that test the significance of excluded variables. 
In particular, the likelihood ratio, Wald's, Rao's and some adaptations
of such statistics, are often proposed as comparison criteria for variable subsets
of the same dimensionality. All these statistics are assympotically equivalent and
can be converted into information criteria, like the AIC, that are also able to compare 
subsets of different dimensionalities (see references [1] and [2] for further 
details).

Among these criteria, Wald's statistic has some computational advantages because it can
always be derived from the same (concerning the full model) maximum likelihood and Fisher 
information estimates. In particular, if \eqn{W_{allv}} is the value of
the Wald statistic testing the significance of the full covariate
vector, b and FI are coefficient and Fisher  
information estimates and H is an auxiliary rank-one matrix given by
\code{H =  FI \%*\% b \%*\% t(b) \%*\% FI}, it
follows that the value of Wald's statistic for the  
excluded variables (\eqn{W_excv}) in a given subset  is given by 
\eqn{W_{excv} = W_{allv} - tr (FI_{indices}^{-1} H_{indices}) ,} where
\eqn{FI_indices} and \eqn{H_indices} are the  
portions of the FI and H matrices associated with the selected variables.

glmHmat retrieves the values of the FI and H matrices from a glm object. These matrices may 
then be used as input to the search functions \code{\link{anneal}}, 
\code{\link{genetic}}, \code{\link{improve}} and \code{\link{eleaps}}.

}

\references{
[1] Lawless, J. and Singhal, K. (1978). Efficient Screening of Nonnormal 
Regression Models, \emph{Biometrics}, Vol. 34, 318-327.

[2] Lawless, J. and Singhal, K. (1987). ISMOD: An All-Subsets Regression
Program for Generalized Models I. Statistical and Computational Background, 
\emph{Computer Methods and Programs in Biomedicine}, Vol. 24, 117-124.
}

\usage{

\method{glmHmat}{glm}(fitdglmmodel,...)

}

\arguments{
  \item{fitdglmmodel}{A glm object containaing the estimates, and respective
    covariance matrix, of a generalized linear model.}
  \item{...}{further arguments for the method.} 
  }

\value{
A list with four items:
  \item{mat}{An estimate (FI) of Fisher's information matrix for the full model variable-coefficient estimates}
  \item{H}{A product of the form \code{(FI \%*\% b \%*\% t(b) \%*\% FI)} where b is a vector of variable-coefficient estimates}
  \item{r}{The rank of the H matrix. Always set to one in glmHmat.} 
  \item{call}{The function call which generated the output.}
}

\seealso{\code{\link{anneal}}, \code{\link{genetic}},
\code{\link{improve}}, \code{\link{eleaps}}, \code{\link{glm}}.} 
\examples{
##----------------------------------------------------------------

##----------------------------------------------------------------

##  An example of variable selection in the context of binary response 
##  regression models. We consider the last 100 observations of
##  the iris data set (versicolor an verginica species) and try
##  to find the best variable subsets for models that take species
##  as the response variable.

data(iris)
iris2sp <- iris[iris$Species != "setosa",]

# Create the input matrices for the  search routines in a logistic regression model 

modelfit <- glm(Species ~ Sepal.Length + Sepal.Width + Petal.Length +
Petal.Width,iris2sp,family=binomial) 
Hmat <- glmHmat(modelfit)
Hmat

## $mat
##              Sepal.Length Sepal.Width Petal.Length Petal.Width
## Sepal.Length  0.28340358 0.03263437  0.09552821  -0.01779067
## Sepal.Width   0.03263437 0.13941541  0.01086596 0.04759284
## Petal.Length  0.09552821 0.01086596  0.08847655  -0.01853044
## Petal.Width   -0.01779067 0.04759284  -0.01853044 0.03258730

## $H
##              Sepal.Length  Sepal.Width Petal.Length  Petal.Width
## Sepal.Length   0.11643732  0.013349227 -0.063924853 -0.050181400
## Sepal.Width    0.01334923  0.001530453 -0.007328813 -0.005753163
## Petal.Length  -0.06392485 -0.007328813  0.035095164  0.027549918
## Petal.Width   -0.05018140 -0.005753163  0.027549918  0.021626854

## $r
## [1] 1

## $call
## glmHmat(fitdglmmodel = modelfit)

# Search for the 3 best variable subsets of each dimensionality by an exausitve search 

eleaps(Hmat$mat,H=Hmat$H,r=1,criterion="Wald",nsol=3)

## $subsets
## , , Card.1

##            Var.1 Var.2 Var.3
## Solution 1     4     0     0
## Solution 2     1     0     0
## Solution 3     3     0     0

## , , Card.2

##           Var.1 Var.2 Var.3
## Solution 1     1     3     0
## Solution 2     3     4     0
## Solution 3     2     4     0

## , , Card.3

##            Var.1 Var.2 Var.3
## Solution 1     2     3     4
## Solution 2     1     3     4
## Solution 3     1     2     3


## $values
##              card.1   card.2   card.3
## Solution 1 4.894554 3.522885 1.060121
## Solution 2 5.147360 3.952538 2.224335
## Solution 3 5.161553 3.972410 3.522879

## $bestvalues
##   Card.1   Card.2   Card.3 
## 4.894554 3.522885 1.060121 

## $bestsets
##        Var.1 Var.2 Var.3
## Card.1     4     0     0
## Card.2     1     3     0
## Card.3     2     3     4

## $call
## eleaps(mat = Hmat$mat, nsol = 3, criterion = "Wald", H = Hmat$H, 
##    r = 1)


## It should be stressed that, unlike other criteria in the
## subselect package, the Wald criterion is not bounded above by
## 1 and is a decreasing function of subset quality, so that the
## 3-variable subsets do, in fact, perform better than their smaller-sized
## counterparts.


## > 
## > proc.time()
## [1] 0.680 0.064 0.736 0.000 0.000
}
\keyword{manip}

\name{rm.coef}

\alias{rm.coef}

\title{Computes the RM coefficient for variable subset selection} 
\description{
Computes the RM coefficient, measuring the similarity of the
spectral decompositions of a p-variable data matrix, and of the matrix which
results from regressing all the variables on a subset of only k variables. 
}

\usage{rm.coef(mat, indices)}

\arguments{
  \item{mat}{the full data set's covariance (or correlation) matrix}

  \item{indices}{a numerical vector, matrix or 3-d array of integers
  giving the indices of the variables in the subset. If a matrix is
  specified, each row is taken to represent a different
  \emph{k}-variable subset. If a 3-d array is given, it is assumed
  that the third dimension corresponds to different cardinalities.}

}

\details{
Computes the RM coefficient that measures the similarity of the
spectral decompositions of a p-variable data matrix, and of the matrix which
results from regressing those variables on a subset (given by "indices") of
the variables.  Input data is expected in the form of a (co)variance or
correlation matrix. If a non-square matrix is given, it is assumed to
be a data matrix, and its correlation matrix is used as input.

The definition of the RM coefficient is as follows:
\deqn{RM = \sqrt{\frac{\mathrm{tr}(X^t P_v X)}{\mathrm{X^t X}}} }{RM = sqrt(tr(X' Pv X)/tr(X'X))}
where \eqn{X} is the full 
(column-centered) data matrix and \eqn{P_v}{Pv} is the matrix of 
orthogonal projections on the subspace spanned by a k-variable subset.

This definition is equivalent to:
\deqn{RM = \sqrt{\frac{\sum\limits_{i=1}^{p}\lambda_i
(r)_i^2}{\sum\limits_{j=1}^{p}\lambda_j}} }{RM = sqrt(sum_i(lambda_i
r_i^2)/sum_j(lambda_j))}
where \eqn{\lambda_i}{lambda_i} stands for the \eqn{i}-th largest
eigenvalue of the covariance matrix defined by X and
\eqn{r} stands for the multiple correlation between the
\code{i}-th Principal Component and the k-variable subset.

These definitions are also equivalent to the expression used in the
code, which only requires the covariance (or correlation) matrix of
the data under consideration.

The fact that \code{indices} can be a matrix or 3-d array allows for
the computation of the RM values of subsets produced by the search
functions \code{\link{anneal}}, \code{\link{genetic}} and
\code{\link{improve}} (whose output option \code{$subsets} are
matrices or 3-d arrays), using a different criterion (see the example
below).
}

\value{
The value of the RM coefficient.
}

\references{
 Cadima, J. and Jolliffe, I.T. (2001), "Variable Selection and the
Interpretation of Principal Subspaces", \emph{Journal of Agricultural,
Biological and Environmental Statistics}, Vol. 6, 62-79.

 McCabe, G.P. (1986) "Prediction of Principal Components by Variable
Subsets", \emph{Technical Report 86-19, Department of Statistics,
Purdue University}.

 Ramsay, J.O., ten Berge, J. and Styan, G.P.H. (1984), "Matrix
Correlation", \emph{Psychometrika}, 49, 403-423. 
}

\examples{

## An example with a very small data set.

data(iris3) 
x<-iris3[,,1]
rm.coef(var(x),c(1,3))
## [1] 0.8724422

## An example computing the RMs of three subsets produced when the
## anneal function attempted to optimize the RV criterion (using an
## absurdly small number of iterations).

data(swiss)
rvresults<-anneal(cor(swiss),2,nsol=4,niter=5,criterion="Rv")
rm.coef(cor(swiss),rvresults$subsets)

##              Card.2
##Solution 1 0.7982296
##Solution 2 0.7945390
##Solution 3 0.7649296
##Solution 4 0.7623326
}

\keyword{manip}

\name{farm}
\docType{data}
\alias{farm}
\title{Sixty-two economic indicators from 99 Portuguese farms.}
\description{
  This data set is a very small subset of economic data regarding
  Portuguese farms in the mid-1990s, from Portugal's Ministry of Agriculture
     }
\usage{farm}
\format{A 99x62 matrix. The 62 columns are numeric economic indicators,
  referenced by their database code. Monetary units are in thousands of
  Escudos (Portugal's pre-Euro currency).
 \tabular{rlll}{
         Column Number \tab Column Name   \tab Units \tab Description \cr
         [,1] \tab R15 \tab 1000 Escudos \tab Total Standard Gross Margins (SGM)\cr
         [,2] \tab R24 \tab Hectares \tab Total land surface \cr
         [,3] \tab R35 \tab Hectares \tab Total cultivated surface \cr
         [,4] \tab R36 \tab Man Work Units \tab Total Man Work Units\cr
         [,5] \tab R46 \tab 1000 Escudos \tab Land Capital\cr
         [,6] \tab R59  \tab 1000 Escudos \tab Total Capital (without forests) \cr
         [,7] \tab R65  \tab 1000 Escudos \tab Total Loans and Debts \cr
         [,8] \tab R72  \tab 1000 Escudos \tab Total Investment \cr
         [,9] \tab R79  \tab 1000 Escudos \tab Subsidies for Investment \cr
         [,10] \tab R86  \tab 1000 Escudos \tab Gross Plant Product Formation\cr
         [,11] \tab R91  \tab 1000 Escudos \tab Gross Animal Product Formation\cr
         [,12] \tab R104  \tab 1000 Escudos \tab Current Subsidies\cr
         [,13] \tab R110  \tab 1000 Escudos \tab Wheat Production\cr
         [,14] \tab R111  \tab 1000 Escudos \tab Maize Production\cr
         [,15] \tab R113  \tab 1000 Escudos \tab Other Cereals (except rice) Production\cr
         [,16] \tab R114  \tab 1000 Escudos \tab Dried Legumes Production\cr
         [,17] \tab R115  \tab 1000 Escudos \tab Potato Production\cr
         [,18] \tab R116  \tab 1000 Escudos \tab Industrial horticulture and Melon Production\cr
         [,19] \tab R117  \tab 1000 Escudos \tab Open-air horticultural Production\cr
         [,20] \tab R118  \tab 1000 Escudos \tab Horticultural forcing Production\cr
         [,21] \tab R119  \tab 1000 Escudos \tab Flower Production\cr
         [,22] \tab R121  \tab 1000 Escudos \tab Sub-products Production\cr
         [,23] \tab R122  \tab 1000 Escudos \tab Fruit Production\cr
         [,24] \tab R123  \tab 1000 Escudos \tab Olive Production\cr
         [,25] \tab R124  \tab 1000 Escudos \tab Wine Production\cr
         [,26] \tab R125  \tab 1000 Escudos \tab Horses\cr
         [,27] \tab R126  \tab 1000 Escudos \tab Bovines (excluding milk)\cr
         [,28] \tab R127  \tab 1000 Escudos \tab Milk and dairy products\cr
         [,29] \tab R129  \tab 1000 Escudos \tab Sheep\cr
         [,30] \tab R132  \tab 1000 Escudos \tab Goats\cr
         [,31] \tab R135  \tab 1000 Escudos \tab Pigs\cr
         [,32] \tab R137  \tab 1000 Escudos \tab Birds\cr
         [,33] \tab R140  \tab 1000 Escudos \tab Bees\cr
         [,34] \tab R142  \tab 1000 Escudos \tab Other animals (except rabbits)\cr
         [,35] \tab R144  \tab 1000 Escudos \tab Wood production\cr
         [,36] \tab R145  \tab 1000 Escudos \tab Other forest products (except cork)\cr
         [,37] \tab R146  \tab Hectares \tab Land surface affected to cereals\cr
         [,38] \tab R151  \tab Hectares \tab Land surface affected to dry legumes\cr
         [,39] \tab R152  \tab Hectares \tab Land surface affected to potatos\cr
         [,40] \tab R158  \tab Hectares \tab Land surface affected to fruits\cr
         [,41] \tab R159  \tab Hectares \tab Land surface affected to olive trees\cr
         [,42] \tab R160  \tab Hectares \tab Land surface affected to vineyards\cr
         [,43] \tab R164  \tab Hectares \tab Fallow land surface area \cr
         [,44] \tab R166  \tab Hectares \tab Forest surface area \cr
         [,45] \tab R168  \tab Head \tab Bovines \cr
         [,46] \tab R174  \tab Head \tab Adult sheep \cr
         [,47] \tab R176  \tab Head \tab Adult goats \cr
         [,48] \tab R178  \tab Head \tab Adult pigs \cr
         [,49] \tab R209  \tab Kg/hectare \tab Maize yield \cr
         [,50] \tab R211  \tab Kg/hectare \tab Barley yield \cr
         [,51] \tab R214  \tab Kg/hectare \tab Potato yield \cr
         [,52] \tab R215  \tab L/cow/year \tab Cow milk productivity \cr
         [,53] \tab R233  \tab 1000 Escudos \tab Wages and social expenditure \cr
         [,54] \tab R237  \tab 1000 Escudos \tab Taxes and tariffs \cr
         [,55] \tab R245  \tab 1000 Escudos \tab Interest and financial costs \cr
         [,56] \tab R250  \tab 1000 Escudos \tab Total real costs \cr
         [,57] \tab R252  \tab 1000 Escudos \tab Gross Product \cr
         [,58] \tab R256  \tab 1000 Escudos \tab Gross Agricultural Product \cr
         [,59] \tab R258  \tab 1000 Escudos \tab Gross Value Added (GVA) \cr
         [,60] \tab R263  \tab 1000 Escudos \tab Final Results \cr
         [,61] \tab R270  \tab 1000 Escudos \tab Family labour income \cr
         [,62] \tab R271  \tab 1000 Escudos \tab Capital Income \cr
       }
}

\source{Obtained directly from the source.}
%\references{  }
\keyword{datasets}

\name{wald.coef}
\alias{wald.coef}

\title{Wald statistic for variable selection in generalized linear models} 
\description{
Computes the value of Wald's statistic, testing the significance of the excluded 
variables, in the context of variable subset selection in generalized linear models
}

\details{

Variable selection in the context of generalized linear models is typically based
on the minimization of statistics that test the significance of excluded variables. 
In particular, the likelihood ratio, Wald's, Rao's and some adaptations
of such statistics, are often proposed as comparison criteria for variable subsets
of the same dimensionality. All these statistics are assympotically equivalent and
can be converted into information criteria, like the AIC, that are also able to compare 
subsets of different dimensionalities (see references [1] and [2] for further 
details).

Among these criteria, Wald's statistic has some computational advantages because it can
always be derived from the same (concerning the full model) maximum likelihood and Fisher 
information estimates. In particular, if \eqn{W_{allv}} is the value of
the Wald statistic testing
the significance of the full covariate vector, b and FI are coefficient and Fisher 
information estimates and H is an auxiliary rank-one matrix given by
\code{H =  FI \%*\% b \%*\% t(b) \%*\% FI}, it follows that the value of Wald's statistic for the 
excluded variables (\eqn{W_{excv}}) in a given subset  is given by 
\eqn{W_{excv} = W_{allv} - tr (FI_{indices}^{-1} H_{indices}) ,} where
\eqn{FI_{indices}} and \eqn{H_{indices}} are the  
portions of the FI and H matrices associated with the selected variables.

The FI and H matrices can be retrieved (from a glm object) by the \code{\link{glmHmat}}
function and may be used as input to the search functions \code{\link{anneal}}, 
\code{\link{genetic}}, \code{\link{improve}} and \code{\link{eleaps}}. The Wald function
computes the value of Wald statistc from these matrices for a subset specified by \code{indices}  

The fact that \code{indices} can be a matrix or 3-d array allows for
the computation of the Wald statistic values of subsets produced by the search
functions \code{\link{anneal}}, \code{\link{genetic}}, 
\code{\link{improve}} and
\code{\link{eleaps}} (whose output option \code{$subsets} are
matrices or 3-d arrays), using a different criterion (see the example
below).

}

\usage{wald.coef(mat, H, indices,
tolval=10*.Machine$double.eps, tolsym=1000*.Machine$double.eps)} 

\arguments{
  \item{mat}{An estimate (FI) of Fisher's information matrix for the 
  full model variable-coefficient estimates}

  \item{H}{A matrix product of the form \code{FI \%*\% b \%*\% t(b) \%*\% FI} 
  where b is a vector of variable-coefficient estimates}
 
  \item{indices}{a numerical vector, matrix or 3-d array of integers
  giving the indices of the variables in the subset. If a matrix is
  specified, each row is taken to represent a different
  \emph{k}-variable subset. If a 3-d array is given, it is assumed
  that the third dimension corresponds to different cardinalities.}

  \item{tolval}{the tolerance level to be used in checks for 
   ill-conditioning and positive-definiteness of the Fisher Information and
   the auxiliar (H) matrices. Values smaller than \code{tolval} are
   considered equivalent to zero.}

 \item{tolsym}{the tolerance level for symmetry of the
   Fisher Information and the auxiliar (H) matrices. If corresponding 
   matrix entries differ by more than this
   value, the input matrices will be considered asymmetric and execution
   will be aborted. If corresponding entries are different, but by less
   than this value, the input matrix will be replaced by its symmetric
   part, i.e., input matrix A becomes (A+t(A))/2.}
}

\value{
The value of the Wald statistic.
}

\references{
[1] Lawless, J. and Singhal, K. (1978). Efficient Screening of Nonnormal 
Regression Models, \emph{Biometrics}, Vol. 34, 318-327.

[2] Lawless, J. and Singhal, K. (1987). ISMOD: An All-Subsets Regression
Program for Generalized Models I. Statistical and Computational Background, 
\emph{Computer Methods and Programs in Biomedicine}, Vol. 24, 117-124.

}

\examples{

## ---------------------------------------------------------------


##  An example of variable selection in the context of binary response
##  regression models. The logarithms and original physical measurements
##  of the "Leptograpsus variegatus crabs" considered in the MASS crabs 
##  data set are used to fit a logistic model that takes the sex of each crab
##  as the response variable.

library(MASS)
data(crabs)
lFL <- log(crabs$FL)
lRW <- log(crabs$RW)
lCL <- log(crabs$CL)
lCW <- log(crabs$CW)
logrfit <- glm(sex ~ FL + RW + CL + CW  + lFL + lRW + lCL + lCW,
crabs,family=binomial) 
## Warning message:
## fitted probabilities numerically 0 or 1 occurred in: glm.fit(x = X, y = Y, 
## weights = weights, start = start, etastart = etastart, 

lHmat <- glmHmat(logrfit) 
wald.coef(lHmat$mat,lHmat$H,c(1,6,7),tolsym=1E-06)
## [1] 2.286739
## Warning message:

## The covariance/total matrix supplied was slightly asymmetric: 
## symmetric entries differed by up to 6.57252030578093e-14.
## (less than the 'tolsym' parameter).
## It has been replaced by its symmetric part.
## in: validmat(mat, p, tolval, tolsym)


## ---------------------------------------------------------------

## 2) An example computing the value of the Wald statistic in a logistic 
##  model for five subsets produced when a probit model was originally 
##  considered

library(MASS)
data(crabs)
lFL <- log(crabs$FL)
lRW <- log(crabs$RW)
lCL <- log(crabs$CL)
lCW <- log(crabs$CW)
probfit <- glm(sex ~ FL + RW + CL + CW  + lFL + lRW + lCL + lCW,
crabs,family=binomial(link=probit)) 
## Warning message:
## fitted probabilities numerically 0 or 1 occurred in: glm.fit(x = X, y = Y, 
## weights = weights, start = start, etastart = etastart) 

pHmat <- glmHmat(probfit) 
probresults <-eleaps(pHmat$mat,kmin=3,kmax=3,nsol=5,criterion="Wald",H=pHmat$H,
r=1,tolsym=1E-10)
## Warning message:

## The covariance/total matrix supplied was slightly asymmetric: 
## symmetric entries differed by up to 3.14059889205964e-12.
## (less than the 'tolsym' parameter).
## It has been replaced by its symmetric part.
## in: validmat(mat, p, tolval, tolsym) 

logrfit <- glm(sex ~ FL + RW + CL + CW  + lFL + lRW + lCL + lCW,
crabs,family=binomial) 
## Warning message:
## fitted probabilities numerically 0 or 1 occurred in: glm.fit(x = X, y = Y, 
## weights = weights, start = start, etastart = etastart)

lHmat <- glmHmat(logrfit) 
wald.coef(lHmat$mat,H=lHmat$H,probresults$subsets,tolsym=1e-06)
##             Card.3
## Solution 1 2.286739
## Solution 2 2.595165
## Solution 3 2.585149
## Solution 4 2.669059
## Solution 5 2.690954
## Warning message:

## The covariance/total matrix supplied was slightly asymmetric: 
## symmetric entries differed by up to 6.57252030578093e-14.
## (less than the 'tolsym' parameter).
## It has been replaced by its symmetric part.
## in: validmat(mat, p, tolval, tolsym)

}

\keyword{manip}

\name{rv.coef}

\alias{rv.coef}

\title{Computes the RV-coefficient applied to the variable subset selection problem} 

\description{
Computes the RV coefficient, measuring the similarity 
 (after rotations, translations and global re-sizing) of two
configurations
of n points given by: (i) observations on each of p variables,  and 
(ii) the regression of those p observed variables on a subset  of 
the variables. 
}

\details{
  Input data is expected in the form of a (co)variance or
correlation matrix of the full data set. If a non-square matrix is
given, it is assumed to 
be a data matrix, and its correlation matrix is used as input.
The subset of variables on which the full data set will be regressed
is given by \code{indices}.

The RV-coefficient, for a (coumn-centered)
data matrix (with p variables/columns) X,
and for the regression of these columns on a k-variable subset, is
given by:
\deqn{RV = \frac{\mathrm{tr}(X X^t \cdot (P_v X)(P_v X)^t)}
{\sqrt{\mathrm{tr}((X X^t)^2) \cdot \mathrm{tr}(((P_v X) (P_v X)^t)^2)}
}}{RV = tr(X X' (PvX)(PvX)') / sqrt(tr(XX' XX') tr((PvX)(PvX)' (PvX)(PvX)'))}
where \eqn{P_v}{Pv} is the matrix of orthogonal projections on the
subspace defined by the k-variable subset.

This definition is equivalent to the expression used in the
code, which only requires the covariance (or correlation) matrix of
the data under consideration.

The fact that \code{indices} can be a matrix or 3-d array allows for
the computation of the RV values of subsets produced by the search
functions \code{\link{anneal}}, \code{\link{genetic}} and
\code{\link{improve}} (whose output option \code{$subsets} are
matrices or 3-d arrays), using a different criterion (see the example
below).
}

\usage{rv.coef(mat, indices)}

\arguments{
  \item{mat}{the full data set's covariance (or correlation) matrix}

  \item{indices}{a numerical vector, matrix or 3-d array of integers
  giving the indices of the variables in the subset. If a matrix is
  specified, each row is taken to represent a different
  \emph{k}-variable subset. If a 3-d array is given, it is assumed
  that the third dimension corresponds to different cardinalities.}
}

\value{
The value of the RV-coefficient.
}

\references{
 Robert, P. and Escoufier, Y. (1976), "A Unifying tool for linear
multivariate statistical methods: the RV-coefficient", \emph{Applied
Statistics}, Vol.25, No.3, p. 257-265.
}

\examples{

# A simple example with a trivially small data set

data(iris3) 
x<-iris3[,,1]
rv.coef(var(x),c(1,3))
## [1] 0.8659685


## An example computing the RVs of three subsets produced when the
## anneal function attempted to optimize the RM criterion (using an
## absurdly small number of iterations).

data(swiss)
rmresults<-anneal(cor(swiss),2,nsol=4,niter=5,criterion="Rm")
rv.coef(cor(swiss),rmresults$subsets)

##              Card.2
##Solution 1 0.8389669
##Solution 2 0.8663006
##Solution 3 0.8093862
##Solution 4 0.7529066
}

\keyword{manip}

\name{zeta2.coef}

\alias{zeta2.coef}

\title{Computes the Zeta squared coefficient for a multivariate linear hypothesis} 
\description{
Computes the Zeta squared  index of "effect magnitude".  
The maximization of this criterion is equivalent to the maximization of the traditional test statistic, the Lawley-Hotelling  trace.

}

\usage{zeta2.coef(mat, H, r, indices,
tolval=10*.Machine$double.eps, tolsym=1000*.Machine$double.eps)} 

\arguments{
  \item{mat}{the Variance or Total sums of squares and products matrix for the full data set.}

  \item{H}{the Effect description sums of squares and products matrix (defined in the same way as the \code{mat} matrix).}

  \item{r}{the Expected rank of the H matrix. See the \code{Details} below.}

  \item{indices}{a numerical vector, matrix or 3-d array of integers
  giving the indices of the variables in the subset. If a matrix is
  specified, each row is taken to represent a different
  \emph{k}-variable subset. If a 3-d array is given, it is assumed
  that the third dimension corresponds to different cardinalities.}

  \item{tolval}{the tolerance level to be used in checks for 
ill-conditioning and positive-definiteness of the 'total' and
'effects' (H) matrices. Values smaller than \code{tolval} are
considered equivalent to zero.}

 \item{tolsym}{the tolerance level for symmetry of the
   covariance/correlation/total matrix and for the effects (\code{H})
   matrix. If corresponding matrix entries differ by more than this
   value, the input matrices will be considered asymmetric and execution
   will be aborted. If corresponding entries are different, but by less
   than this value, the input matrix will be replaced by its symmetric
   part, i.e., input matrix A becomes (A+t(A))/2.}
}

\details{
Different kinds of statistical methodologies are considered within the
framework, of a multivariate linear model:
\deqn{X = A \Psi + U}{X = A B + U} where \eqn{X} is the (nxp) data
matrix of original variables, \eqn{A} is a known (nxp) design matrix, 
\eqn{\Psi}{B} an (qxp) matrix of unknown parameters and \eqn{U} an
(nxp) matrix of residual vectors. The \eqn{\zeta^2}{zeta2}  index is related to the traditional test statistic (Lawley-Hotelling  trace) and 
measures the contribution of each subset to an Effect characterized by
the violation of a linear hypothesis of the form 
\eqn{C \Psi = 0}{C B = 0}, where \eqn{C} is a known cofficient matrix
of rank r. The Lawley-Hotelling  trace is given by: 
\eqn{V=tr(HE^{-1})} where \eqn{H} is the Effect matrix and \eqn{E} is
the Error matrix. 
 The index \eqn{\zeta^2}{zeta2} is related to Lawley-Hotelling  trace
(\eqn{V}) by:  \deqn{\zeta^2 =\frac{V}{V+r}}{zeta^2 = V/(V+r)} where
\eqn{r} is the rank of \eqn{H} matrix.




The fact that \code{indices} can be a matrix or 3-d array allows for
the computation of the \eqn{\zeta^2} values of subsets produced by the search
functions \code{\link{anneal}}, \code{\link{genetic}}, 
\code{\link{improve}} and
\code{\link{eleaps}} (whose output option \code{$subsets} are
matrices or 3-d arrays), using a different criterion (see the example
below).
}

\value{
The value of the \eqn{\zeta^2}{zeta2} coefficient.
}

\examples{

## ---------------------------------------------------------------

## 1) A Linear Discriminant Analysis example with a very small data set. 
## We considered the Iris data and three groups, 
## defined by species (setosa, versicolor and virginica).

data(iris)
irisHmat <- ldaHmat(iris[1:4],iris$Species)
zeta2.coef(irisHmat$mat,H=irisHmat$H,r=2,c(1,3))
## [1] 0.9211501


## ---------------------------------------------------------------

## 2) An example computing the value of the zeta_2 criterion for two  
## subsets produced when the anneal function attempted to optimize  
## the ccr1_2 criterion (using an absurdly small number of iterations).

ccr1results<-anneal(irisHmat$mat,2,nsol=2,niter=2,criterion="ccr12",
H=irisHmat$H,r=2)
zeta2.coef(irisHmat$mat,H=irisHmat$H,r=2,ccr1results$subsets)

##              Card.2
##Solution 1 0.9105021
##Solution 2 0.9161813

## ---------------------------------------------------------------

}

\keyword{manip}

\name{subselect-internal}
\title{Internal subselect Objects}

\alias{initialization}
\alias{validannimp}
\alias{validation}
\alias{validgenetic}
\alias{validmat}
\alias{validnovcrit}
\alias{prepRetOtp}

\description{
  Internal subselect functions that validate input to the search functions 
(if their names start with valid). 
}
\usage{

initialization(mat, criterion, r)

validannimp(kmin, kmax, nsol, exclude, nexclude, include, ninclude, initialsol, implog)

validation(mat, kmin, kmax, exclude, include, criterion, pcindices, tolval, tolsym, 
algorithm)

validgenetic(kmin, kmax, popsize, mutprob, exclude, nexclude, include, ninclude, 
initialpop)

validmat(mat, p, tolval, tolsym, allowsingular, algorithm)

validnovcrit(mat, criterion, H, r, p, tolval, tolsym)

prepRetOtp(output, kmin, kmax, nsol, Waldcrt, Waldval, algorithm ,Numprb, Optimal, tl)

}
\details{
  These functions are not to be called by the user.
}
\keyword{internal}

\name{gcd.coef}

\alias{gcd.coef}

\title{Computes Yanai's GCD in the context of the variable-subset selection problem}

\description{
Computes Yanai's Generalized Coefficient of Determination for the
similarity of the subspaces spanned by a subset of
variables and a subset of the full data set's Principal
Components.
}

\usage{gcd.coef(mat, indices, pcindices =  NULL)}

\arguments{

  \item{mat}{the full data set's covariance (or correlation) matrix.}

  \item{indices}{a numerical vector, matrix or 3-d array of integers
  giving the indices of the variables in the subset. If a matrix is
  specified, each row is taken to represent a different
  \emph{k}-variable subset. If a 3-d array is given, it is assumed
  that the third dimension corresponds to different cardinalities.} 
 
  \item{pcindices}{a numerical vector of indices of Principal
  Components. By default, the first \emph{k} PCs are chosen, where
  \emph{k} is the cardinality of the subset of variables whose
  criterion value is being computed. \bold{If a vector of PCs is
  specified by the user, those PCs will be used for all cardinalities
  that were requested}.}  
}

\details{
Computes Yanai's Generalized Coefficient of Determination for the
similarity of the subspaces spanned by a subset of
variables (specified by \code{indices}) and a subset of the
full-data set's Principal Components (specified by \code{pcindices}).
 Input data is expected in the form of a (co)variance or
correlation matrix. If a non-square matrix is given, it is assumed to
be a data matrix, and its correlation matrix is used as input. The
number of variables (k) and of PCs (q) does not have to be the same.

Yanai's GCD is defined as:
\deqn{GCD = \frac{\mathrm{tr}(P_v\cdot P_c)}{\sqrt{k\cdot q}}}{GCD = tr(PvPc)/sqrt(k q)}
where \eqn{P_v}{Pv} and \eqn{P_c}{Pc} are the matrices of orthogonal
projections on the subspaces spanned by the k-variable subset and by
the q-Principal Component subset, respectively.

This definition is equivalent to:
\deqn{GCD = \frac{1}{\sqrt{k q}} \sum\limits_{i}(r_m)_i^2}{GCD = sum_i (r_i^2) / sqrt(k q)}
where \eqn{(r_m)_i}{r_i} stands for the multiple correlation between the
\code{i}-th Principal Component and the k-variable subset, and the sum
is carried out over the q PCs (i=1,...,q) selected.

These definitions are also equivalent to the expression used in the
code, which only requires the covariance (or correlation) matrix of
the data under consideration.

The fact that \code{indices} can be a matrix or 3-d array allows for
the computation of the GCD values of subsets produced by the search
functions \code{\link{anneal}}, \code{\link{genetic}} and
\code{\link{improve}} (whose output option \code{$subsets} are
matrices or 3-d arrays), using a different criterion (see the example
below).
}

\value{
The value of the GCD coefficient.
}

\references{
 Cadima, J. and Jolliffe, I.T. (2001), "Variable Selection and the
Interpretation of Principal Subspaces", \emph{Journal of Agricultural,
Biological and Environmental Statistics}, Vol. 6, 62-79.

 Ramsay, J.O., ten Berge, J. and Styan, G.P.H. (1984), "Matrix
Correlation", \emph{Psychometrika}, 49, 403-423. 
}

\examples{

## An example with a very small data set.

data(iris3) 
x<-iris3[,,1]
gcd.coef(cor(x),c(1,3))
## [1] 0.7666286
gcd.coef(cor(x),c(1,3),pcindices=c(1,3))
## [1] 0.584452
gcd.coef(cor(x),c(1,3),pcindices=1)
## [1] 0.6035127

## An example computing the GCDs of three subsets produced when the
## anneal function attempted to optimize the RV criterion (using an
## absurdly small number of iterations).

data(swiss)
rvresults<-anneal(cor(swiss),2,nsol=4,niter=5,criterion="Rv")
gcd.coef(cor(swiss),rvresults$subsets)

##              Card.2
##Solution 1 0.4962297
##Solution 2 0.7092591
##Solution 3 0.4748525
##Solution 4 0.4649259
}
\keyword{manip}
